var vtoy_language_data = 
[
    {
        "name":"Arabic (العربية)",        
        "FontFamily":"Courier New",
        "FontSize":20,
        "Author":"Omar Namis Mostafa",
        
        "STR_ERROR":"خطأ",
        "STR_WARNING":"تحذير",
        "STR_INFO":"معلومات",
        "STR_INCORRECT_DIR":"يرجى التشغيل من المسار الصحيح!",
        "STR_INCORRECT_TREE_DIR":"لا تقم بالتشغيل من هنا، يرجى تحميل حزمة التثبيت، وتشغيلها من هناك.",
        "STR_DEVICE":"الجهاز",
        "STR_LOCAL_VER":"في الحزمة Ventoy",
        "STR_DISK_VER":"في الجهاز Ventoy",
        "STR_STATUS":"الحالة - جاهز",
        "STR_INSTALL":"تثبيت",
        "STR_UPDATE":"تحديث",
        "STR_UPDATE_TIP":"عملية التحديث آمنة, لن يتم عمل تغييرات على ملفات الـISO.#@المتابعة؟",
        "STR_INSTALL_TIP":"سوف يتم تهيئة القرص وسيتم حذف جميع البيانات.#@المتابعة؟",
        "STR_INSTALL_TIP2":"سوف يتم تهيئة القرص وسيتم حذف جميع البيانات.#@المتابعة؟ (التحقق مرة أخرى)",
        "STR_INSTALL_SUCCESS":"مبروك!#@تم تثبيت Ventoy بنجاح على الجهاز.",
        "STR_INSTALL_FAILED":"حدث خطأ أثناء التثبيت. يمكنك إعادة توصيل الـUSB والمحاولة مرة أخرى. تفقّد /var/log/ventoy.log للتفاصيل.",
        "STR_UPDATE_SUCCESS":"مبروك!#@تم تحديث Ventoy بنجاح على الجهاز.",
        "STR_UPDATE_FAILED":"حدث خطأ أثناء التحديث. يمكنك إعادة توصيل الـUSB والمحاولة مرة أخرى. تفقّد /var/log/ventoy.log للتفاصيل.",
        "STR_WAIT_PROCESS":"هناك عملية نشطة، يرجى الانتظار...",
        "STR_MENU_OPTION":"خيارات",
        "STR_MENU_SECURE_BOOT":"تمهيد آمن",
        "STR_MENU_PART_CFG":"تكوين التقسيم",
        "STR_BTN_OK":"موافق",
        "STR_BTN_CANCEL":"إلغاء",
        "STR_PRESERVE_SPACE":"الحفاظ على بعض المساحة في الجزء السفلي من القرص",
        "STR_SPACE_VAL_INVALID":"قيمة غير صالحة للمساحة المحجوزة",
        "STR_MENU_CLEAR":"إزالة Ventoy",
        "STR_CLEAR_SUCCESS":"تمت إزالة Ventoy من الجهاز بنجاح.",
        "STR_CLEAR_FAILED":"حدث خطأ عند إزالة Ventoy من القرص. يمكنك إعادة توصيل الـUSB والمحاولة مرة أخرى. تفقد /var/log/ventoy.log للتفاصيل.",
        "STR_MENU_PART_STYLE":"أسلوب التقسيم",
        "STR_DISK_2TB_MBR_ERROR":"يرجى اختيار GPT للأقراص الأكبر من 2 تيرابايت",
        "STR_SHOW_ALL_DEV":"عرض كافة الأجهزة",
        "STR_PART_ALIGN_4KB":"محاذاة الأقسام مع 4KB",
        "STR_WEB_COMMUNICATION_ERR":"خطأ في الاتصال:",
        "STR_WEB_REMOTE_ABNORMAL":"خطأ في الاتصال: بعيد غير طبيعي",
        "STR_WEB_REQUEST_TIMEOUT":"خطأ في الاتصال: انتهت مهلة الطلب",
        "STR_WEB_SERVICE_UNAVAILABLE":"خطأ في الاتصال: الخدمة غير متوفرة",
        "STR_WEB_TOKEN_MISMATCH":"تم تحديث حالة البرنامج الخفي ، يرجى إعادة المحاولة لاحقًا.",
        "STR_WEB_SERVICE_BUSY":"الخدمة مشغولة ، يرجى إعادة المحاولة لاحقًا.",
        "STR_MENU_VTSI_CREATE":"إنشاء ملف VTSI",
        "STR_VTSI_CREATE_TIP":"هذه المرة لن يتم الكتابة على الجهاز، ولكن سيتم إنشاء ملف VTSI#@المتابعة؟",
        "STR_VTSI_CREATE_SUCCESS":"تم إنشاء ملف VTSI بنجاح!#@يمكنك استخدام Rufus(3.15+) للكتابة على الجهاز لاستكمال تثبيت Ventoy.",
        "STR_VTSI_CREATE_FAILED":"فشل إنشاء ملف VTSI.",
        "STR_MENU_PART_RESIZE":"تثبيت بدون تهيئة",
        "STR_PART_RESIZE_TIP":"سيحاول Ventoy أن يقوم بالتثبيت بدون عمل تهيئة إن أمكن. #@المتابعة؟",
        "STR_PART_RESIZE_SUCCESS":"تهانينا!#@اكتملت عملية تثبيت Ventoy بدون تهيئة بنجاح.",
        "STR_PART_RESIZE_FAILED":"فشلت عملية التثبيت بدون تهيئة، تفقد ملف /var/log/ventoy.log للتفاصيل.",
        "STR_PART_RESIZE_UNSUPPORTED":"توقفت عملية تثبيت Ventoy بدون تهيئة لعدم استيفاء بعض الشروط. تفقد ملف /var/log/ventoy.log للتفاصيل.",
        "STR_INSTALL_YES_TIP1":"تحذير: سيتم فقدان البيانات!",
        "STR_INSTALL_YES_TIP2":"يرجى إدخال YES في مربع النص في الأسفل للتأكيد بأنك تريد عمل تثبيت جديد بدلاً من التحديث.",
        "STR_PART_VENTOY_FS":"نظام الملفات لقسم Ventoy",
        "STR_PART_FS":"نظام الملفات",
        "STR_PART_CLUSTER":"حجم الكتلة",
        "STR_PART_CLUSTER_DEFAULT":"القيمة الافتراضية للنظام",
        "STR_DONATE":"يتبرع",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
      {
        "name": "Urdu (اردو)",
        "FontFamily": "Noto Nastaliq Urdu",
        "FontSize": 16,
        "Author": "Muhammad Ramzan",

        "STR_ERROR": "غلطی",
        "STR_WARNING": "انتباہ",
        "STR_INFO": "معلومات",
        "STR_INCORRECT_DIR": "براہ کرم درست ڈائریکٹری میں چلائیں!",
        "STR_INCORRECT_TREE_DIR": "یہاں نہ چلائیں، براہ کرم ریلیز پیکیج ڈاؤن لوڈ کریں اور وہاں چلائیں۔",
        "STR_DEVICE": "ڈیوائس",
        "STR_LOCAL_VER": "پیکج میں وینٹوی",
        "STR_DISK_VER": "ڈیوائس میں وینٹوی",
        "STR_STATUS": "اسٹیٹس - تیار",
        "STR_INSTALL": "انسٹال کریں",
        "STR_UPDATE": "اپڈیٹ کریں",
        "STR_UPDATE_TIP": "اپگریڈ محفوظ ہے، ISO فائلز تبدیل نہیں ہوں گی۔#@کیا آپ جاری رکھنا چاہتے ہیں؟",
        "STR_INSTALL_TIP": "ڈیوائس فارمیٹ ہو جائے گا اور تمام ڈیٹا ضائع ہو جائے گا۔#@کیا آپ جاری رکھنا چاہتے ہیں؟",
        "STR_INSTALL_TIP2": "ڈیوائس فارمیٹ ہو جائے گا اور تمام ڈیٹا ضائع ہو جائے گا۔#@کیا آپ واقعی جاری رکھنا چاہتے ہیں؟ (دوبارہ تصدیق)",
        "STR_INSTALL_SUCCESS": "مبارک ہو!#@وینٹوی کامیابی سے ڈیوائس پر انسٹال ہو گیا ہے۔",
        "STR_INSTALL_FAILED": "انسٹالیشن کے دوران غلطی پیش آئی۔ دوبارہ USB لگائیں اور کوشش کریں۔ تفصیلات کے لیے /var/log/ventoy.log چیک کریں۔ اگر مسئلہ برقرار رہے تو سرکاری ویب سائٹ کے FAQ دیکھیں۔",
        "STR_UPDATE_SUCCESS": "مبارک ہو!#@وینٹوی کامیابی سے ڈیوائس پر اپڈیٹ ہو گیا ہے۔",
        "STR_UPDATE_FAILED": "اپڈیٹ کے دوران غلطی پیش آئی۔ دوبارہ USB لگائیں اور کوشش کریں۔ تفصیلات کے لیے /var/log/ventoy.log چیک کریں۔ اگر مسئلہ برقرار رہے تو سرکاری ویب سائٹ کے FAQ دیکھیں۔",
        "STR_WAIT_PROCESS": "ایک عمل جاری ہے، براہ کرم انتظار کریں...",
        "STR_MENU_OPTION": "اختیار",
        "STR_MENU_SECURE_BOOT": "سکیور بوٹ سپورٹ",
        "STR_MENU_PART_CFG": "پارٹیشن کنفیگریشن",
        "STR_BTN_OK": "ٹھیک ہے",
        "STR_BTN_CANCEL": "منسوخ کریں",
        "STR_PRESERVE_SPACE": "ڈسک کے آخر میں کچھ جگہ محفوظ کریں",
        "STR_SPACE_VAL_INVALID": "محفوظ جگہ کے لیے غلط ویلیو",
        "STR_MENU_CLEAR": "وینٹوی ہٹائیں",
        "STR_CLEAR_SUCCESS": "وینٹوی کامیابی سے ڈیوائس سے ہٹا دیا گیا ہے۔",
        "STR_CLEAR_FAILED": "وینٹوی کو ہٹاتے وقت غلطی پیش آئی۔ دوبارہ USB لگائیں اور کوشش کریں۔ تفصیلات کے لیے /var/log/ventoy.log چیک کریں۔",
        "STR_MENU_PART_STYLE": "پارٹیشن اسٹائل",
        "STR_DISK_2TB_MBR_ERROR": "2TB سے بڑی ڈیوائس کے لیے GPT منتخب کریں",
        "STR_SHOW_ALL_DEV": "تمام ڈیوائسز دکھائیں",
        "STR_PART_ALIGN_4KB": "پارٹیشنز کو 4KB کے ساتھ سیدھ میں رکھیں",
        "STR_WEB_COMMUNICATION_ERR": "کمیونیکیشن ایرر:",
        "STR_WEB_REMOTE_ABNORMAL": "کمیونیکیشن ایرر: ریموٹ ایرر",
        "STR_WEB_REQUEST_TIMEOUT": "کمیونیکیشن ایرر: ریکویسٹ کا وقت ختم ہو گیا",
        "STR_WEB_SERVICE_UNAVAILABLE": "کمیونیکیشن ایرر: سروس دستیاب نہیں",
        "STR_WEB_TOKEN_MISMATCH": "ڈیمن اسٹیٹس اپڈیٹ ہو گیا، بعد میں دوبارہ کوشش کریں۔",
        "STR_WEB_SERVICE_BUSY": "سروس مصروف ہے، بعد میں دوبارہ کوشش کریں۔",
        "STR_MENU_VTSI_CREATE": "VTSI فائل بنائیں",
        "STR_VTSI_CREATE_TIP": "یہ عمل ڈیوائس پر کچھ نہیں لکھے گا، صرف VTSI فائل بنائے گا۔#@کیا آپ جاری رکھنا چاہتے ہیں؟",
        "STR_VTSI_CREATE_SUCCESS": "VTSI فائل کامیابی سے بن گئی!#@آپ Rufus(3.15+) کا استعمال کر کے اسے ڈیوائس پر لکھ سکتے ہیں تاکہ وینٹوی انسٹال مکمل ہو سکے۔",
        "STR_VTSI_CREATE_FAILED": "VTSI فائل بنانے میں ناکام۔",
        "STR_MENU_PART_RESIZE": "ڈیٹا محفوظ انسٹالیشن",
        "STR_PART_RESIZE_TIP": "وینٹوی ممکن ہو تو ڈیٹا محفوظ انسٹالیشن کرے گا۔#@کیا آپ جاری رکھنا چاہتے ہیں؟",
        "STR_PART_RESIZE_SUCCESS": "مبارک ہو!#@وینٹوی کی ڈیٹا محفوظ انسٹالیشن کامیابی سے مکمل ہو گئی۔",
        "STR_PART_RESIZE_FAILED": "ڈیٹا محفوظ انسٹالیشن ناکام ہو گئی، تفصیلات کے لیے /var/log/ventoy.log چیک کریں۔",
        "STR_PART_RESIZE_UNSUPPORTED": "وینٹوی نے انسٹالیشن روک دی کیونکہ کچھ شرائط پوری نہیں ہو سکیں۔ تفصیلات کے لیے /var/log/ventoy.log چیک کریں۔",
        "STR_INSTALL_YES_TIP1": "انتباہ: ڈیٹا ضائع ہو جائے گا!",
        "STR_INSTALL_YES_TIP2": "براہ کرم نیچے ٹیکسٹ باکس میں YES لکھیں تاکہ تصدیق ہو سکے کہ آپ اپگریڈ کی بجائے نیا انسٹال کرنا چاہتے ہیں۔",
        "STR_PART_VENTOY_FS": "وینٹوی پارٹیشن کے لیے فائل سسٹم",
        "STR_PART_FS": "فائل سسٹم",
        "STR_PART_CLUSTER": "کلسٹر سائز",
        "STR_PART_CLUSTER_DEFAULT": "سسٹم کا ڈیفالٹ ویلیو",
        "STR_DONATE": "عطیہ کریں",
        "STR_4KN_UNSUPPORTED": "فی الحال وینٹوی 4K نیٹو ڈیوائسز کو سپورٹ نہیں کرتا۔",
        
        "STRXXX": ""
    },
    {
        "name":"Azerbaijani (Azərbaycanca)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Xəyyam Qocayev",
        
        "STR_ERROR":"Xəta",
        "STR_WARNING":"Xəbərdarlıq",
        "STR_INFO":"Məlumat",
        "STR_INCORRECT_DIR":"Lütfən, başqa kataloqda başladın!",
        "STR_INCORRECT_TREE_DIR":"Lütfən məni burada başlatmayın, buraxılmış quraşdırıcı paketini endirin və başqa yerdə işə salın.",
        "STR_DEVICE":"Cihaz",
        "STR_LOCAL_VER":"Ventoy paketdə",
        "STR_DISK_VER":"Ventoy cihazda",
        "STR_STATUS":"Vəziyyəti - hazır",
        "STR_INSTALL":"Quraşdırın",
        "STR_UPDATE":"Yeniləyin",
        "STR_UPDATE_TIP":"Yeniləmək təhlükəsizdir, ISO faylı dəyişməz qalır, #@Davam edək?",
        "STR_INSTALL_TIP":"Disk formatlanacaq və ondakı bütün məlumatlkar itiriləcəkdir.#@Davam edək?",
        "STR_INSTALL_TIP2":"Disk formatlanacaq və ondakı bütün məlumatlkar itiriləcəkdir.#@Həqiqətən davam etmək istəyirsiniz?",
        "STR_INSTALL_SUCCESS":"Təbriklər!#@Ventoy, uğurla cihazınıza quraşdırıldı.",
        "STR_INSTALL_FAILED":"Ventoyun quraşdırılması zamanı xəta baş verdi. Cihazı təkrar qoşun və yenidən cəhd edin. Daha çox məlumat almaq üçün /var/log/ventoy.log jurnal faylına baxın.",
        "STR_UPDATE_SUCCESS":"Təbriklər!#@Ventoy, uğurla cihazınıza quraşdırıldı.",
        "STR_UPDATE_FAILED":"Ventoy yenilənən zaman xəta ba. verdi. Cihazı yenidən qoşun və yenidən cəhd edin. Daha ətraflı məlumat üçün /var/log/ventoy.log jurnal faylına baxın.",
        "STR_WAIT_PROCESS":"Proses başladıldı, lütfən gözləyin...",
        "STR_MENU_OPTION":"Seçimlər",
        "STR_MENU_SECURE_BOOT":"Secure Boot dəstəyi",
        "STR_MENU_PART_CFG":"Əlavə bölmə",
        "STR_BTN_OK":"ОLDU",
        "STR_BTN_CANCEL":"Ləğv edin",
        "STR_PRESERVE_SPACE":"Diskin sonunda əlavə bölmə yaradın",
        "STR_SPACE_VAL_INVALID":"Bölmənin ölçüsündə səhv var",
        "STR_MENU_CLEAR":"Ventoy'u silin",
        "STR_CLEAR_SUCCESS":"Ventoy, cihazdan uğurla silindi.",
        "STR_CLEAR_FAILED":"Ventoyun silinməsi zamanı xəta baş verdi. Cihazı yenidən qoşun və təkrar cəhd edin. Daha ətraflı məlumat üçün /var/log/ventoy.log jurnal faylına baxın.",
        "STR_MENU_PART_STYLE":"Diskin bölünməsi qaydası",
        "STR_DISK_2TB_MBR_ERROR":"2TB-dan artıq disklər üçün GPT seçin",
        "STR_SHOW_ALL_DEV":"Bütün cihazları göstərmək",
        "STR_PART_ALIGN_4KB":"4KB ölçüsündəki bölməni düzləndirmək",
        "STR_WEB_COMMUNICATION_ERR":"Bağlantı xətası:",
        "STR_WEB_REMOTE_ABNORMAL":"Bağlantı xətası: Uzaq bağlantı aktiv deyil",
        "STR_WEB_REQUEST_TIMEOUT":"Bağlantı xətası: Gözləmə vaxtı başa çatdı",
        "STR_WEB_SERVICE_UNAVAILABLE":"Bağlantı xətası: Xidmət əlçatmazdır",
        "STR_WEB_TOKEN_MISMATCH":"Domen vəziyyəti yeniləndi. Sonra yenidən cəhd edin.",
        "STR_WEB_SERVICE_BUSY":"Xidmət mə.ğuldur, sonra cəhd edin.",
        "STR_MENU_VTSI_CREATE":"VTSI faylı yaradın",
        "STR_VTSI_CREATE_TIP":"Bu dəfə Ventoy, cihaza quraşdırılmayacaq, yalnız VTSI faylı yaradılacaq #@Davam edək?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI faylı uğurla yaradıldı!#@Ventoy'u quraşdırmağı tamamlamaq üçün həmin faylı diskə Rufus(3.15+) istifadə edərək yaza bilərsiniz.",
        "STR_VTSI_CREATE_FAILED":"VTSI faylının yaradlmasında xəta.",
        "STR_MENU_PART_RESIZE":"Zərər verməyən quraşdırma",
        "STR_PART_RESIZE_TIP":"Ventoy mümkün olduqca zərər vermədən quraşdırmağa cəhd edir. #@Davam edək?",
        "STR_PART_RESIZE_SUCCESS":"Təbriklər!#@Zərərsiz Ventoy quraşdırması başa çatdı.",
        "STR_PART_RESIZE_FAILED":"Zərərsiz quraşdırma baş tutmadı, daha ətraflı öyrənmək üçün /var/log/ventoy.log jurnal faylına baxın.",
        "STR_PART_RESIZE_UNSUPPORTED":"ZƏrərsiz Ventoy quraşdırması dayandırıldı, belə ki, bəzi şərtlər yerinə yetirilə bilməd. Daha ətraflı məlumat üçün.txt jurnal faylına baxın.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"Ianə",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Korean (한국어)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"VenusGirl, HYB1022",
        
        "STR_ERROR":"오류",
        "STR_WARNING":"경고",
        "STR_INFO":"정보",
        "STR_INCORRECT_DIR":"올바른 파일 경로에서 실행하십시오!",
        "STR_INCORRECT_TREE_DIR":"여기(INSTALL 경로)서 프로그램을 실행하지 말고 설치 패키지를 다운로드하여 압축을 푼 후 그곳에서 실행해 주십시오.",
        "STR_DEVICE":"장치",
        "STR_LOCAL_VER":"패키지의 Ventoy 버전",
        "STR_DISK_VER":"장치 내부의 Ventoy 버전",
        "STR_STATUS":"상태 - 준비 완료",
        "STR_INSTALL":"설치",
        "STR_UPDATE":"업데이트",
        "STR_UPDATE_TIP":"업그레이드 작업은 안전하며 ISO 파일이 변경되지 않습니다.#@계속하시겠습니까?",
        "STR_INSTALL_TIP":"디스크가 포맷되고 모든 데이터가 손실됩니다.#@계속하시겠습니까?",
        "STR_INSTALL_TIP2":"디스크가 포맷되고 모든 데이터가 손실됩니다.#@계속하시겠습니까? (다시 확인)",
        "STR_INSTALL_SUCCESS":"축하합니다!#@Ventoy가 장치에 성공적으로 설치되었습니다.",
        "STR_INSTALL_FAILED":"설치 중 오류가 발생했습니다. USB를 다시 삽입하고 다시 시도할 수 있습니다. 자세한 내용은 /var/log/ventoy.log를 확인하십시오.",
        "STR_UPDATE_SUCCESS":"축하합니다! 장치에 설치된#@Ventoy가 성공적으로 업데이트되었습니다.",
        "STR_UPDATE_FAILED":"업데이트 중에 오류가 발생했습니다. USB를 다시 삽입하고 다시 시도할 수 있습니다. 자세한 내용은 /var/log/ventoy.log를 확인하십시오.",
        "STR_WAIT_PROCESS":"현재 작업이 실행 중입니다. 기다려 주십시오...",
        "STR_MENU_OPTION":"설정",
        "STR_MENU_SECURE_BOOT":"보안 부트 (Secure Boot) 지원",
        "STR_MENU_PART_CFG":"파티션 구성",
        "STR_BTN_OK":"확인",
        "STR_BTN_CANCEL":"취소",
        "STR_PRESERVE_SPACE":"디스크의 끝에 예약된 공간 확보",
        "STR_SPACE_VAL_INVALID":"예약된 공간의 값이 잘못되었습니다",
        "STR_MENU_CLEAR":"Ventoy 제거",
        "STR_CLEAR_SUCCESS":"Ventoy가 장치에서 성공적으로 제거되었습니다.",
        "STR_CLEAR_FAILED":"디스크에서 Ventoy를 지울 때 오류가 발생했습니다. USB를 다시 삽입하고 다시 시도할 수 있습니다. 자세한 내용은 /var/log/ventoy.log를 확인하십시오.",
        "STR_MENU_PART_STYLE":"파티션 유형",
        "STR_DISK_2TB_MBR_ERROR":"2TB 이상의 디스크는 GPT를 선택하십시오",
        "STR_SHOW_ALL_DEV":"모든 장치 표시",
        "STR_PART_ALIGN_4KB":"파티션을 4KB로 정렬",
        "STR_WEB_COMMUNICATION_ERR":"통신 오류:",
        "STR_WEB_REMOTE_ABNORMAL":"통신 오류: 원격 이상",
        "STR_WEB_REQUEST_TIMEOUT":"통신 오류: 요청이 시간 초과됨",
        "STR_WEB_SERVICE_UNAVAILABLE":"통신 오류: 서비스를 사용할 수 없음",
        "STR_WEB_TOKEN_MISMATCH":"백그라운드 서비스 상태가 업데이트되었습니다. 나중에 다시 시도하십시오.",
        "STR_WEB_SERVICE_BUSY":"서비스가 사용 중입니다. 나중에 다시 시도하십시오.",
        "STR_MENU_VTSI_CREATE":"VTSI 파일 생성",
        "STR_VTSI_CREATE_TIP":"이 작업은 장치에 데이터를 쓰지 않고 VTSI 파일#@만 생성합니다.#@계속하시겠습니까?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI 파일이 성공적으로 생성되었습니다!#@Rufus (3.15 이상)를 사용하여 장치에 기록하여 Ventoy 설치를 완료할 수 있습니다.",
        "STR_VTSI_CREATE_FAILED":"VTSI 파일을 생성하지 못했습니다.",
        "STR_MENU_PART_RESIZE":"무손실 설치",
        "STR_PART_RESIZE_TIP":"가능하면 Ventoy가 무손실 설치를 시도합니다. #@계속하시겠습니까?",
        "STR_PART_RESIZE_SUCCESS":"축하합니다!#@Ventoy 무손실 설치가 완료되었습니다.",
        "STR_PART_RESIZE_FAILED":"무손실 설치에 실패했습니다. 자세한 내용은 /var/log/ventoy.log를 확인하십시오.",
        "STR_PART_RESIZE_UNSUPPORTED":"일부 조건을 충족할 수 없기 때문에 Ventoy 무손실 설치가 중지되었습니다. 자세한 내용은 /var/log/ventoy.log를 확인하십시오.",
        "STR_INSTALL_YES_TIP1":"경고: 데이터가 손실됩니다!",
        "STR_INSTALL_YES_TIP2":"업그레이드 대신 새로 설치할지 확인하려면 아래 입력란에 YES를 입력하십시오.",
        "STR_PART_VENTOY_FS":"Ventoy 파티션용 파일 시스템",
        "STR_PART_FS":"파일 시스템",
        "STR_PART_CLUSTER":"클러스터 크기",
        "STR_PART_CLUSTER_DEFAULT":"시스템 기본값",
        "STR_DONATE":"기부",
        "STR_4KN_UNSUPPORTED":"현재 Ventoy는 네이티브 4K 섹터의 디스크를 지원하지 않습니다.",
        
        "STRXXX":""
    },
    {
        "name":"Chinese Simplified (简体中文)",
        "FontFamily":"宋体",
        "FontSize":14,
        "Author":"longpanda",
        
        "STR_ERROR":"错误",
        "STR_WARNING":"警告",
        "STR_INFO":"提醒",
        "STR_INCORRECT_DIR":"请在正确的目录下运行!",
        "STR_INCORRECT_TREE_DIR":"请不要在代码树下运行此程序(INSTALL目录)，请下载安装包，解压后在那里运行。",
        "STR_DEVICE":"设备",
        "STR_LOCAL_VER":"安装包内 Ventoy 版本",
        "STR_DISK_VER":"设备内部 Ventoy 版本",
        "STR_STATUS":"状态 - 准备就绪",
        "STR_INSTALL":"安装",
        "STR_UPDATE":"升级",
        "STR_UPDATE_TIP":"升级操作是安全的, ISO文件不会丢失#@是否继续？",
        "STR_INSTALL_TIP":"磁盘会被格式化, 所有数据都会丢失!#@是否继续？",
        "STR_INSTALL_TIP2":"磁盘会被格式化, 所有数据都会丢失!#@再次确认是否继续？",
        "STR_INSTALL_SUCCESS":"恭喜你! Ventoy 已经成功安装到此设备中.",
        "STR_INSTALL_FAILED":"安装 Ventoy 过程中发生错误. 你可以重新拔插一下U盘然后重试一次, 详细信息请查阅 /var/log/ventoy.log 文件. 如果一直出错，请参考官网常见问题说明.",
        "STR_UPDATE_SUCCESS":"恭喜你! 新版本的 Ventoy 已经成功更新到此设备中.",
        "STR_UPDATE_FAILED":"更新 Ventoy 过程中遇到错误. 你可以重新拔插一下U盘然后重试一次, 详细信息请查阅 /var/log/ventoy.log 文件. 如果一直出错，请参考官网常见问题说明.",
        "STR_WAIT_PROCESS":"当前有任务正在运行, 请等待...",
        "STR_MENU_OPTION":"配置选项",
        "STR_MENU_SECURE_BOOT":"安全启动支持",
        "STR_MENU_PART_CFG":"分区设置",
        "STR_BTN_OK":"确定",
        "STR_BTN_CANCEL":"取消",
        "STR_PRESERVE_SPACE":"在磁盘最后保留一段空间",
        "STR_SPACE_VAL_INVALID":"保留空间的数值非法",
        "STR_MENU_CLEAR":"清除 Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy已经成功从此设备中清除.",
        "STR_CLEAR_FAILED":"清除 Ventoy 过程中遇到错误. 你可以重新拔插一下U盘然后重试一次, 详细信息请查阅 /var/log/ventoy.log 文件.",
        "STR_MENU_PART_STYLE":"分区类型",
        "STR_DISK_2TB_MBR_ERROR":"磁盘容量超过2TB，请选择GPT分区格式",
        "STR_SHOW_ALL_DEV":"显示所有设备",
        "STR_PART_ALIGN_4KB":"分区按照 4KB 对齐",
        "STR_WEB_COMMUNICATION_ERR":"通信失败：",
        "STR_WEB_REMOTE_ABNORMAL":"通信失败：服务端异常",
        "STR_WEB_REQUEST_TIMEOUT":"通信失败：请求超时",
        "STR_WEB_SERVICE_UNAVAILABLE":"通信失败：服务不可用",
        "STR_WEB_TOKEN_MISMATCH":"后台服务状态更新，请稍后重试",
        "STR_WEB_SERVICE_BUSY":"后台服务正忙，请稍后重试",
        "STR_MENU_VTSI_CREATE":"创建 VTSI 文件",
        "STR_VTSI_CREATE_TIP":"本操作不会向设备中写入数据，而只会生成一个 VTSI 文件#@是否继续？",
        "STR_VTSI_CREATE_SUCCESS":"VTSI 文件创建成功，你可以使用Rufus(3.15+)将其写入对应设备，从而完成Ventoy的安装",
        "STR_VTSI_CREATE_FAILED":"VTSI 文件创建失败",
        "STR_MENU_PART_RESIZE":"无损安装",
        "STR_PART_RESIZE_TIP":"Ventoy将尝试进行无损安装。注意：任何磁盘操作都有一定的风险，对于重要数据建议做好备份。#@是否继续？",
        "STR_PART_RESIZE_SUCCESS":"恭喜你! Ventoy 成功完成了无损安装。",
        "STR_PART_RESIZE_FAILED":"Ventoy 无损安装失败。详细信息请查阅 /var/log/ventoy.log 文件。",
        "STR_PART_RESIZE_UNSUPPORTED":"条件不满足，无法进行无损安装。详细信息请查阅 /var/log/ventoy.log 文件。",
        "STR_INSTALL_YES_TIP1":"警告: 所有数据将会丢失！",
        "STR_INSTALL_YES_TIP2":"请在下面文本框中输入 YES 以确认你是要进行安装而不是升级。",
        "STR_PART_VENTOY_FS":"Ventoy 分区文件系统类型",
        "STR_PART_FS":"文件系统",
        "STR_PART_CLUSTER":"簇大小",
        "STR_PART_CLUSTER_DEFAULT":"系统默认值",
        "STR_DONATE":"捐助",
        "STR_4KN_UNSUPPORTED":"目前 Ventoy 不支持原生 4K 扇区的磁盘。",
        
        "STRXXX":""
    },
    {
        "name":"Danish (Dansk)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"fhdk",

        "STR_ERROR":"Fejl",
        "STR_WARNING":"Advarsel",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Skal udføres i den rette mappe!",
        "STR_INCORRECT_TREE_DIR":"Udfør ikke her, download pakke udgivelsen og udfør i den mappe.",
        "STR_DEVICE":"Enhed",
        "STR_LOCAL_VER":"Pakke Version",
        "STR_DISK_VER":"Enhed Version",
        "STR_STATUS":"Status - KLAR",
        "STR_INSTALL":"Installer",
        "STR_UPDATE":"Opdater",
        "STR_UPDATE_TIP":"Updatering er sikker, Data indhold bevares uændret.#@Fortsæt?",
        "STR_INSTALL_TIP":"Alle data på enheden slettes og kan ikke gendannes.#@Fortsæt?",
        "STR_INSTALL_TIP2":"BEKRÆFT! Alle data på enheden slettes og kan ikke gendannes.#@Fortsæt?",
        "STR_INSTALL_SUCCESS":"SÅDAN!#@Ventoy er nu overført til enheden.",
        "STR_INSTALL_FAILED":"Der opstod en fejl under overførslen. Fjern og indsæt enheden på ny for at prøve igen. Læs /var/log/ventoy.log for detaljer. Hvis det gantager sig, Se FAQ på den officielle web.",
        "STR_UPDATE_SUCCESS":"SÅDAN!#@Ventoy er nu opdaterert på enheden",
        "STR_UPDATE_FAILED":"Der opstod en fejl opdateringen. Fjern og indsæt enheden på ny for at prøve igen. Læs /var/log/ventoy.log for detaljer. Hvis det gantager sig, Se FAQ på den officielle web.",
        "STR_WAIT_PROCESS":"Programmet er optaget, vent venligst...",
        "STR_MENU_OPTION":"Valg",
        "STR_MENU_SECURE_BOOT":"Secure Boot Support",
        "STR_MENU_PART_CFG":"Partition Opsætning",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Afbryd",
        "STR_PRESERVE_SPACE":"Efterlad reserveret plads på enheden",
        "STR_SPACE_VAL_INVALID":"Ugyldig værdi for reserveret plads",
        "STR_MENU_CLEAR":"Fjern Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy er nu fjernet fra enheden.",
        "STR_CLEAR_FAILED":"Der opstod en fejl under fjernelse fra enheden. Fjern og indsæt enheden på ny for at prøve igen. Læs /var/log/ventoy.log for detaljer.",
        "STR_MENU_PART_STYLE":"Partition Type",
        "STR_DISK_2TB_MBR_ERROR":"Vælg GPT for enheder over 2TB",
        "STR_SHOW_ALL_DEV":"Vis Alle Enheder",
        "STR_PART_ALIGN_4KB":"Juster partitioner til 4KB blokke.",
        "STR_WEB_COMMUNICATION_ERR":"Kommunikationsfejl:",
        "STR_WEB_REMOTE_ABNORMAL":"Kommunikationsfejl: Uventet server",
        "STR_WEB_REQUEST_TIMEOUT":"Kommunikationsfejl: Server svarer ikke",
        "STR_WEB_SERVICE_UNAVAILABLE":"Kommunikationsfejl: Server ikke tilgængelig",
        "STR_WEB_TOKEN_MISMATCH":"Baggrundsstatus er opdateret. Prøv igen seenere.",
        "STR_WEB_SERVICE_BUSY":"Webservice er optaget, prøv igen senere.",
        "STR_MENU_VTSI_CREATE":"Opret VTSI File",
        "STR_VTSI_CREATE_TIP":"Der skrives ikke til enheden, der bliver blot dannet en VTSI fil#@Fortsæt?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI fil oprettet!#@Du kan nu skrive til enheden ved at bruge Rufus(3.15+) og dermed færddiggøre enheds opsætningen.",
        "STR_VTSI_CREATE_FAILED":"Oprettelse af VTSI mislykkedes.",
        "STR_MENU_PART_RESIZE":"Ikke-Destruktiv overførsel",
        "STR_PART_RESIZE_TIP":"Ventoy vil forsøge en ikke-destruktion overførsel om muligt. #@Fortsæt?",
        "STR_PART_RESIZE_SUCCESS":"SÅDAN!#@ Den ikke-destruktive overførsel var perfekt.",
        "STR_PART_RESIZE_FAILED":"Ikke-Destruktiv overførsel fejlede, Læs /var/log/ventoy.log for detaljer.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ikke-Destruktiv overførsel er afbrudt. Betingelser er ikke opfyldt. Læs /var/log/ventoy.log for detaljer.",
        "STR_INSTALL_YES_TIP1":"Advarsel: Data vil gå tabt!",
        "STR_INSTALL_YES_TIP2":"Indtast ordet YES for at bekræfte en ny overførsel istedet for en opdatring.",
        "STR_PART_VENTOY_FS":"Fil System For Ventoy Partition",
        "STR_PART_FS":"Fil System",
        "STR_PART_CLUSTER":"Klynge størrelse",
        "STR_PART_CLUSTER_DEFAULT":"System Standard Værdi",
        "STR_DONATE":"Donering",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"English (English)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"longpanda",
        
        "STR_ERROR":"Error",
        "STR_WARNING":"Warning",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Please run under the correct directory!",
        "STR_INCORRECT_TREE_DIR":"Don't run me here, please download the released install package, and run there.",
        "STR_DEVICE":"Device",
        "STR_LOCAL_VER":"Ventoy In Package",
        "STR_DISK_VER":"Ventoy In Device",
        "STR_STATUS":"Status - READY",
        "STR_INSTALL":"Install",
        "STR_UPDATE":"Update",
        "STR_UPDATE_TIP":"Upgrade operation is safe, ISO files will be unchanged.#@Continue?",
        "STR_INSTALL_TIP":"The device will be formatted and all the data will be lost.#@Continue?",
        "STR_INSTALL_TIP2":"The device will be formatted and all the data will be lost.#@Continue? (Double Check)",
        "STR_INSTALL_SUCCESS":"Congratulations!#@Ventoy has been successfully installed to the device.",
        "STR_INSTALL_FAILED":"An error occurred during the installation. You can reinsert the USB device and try again. Check /var/log/ventoy.log for details. If it always fails, please refer the FAQ on the official website.",
        "STR_UPDATE_SUCCESS":"Congratulations!#@Ventoy has been successfully updated on the device.",
        "STR_UPDATE_FAILED":"An error occurred during the update. You can reinsert the USB device and try again. Check /var/log/ventoy.log for details. If it always fails, please refer the FAQ on the official website.",
        "STR_WAIT_PROCESS":"A thread is running, please wait...",
        "STR_MENU_OPTION":"Option",
        "STR_MENU_SECURE_BOOT":"Secure Boot Support",
        "STR_MENU_PART_CFG":"Partition Configuration",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Cancel",
        "STR_PRESERVE_SPACE":"Preserve some space at the end of the disk",
        "STR_SPACE_VAL_INVALID":"Invalid value for reserved space",
        "STR_MENU_CLEAR":"Clear Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy has been successfully removed from the device.",
        "STR_CLEAR_FAILED":"An error occurred when removing Ventoy from disk. You can reinsert the USB device and try again. Check /var/log/ventoy.log for details.",
        "STR_MENU_PART_STYLE":"Partition Style",
        "STR_DISK_2TB_MBR_ERROR":"Please select GPT for devices over 2TB",
        "STR_SHOW_ALL_DEV":"Show All Devices",
        "STR_PART_ALIGN_4KB":"Align partitions with 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Communication error:",
        "STR_WEB_REMOTE_ABNORMAL":"Communication error: remote abnormal",
        "STR_WEB_REQUEST_TIMEOUT":"Communication error: Request timed out",
        "STR_WEB_SERVICE_UNAVAILABLE":"Communication error: Service Unavailable",
        "STR_WEB_TOKEN_MISMATCH":"Daemon status updated, please retry later.",
        "STR_WEB_SERVICE_BUSY":"Service is busy, please retry later.",
        "STR_MENU_VTSI_CREATE":"Generate VTSI File",
        "STR_VTSI_CREATE_TIP":"This time will not write to the device, but only generate a VTSI file#@Continue?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI file created successfully!#@You can use Rufus(3.15+) to write it to the device so as to complete the installation of Ventoy.",
        "STR_VTSI_CREATE_FAILED":"VTSI file created failed.",
        "STR_MENU_PART_RESIZE":"Non-destructive Install",
        "STR_PART_RESIZE_TIP":"Ventoy will try non-destructive installation if possible. #@Continue?",
        "STR_PART_RESIZE_SUCCESS":"Congratulations!#@Ventoy non-destructive installation successfully finished.",
        "STR_PART_RESIZE_FAILED":"Non-destructive installation failed, Check /var/log/ventoy.log for details.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy non-destructive installation stopped because some conditions cannot be met. Check /var/log/ventoy.log for details.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"Donate",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name": "georgian (ქართული)",
        "FontFamily": "Courier New",
        "FontSize": 16,
        "Author": "44f33",
        
        "STR_ERROR": "შეცდომა",
        "STR_WARNING": "გაფრთხილება",
        "STR_INFO": "ინფორმაცია",
        "STR_INCORRECT_DIR": "გთხოვთ გაუშვათ სწორ დირექტორიაში!",
        "STR_INCORRECT_TREE_DIR": "აქ არ გაუშვათ, გთხოვთ ჩამოტვირთოთ გამოშვებული ინსტალაციის პაკეტი და იქ და-run-ოთ",
        "STR_DEVICE": "მოწყობილობა",
        "STR_INSTALL_YES_TIP1": "გაფრთხილება: მონაცემები დაიკარგება",
        "STR_INSTALL_YES_TIP2": "გთხოვთ შეიყვანოთ YES რათა დამტკიცდეს რომ თვენ დარწმუნებული ხართ რომ გსურთ ხელახლა დაყენაბა ვიდრე განახლება",
        "STR_LOCAL_VER": "Ventoy პაკეტში",
        "STR_DISK_VER": "Ventoy მოწყობილობაში",
        "STR_STATUS": "სტატუსი - მზადაა",
        "STR_PART_RESIZE_FAILED": "არა-დესტრუქციული ინსტალაცია წარუმატებლივ დასრულა, ნახეთ /var/log/ventoy.log დეტალებისთვის",
        "STR_PART_RESIZE_SUCCESS": "გილოცავთ #@Ventoy არა-დესტრუქციული ინსტალაცია წარმატებით დასრულდა",
        "STR_PART_RESIZE_UNSUPPORTED": "Ventoy-ს არასაზიანო ინსტალაცია შეჩერდა რადგან რაღაც პირობები არ კმაყოფილდება. ნახეთ /var/log/ventoy.log დეტალებისთვის.",
        "STR_INSTALL": "ინსტალაცია",
        "STR_UPDATE": "განახლება",
        "STR_UPDATE_TIP": "განახლების ოპერაცია უსაფრთხოა, ISO ფაილები არ შეიცვლება. გსურს გაგრძელება?",
        "STR_INSTALL_TIP": "მოწყობილობა დაფორმატდება და ყველა მონაცემი დაიკარგება.გსურს გაგრძელება?",
        "STR_INSTALL_TIP2": "მოწყობილობა დაფორმატდება და ყველა მონაცემი დაიკარგება! დარწმუნებული ხართ რომ ამის განხორციელება ნამდვილად გსურთ?",
        "STR_INSTALL_SUCCESS": "გილოცავთ! Ventoy წარმატებით დაინსტალირდა თქვენს მოწყობილობაზე.",
        "STR_INSTALL_FAILED": "დაყენებისას მოხდა შეცდომა. გთხოვთ მიმართოთ FAQ-ს ოფიციალურ ვებსაიტზე.",
        "STR_UPDATE_SUCCESS": "გილოცავთ!#@Ventoy წარმატებით განახლდა მოწყობილობაზე.",
        "STR_UPDATE_FAILED": "შეცდომა მოხდა განახლების დროს. შეგიძლიათ აკონტროლოთ… გაუქმება",
        "STR_WAIT_PROCESS": "შეინარჩუნე გარკვეული სივრცე დისკის ბოლოს",
        "STR_MENU_OPTION": "არასწორი მნიშვნელობა რეზერვირებული სივრცისთვის",
        "STR_MENU_SECURE_BOOT": "გასუფთავება Ventoy",
        "STR_MENU_PART_CFG": "Ventoy წარმატებით წაიშალა მოწყობილობიდან.",
        "STR_BTN_OK": "დაიქმნა შეცდომა Ventoy-ის დისკიდან ამოღებისას მოწყობილობა და სცადეთ ხელახლა. შეამოწმეთ /var/log/ventoy.log დეტალებისთვის.",
        "STR_BTN_CANCEL": "პარტიციის სტილი",
        "STR_PRESERVE_SPACE": "გთხოვთ აირჩიოთ GPT 2 ტერაბიატზე მეტი მოწყობილობებისთვის",
        "STR_SPACE_VAL_INVALID": "ყველა მოწყობილობის ჩვენება",
        "STR_MENU_CLEAR": "პარტიციის გასწორება 4KB-ით",
        "STR_CLEAR_SUCCESS": "კომუნიკაციის შეცდომა:",
        "STR_CLEAR_FAILED": "კომუნიკაციის შეცდომა: დისტანციური არანორმალური",
        "STR_MENU_PART_STYLE": "კომუნიკაციის შეცდომა: მოთხოვნის დრო ამოიწურა",
        "STR_DISK_2TB_MBR_ERROR": "კომუნიკაციის შეცდომა: სერვისი მიუწვდომელია",
        "STR_SHOW_ALL_DEV": "Daemon-ის სტატუსი განახლებულია, გთხოვთ, სცადოთ მოგვიანებით.",
        "STR_PART_ALIGN_4KB": "სერვისი დაკავებულია, გთხოვთ, სცადოთ მოგვიანებით.",
        "STR_WEB_COMMUNICATION_ERR": "VTSI ფაილის გენერირება",
        "STR_WEB_REMOTE_ABNORMAL": "ამჯერად არ დაწეროთ მოწყობილობაში, არამედ მხოლოდ შექმნათ VTSI ფაილი#@გაგრძელება?",
        "STR_WEB_REQUEST_TIMEOUT": "VTSI ფაილი შეიქმნა წარმატებით!#@შეგიძლიათ გამოიყენოთ Rufus…ice, რათა დაასრულოთ Ventoy-ის ინსტალაცია.",
        "STR_WEB_SERVICE_UNAVAILABLE": "VTSI ფაილი შეიქმნა ვერ მოხერხდა.",
        "STR_WEB_TOKEN_MISMATCH": "არადესტრუქციული ინსტალაცია",
        "STR_WEB_SERVICE_BUSY": "Ventoy შეეცდება არა-დესტრუქციულ ინსტალაციას, თუ ეს შესაძლებელია. #@გაგრძელება?",
        "STR_MENU_VTSI_CREATE": "გილოცავთ! #@Ventoy არა-დესტრუქციული ინსტალაცია წარმატებით დასრულდა.",
        "STR_VTSI_CREATE_TIP": "არადესტრუქციული ინსტალაცია ვერ მოხერხდა, შეამოწმეთ /var/log/ventoy.log დეტალებისთვის.",
        "STR_VTSI_CREATE_SUCCESS": "Ventoy არა-დესტრუქციული ინსტალაცია შეჩერდა, რადგან… პირობა ვერ შესრულდება . შეამოწმეთ /var/log/ventoy.log დეტალებისთვის.",
        "STR_VTSI_CREATE_FAILED": "გაფრთხილება: მონაცემები დაიკარგება!",
        "STR_MENU_PART_RESIZE": "გთხოვთ, შეიყვანოთ YES ქვემოთ მოცემულ ტექსტურ ველში, რათა დაადასტუროთ... გსურთ განახლების ნაცვლად განახორციელოთ ახალი ინსტალაცია.",
        "STR_PART_RESIZE_TIP": "Ventoy ჩეეცდება დააყენოს არა-დესტრუქციული რამდენადაც შეეცდება. გსურს გაგრძელება?",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"შემოწირულობა",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX": ""
    },
    {
        "name":"Persian (فارسی)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"ivadham",
        
        "STR_ERROR":"خطا",
        "STR_WARNING":"هشدار",
        "STR_INFO":"توضیحات",
        "STR_INCORRECT_DIR":".لطفاً در پوشه‌دان صحیح اجرا کنید",
        "STR_INCORRECT_TREE_DIR":".این‌جا اجرا نکنید، لطفاً بستهٔ نصب را دانلود و آن‌جا اجرا کنید",
        "STR_DEVICE":"حافظهٔ قابل حمل",
        "STR_LOCAL_VER":"آمادهٔ نصب Ventoy",
        "STR_DISK_VER":"نصب‌شده روی حافظه Ventoy",
        "STR_STATUS":"وضعیت - آماده",
        "STR_INSTALL":"فرمت و نصب",
        "STR_UPDATE":"به‌روزرسانی",
        "STR_UPDATE_TIP":"عملیات به‌روزرسانی بی‌خطر است و داده‌های روی حافظهٔ قابل حمل سالم باقی خواهند ماند. ادامه؟",
        "STR_INSTALL_TIP":"حافظهٔ قابل حمل فرمت و تمامی داده‌های آن پاک خواهد شد. ادامه؟",
        "STR_INSTALL_TIP2":"توجه: این عملیات غیرقابل بازگشت است. ادامه؟",
        "STR_INSTALL_SUCCESS":".تبریک! عملیات نصب بر روی حافظهٔ قابل حمل با موفقیت پایان یافت",
        "STR_INSTALL_FAILED":".خطا در عملیات نصب! یو‌اس‌بی را از نو متصل و دوباره تلاش کنید",
        "STR_UPDATE_SUCCESS":".تبریک! نرم‌افزار با موفقیت بر روی حافظهٔ قابل حمل به‌روزرسانی شد",
        "STR_UPDATE_FAILED":".خطا در به‌روزرسانی! یو‌اس‌بی را از نو متصل و دوباره تلاش کنید",
        "STR_WAIT_PROCESS":"...پروسه‌ای در حال اجرا است، لطفاً صبر کنید",
        "STR_MENU_OPTION":"تنظیمات",
        "STR_MENU_SECURE_BOOT":"(Secure Boot) بوت امن",
        "STR_MENU_PART_CFG":"پیکربندی پارتیشن",
        "STR_BTN_OK":"تأیید",
        "STR_BTN_CANCEL":"انصراف",
        "STR_PRESERVE_SPACE":"اختصاص مقداری فضا به انتهای حافظهٔ قابل حمل",
        "STR_SPACE_VAL_INVALID":".این مقدار فضا برای اختصاص‌یافتن نامعتبر است",
        "STR_MENU_CLEAR":"Ventoy را پاک کنید",
        "STR_CLEAR_SUCCESS":"Ventoy با موفقیت از دستگاه برداشته شد.",
        "STR_CLEAR_FAILED":"هنگام پاک کردن Ventoy از روی دیسک خطایی روی داد. می توانید USB را دوباره از برق جدا کنید و دوباره امتحان کنید. /var/log/ventoy.log را برای جزئیات بررسی کنید.",
        "STR_MENU_PART_STYLE":"سبک پارتیشن بندی",
        "STR_DISK_2TB_MBR_ERROR":"لطفاً GPT را برای دیسک بیش از 2 ترابایت انتخاب کنید",
        "STR_SHOW_ALL_DEV":"نمایش همه دستگاه ها",
        "STR_PART_ALIGN_4KB":"پارتیشن ها را با 4KB تراز کنید",
        "STR_WEB_COMMUNICATION_ERR":"خطای ارتباطی:",
        "STR_WEB_REMOTE_ABNORMAL":"خطای ارتباطی: از راه دور غیر طبیعی است",
        "STR_WEB_REQUEST_TIMEOUT":"خطای ارتباطی: زمان درخواست به پایان رسیده است",
        "STR_WEB_SERVICE_UNAVAILABLE":"خطای ارتباطی: سرویس موجود نیست",
        "STR_WEB_TOKEN_MISMATCH":"وضعیت Daemon به روز شد ، لطفاً بعداً دوباره امتحان کنید.",
        "STR_WEB_SERVICE_BUSY":"سرویس شلوغ است ، لطفاً بعداً دوباره امتحان کنید.",
        "STR_MENU_VTSI_CREATE":"Generate VTSI File",
        "STR_VTSI_CREATE_TIP":"This time will not write to the device, but only generate a VTSI file#@Continue?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI file created successfully!#@You can use Rufus(3.15+) to write it to the device so as to complete the installation of Ventoy.",
        "STR_VTSI_CREATE_FAILED":"VTSI file created failed.",
        "STR_MENU_PART_RESIZE":"Non-destructive Install",
        "STR_PART_RESIZE_TIP":"Ventoy will try non-destructive installation if possible. #@Continue?",
        "STR_PART_RESIZE_SUCCESS":"Congratulations!#@Ventoy non-destructive installation successfully finished.",
        "STR_PART_RESIZE_FAILED":"Non-destructive installation failed, Check /var/log/ventoy.log for details.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy non-destructive installation stopped because some conditions cannot be met. Check /var/log/ventoy.log for details.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"اهداء",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Polish (Polski)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"B.O.S.S, ChuckMichael",
        
        "STR_ERROR":"Błąd",
        "STR_WARNING":"Ostrzeżenie",
        "STR_INFO":"Informacja",
        "STR_INCORRECT_DIR":"Proszę uruchom w odpowiednim katalogu!",
        "STR_INCORRECT_TREE_DIR":"Nie uruchamiaj mnie tutaj, pobierz pakiet instalacyjny i uruchom go.",
        "STR_DEVICE":"Urządzenie",
        "STR_LOCAL_VER":"Ventoy w programie",
        "STR_DISK_VER":"Ventoy na urządzeniu",
        "STR_STATUS":"Status - GOTOWY",
        "STR_INSTALL":"Zainstaluj",
        "STR_UPDATE":"Zaktualizuj",
        "STR_UPDATE_TIP":"Operacja aktualizacji jest bezpieczna, pliki ISO pozostaną niezmienione.#@Kontynuować?",
        "STR_INSTALL_TIP":"Dysk zostanie sformatowany, a wszystkie dane zostaną utracone.#@Kontynuować?",
        "STR_INSTALL_TIP2":"Dysk zostanie sformatowany, a wszystkie dane zostaną utracone.#@Kontynuować? (Ponowne sprawdzenie)",
        "STR_INSTALL_SUCCESS":"Gratulacje!#@Ventoy został pomyślnie zainstalowany na urządzeniu.",
        "STR_INSTALL_FAILED":"Wystąpił błąd podczas instalacji. Możesz ponownie podłączyć USB i spróbować jeszcze raz. Szczegóły w /var/log/ventoy.log.",
        "STR_UPDATE_SUCCESS":"Gratulacje!#@Ventoy został pomyślnie zaktualizowany na urządzeniu.",
        "STR_UPDATE_FAILED":"Wystąpił błąd podczas aktualizacji. Możesz ponownie podłączyć USB i spróbować jeszcze raz. Szczegóły w /var/log/ventoy.log.",
        "STR_WAIT_PROCESS":"Wątek jest uruchomiony, proszę czekać...",
        "STR_MENU_OPTION":"Opcje",
        "STR_MENU_SECURE_BOOT":"Wsparcie Secure Boot",
        "STR_MENU_PART_CFG":"Konfiguracja partycji",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Anuluj",
        "STR_PRESERVE_SPACE":"Zarezerwuj trochę miejsca na końcu dysku",
        "STR_SPACE_VAL_INVALID":"Nieprawidłowa wartość dla zarezerwowanego miejsca",
        "STR_MENU_CLEAR":"Usuń Ventoy z urządzenia",
        "STR_CLEAR_SUCCESS":"Ventoy został pomyślnie usunięty z urządzenia.",
        "STR_CLEAR_FAILED":"Wystąpił błąd podczas usuwania Ventoy z dysku. Możesz podłączyć USB i spróbować jeszcze raz. Szczegóły w /var/log/ventoy.log.",
        "STR_MENU_PART_STYLE":"Schemat partycjonowania",
        "STR_DISK_2TB_MBR_ERROR":"Wybierz GPT dla dysku powyżej 2TB",
        "STR_SHOW_ALL_DEV":"Pokaż wszystkie urządzenia",
        "STR_PART_ALIGN_4KB":"Wyrównaj partycje do 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Błąd komunikacji:",
        "STR_WEB_REMOTE_ABNORMAL":"Błąd komunikacji: nieprawidłowy serwer",
        "STR_WEB_REQUEST_TIMEOUT":"Błąd komunikacji: przekroczono limit czasu żądania",
        "STR_WEB_SERVICE_UNAVAILABLE":"Błąd komunikacji: usługa niedostępna",
        "STR_WEB_TOKEN_MISMATCH":"Stan daemona został zaktualizowany, spróbuj ponownie później.",
        "STR_WEB_SERVICE_BUSY":"Usługa jest zajęta, spróbuj ponownie później.",
        "STR_MENU_VTSI_CREATE":"Wygeneruj plik VTSI",
        "STR_VTSI_CREATE_TIP":"Tym razem nie zapisze na urządzeniu, ale tylko wygeneruje plik VTSI#@Kontynuować?",
        "STR_VTSI_CREATE_SUCCESS":"Pomyślnie wygenerowany plik VTSI!#@Możesz teraz użyć Rufus'a(3.15+) aby zapisać go na urządzeniu by dokończyć instalację Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Wygenerowanie pliku VTSI nie powiodło się.",
        "STR_MENU_PART_RESIZE":"Instalacja bez utraty danych",
        "STR_PART_RESIZE_TIP":"Jeśli to możliwe, Ventoy spróbuje przeprowadzić instalację bez utraty danych. #@Kontynuować?",
        "STR_PART_RESIZE_SUCCESS":"Gratulacje! Instalacja bez utraty danych #@Ventoy zakończona pomyślnie.",
        "STR_PART_RESIZE_FAILED":"Instalacja bez utraty danych zakończyła się niepowodzeniem. Szczegóły znajdziesz w pliku /var/log/ventoy.log.",
        "STR_PART_RESIZE_UNSUPPORTED":"Instalacja bez utraty danych Ventoy została przerwana, ponieważ nie można spełnić niektórych warunków. Szczegóły znajdziesz w pliku /var/log/ventoy.log.",
        "STR_INSTALL_YES_TIP1":"Ostrzeżenie: Dane zostaną utracone!",
        "STR_INSTALL_YES_TIP2":"Wpisz YES w poniższym polu tekstowym, aby potwierdzić, że naprawdę chcesz przeprowadzić nową instalację zamiast aktualizacji.",
        "STR_PART_VENTOY_FS":"System plików dla partycji Ventoy",
        "STR_PART_FS":"System plików",
        "STR_PART_CLUSTER":"Wielkość klastra",
        "STR_PART_CLUSTER_DEFAULT":"Wartość domyślna systemu",
        "STR_DONATE":"Przekaż darowiznę",
        "STR_4KN_UNSUPPORTED":"Obecnie Ventoy nie obsługuje urządzeń o fizycznym rozmiarze sektora 4K.",
        
        "STRXXX":""
    },
    {
        "name":"Portuguese Brazilian (Português Brasileiro)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"EstevaoCostaG3, David BrazSan, Hildo Guillardi Júnior, Chrystian Rubio, Felipe",
        
        "STR_ERROR":"Erro",
        "STR_WARNING":"Aviso",
        "STR_INFO":"Informações",
        "STR_INCORRECT_DIR":"Por favor execute no diretório correto!",
        "STR_INCORRECT_TREE_DIR":"Não me execute aqui, por favor baixe o pacote de instalação lançado e me execute lá.",
        "STR_DEVICE":"Dispositivo",
        "STR_LOCAL_VER":"Ventoy no Pacote",
        "STR_DISK_VER":"Ventoy no Dispositivo",
        "STR_STATUS":"Status - PRONTO",
        "STR_INSTALL":"Instalar",
        "STR_UPDATE":"Atualizar",
        "STR_UPDATE_TIP":"A operação de atualização é segura, os arquivos ISO não serão alterados.#@Continuar?",
        "STR_INSTALL_TIP":"O dispositivo será formatado e todos os dados serão perdidos.#@Continuar?",
        "STR_INSTALL_TIP2":"O dispositivo será formatado e todos os dados serão perdidos.#@Continuar? (Verificação Dupla)",
        "STR_INSTALL_SUCCESS":"Parabéns!#@O Ventoy foi instalado com sucesso no dispositivo.",
        "STR_INSTALL_FAILED":"Um erro ocorreu durante a instalação. Você pode re-inserir o dispositivo USB e tentar de novo. Verifique o /var/log/ventoy.log para mais detalhes. Se ele sempre falha por favor refira-se ao FAQ no site oficial da web.",
        "STR_UPDATE_SUCCESS":"Parabéns!#@O Ventoy foi atualizado com sucesso no dispositivo.",
        "STR_UPDATE_FAILED":"Um erro ocorreu durante a atualização. Você pode re-inserir o dispositivo USB e tentar de novo. Verifique o /var/log/ventoy.log para mais detalhes. Se ele sempre falha por favor refira-se ao FAQ no site oficial da web.",
        "STR_WAIT_PROCESS":"Um thread está em execução, por favor espere...",
        "STR_MENU_OPTION":"Opção",
        "STR_MENU_SECURE_BOOT":"Suporte ao Boot Seguro",
        "STR_MENU_PART_CFG":"Configuração da Partição",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Cancelar",
        "STR_PRESERVE_SPACE":"Preservar algum espaço no fim do disco",
        "STR_SPACE_VAL_INVALID":"Valor inválido para o espaço reservado",
        "STR_MENU_CLEAR":"Limpar o Ventoy",
        "STR_CLEAR_SUCCESS":"O Ventoy foi removido do dispositivo com sucesso.",
        "STR_CLEAR_FAILED":"Um erro ocorreu quando removia o Ventoy do disco. Você pode re-inserir o dispositivo USB e tentar de novo. Verifique o /var/log/ventoy.log para mais detalhes.",
        "STR_MENU_PART_STYLE":"Estilo da Partição",
        "STR_DISK_2TB_MBR_ERROR":"Por favor selecione o GPT para dispositivos acima de 2 TBs",
        "STR_SHOW_ALL_DEV":"Mostrar Todos os Dispositivos",
        "STR_PART_ALIGN_4KB":"Alinhar as partições com 4 KBs",
        "STR_WEB_COMMUNICATION_ERR":"Erro de comunicação:",
        "STR_WEB_REMOTE_ABNORMAL":"Erro de comunicação: Remoto Anormal",
        "STR_WEB_REQUEST_TIMEOUT":"Erro de comunicação: O Tempo da Requisição se Esgotou",
        "STR_WEB_SERVICE_UNAVAILABLE":"Erro de comunicação: Serviço Indisponível",
        "STR_WEB_TOKEN_MISMATCH":"Status do daemon atualizado, por favor tente de novo mais tarde.",
        "STR_WEB_SERVICE_BUSY":"O serviço está ocupado, por favor tente de novo mais tarde.",
        "STR_MENU_VTSI_CREATE":"Gerar arquivo VTSI",
        "STR_VTSI_CREATE_TIP":"Desta vez não gravará no dispositivo mas vai gerar apenas um arquivo VTSI.#@Continuar?",
        "STR_VTSI_CREATE_SUCCESS":"Arquivo VTSI criado com sucesso!#@Você pode usar o Rufus (3.15+) pra gravá-lo no dispositivo e completar a instalação do Ventoy.",
        "STR_VTSI_CREATE_FAILED":"A criação do arquivo VTSI falhou.",
        "STR_MENU_PART_RESIZE":"Instalação não destrutiva",
        "STR_PART_RESIZE_TIP":"O Ventoy tentará uma instalação não destrutiva se possível.#@Continuar?",
        "STR_PART_RESIZE_SUCCESS":"Parabéns!#@Instalação não destrutiva do Ventoy concluída com sucesso.",
        "STR_PART_RESIZE_FAILED":"A instalação não destrutiva falhou, verifique o /var/log/ventoy.log para detalhes.",
        "STR_PART_RESIZE_UNSUPPORTED":"A instalação não destrutiva do Ventoy parou porque algumas condições não podem ser atendidas. Verifique o /var/log/ventoy.log para detalhes.",
        "STR_INSTALL_YES_TIP1":"Aviso: Os dados serão perdidos!",
        "STR_INSTALL_YES_TIP2":"Por favor insira YES na caixa de texto abaixo pra confirmar que você de fato quer fazer uma nova instalação ao invés de atualizar.",
        "STR_PART_VENTOY_FS":"Sistema de Arquivos para a partição do Ventoy",
        "STR_PART_FS":"Sistema de Arquivos",
        "STR_PART_CLUSTER":"Tamanho do Cluster",
        "STR_PART_CLUSTER_DEFAULT":"Valor Padrão do Sistema",
        "STR_DONATE":"Doar",
        "STR_4KN_UNSUPPORTED":"Atualmente o Ventoy não suporta dispositivos 4K nativos.",
        
        "STRXXX":""
    },
	{
        "name":"Slovenian (Slovenščina)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Andrew Poženel, Urajmal",
        
        "STR_ERROR":"Napaka",
        "STR_WARNING":"Opozorilo",
        "STR_INFO":"Informacija",
        "STR_INCORRECT_DIR":"Prosim, zaženite program v pravilni mapi!",
        "STR_INCORRECT_TREE_DIR":"Ne izvajajte programa tukaj, prosim prenesite izdani namestitveni paket in ga izvedite tam.",
        "STR_DEVICE":"Naprava",
        "STR_LOCAL_VER":"Ventoy različica",
        "STR_DISK_VER":"Ventoy v napravi",
        "STR_STATUS":"Stanje - PRIPRAVLJEN",
        "STR_INSTALL":"Namesti",
        "STR_UPDATE":"Posodobi",
        "STR_UPDATE_TIP":"Nadgradnja je varna, datoteke ISO ostanejo nespremenjene.#@Nadaljuj?",
        "STR_INSTALL_TIP":"Naprava bo formatirana in vsi podatki bodo izgubljeni.#@Nadaljuj?",
        "STR_INSTALL_TIP2":"Naprava bo formatirana in vsi podatki bodo izgubljeni.#@Nadaljuj? (dvakrat preverite)",
        "STR_INSTALL_SUCCESS":"Čestitamo!#@Ventoy je bil uspešno nameščen na napravo.",
        "STR_INSTALL_FAILED":"Med namestitvijo je prišlo do napake. Ponovno vstavite USB-napravo in poskusite znova. Podrobnosti najdete v datoteki /var/log/ventoy.log. Če napaka še vedno obstaja, si oglejte pogosta vprašanja na uradni spletni strani.",
        "STR_UPDATE_SUCCESS":"Čestitamo!#@Ventoy je bil uspešno posodobljen na napravi.",
        "STR_UPDATE_FAILED":"Med posodobitvijo je prišlo do napake. Ponovno vstavite USB-napravo in poskusite znova. Podrobnosti najdete v datoteki /var/log/ventoy.log. Če se napaka ponavlja, si oglejte pogosta vprašanja na uradni spletni strani.",
        "STR_WAIT_PROCESS":"Nit se izvaja, prosimo počakajte...",
        "STR_MENU_OPTION":"Možnost",
        "STR_MENU_SECURE_BOOT":"Podpora za varen zagon",
        "STR_MENU_PART_CFG":"Konfiguracija particije",
        "STR_BTN_OK":"V redu",
        "STR_BTN_CANCEL":"Prekliči",
        "STR_PRESERVE_SPACE":"Ohrani nekaj prostora na koncu diska",
        "STR_SPACE_VAL_INVALID":"Neveljavna vrednost za rezervirani prostor",
        "STR_MENU_CLEAR":"Počisti Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy je bil uspešno odstranjen iz naprave.",
        "STR_CLEAR_FAILED":"Pri odstranjevanju Ventoya z diska je prišlo do napake. Ponovno vstavite USB-napravo in poskusite znova. Podrobnosti najdete v datoteki /var/log/ventoy.log.",
        "STR_MENU_PART_STYLE":"Slog particije",
        "STR_DISK_2TB_MBR_ERROR":"Za naprave z več kot 2 TB izberite GPT.",
        "STR_SHOW_ALL_DEV":"Pokaži vse naprave",
        "STR_PART_ALIGN_4KB":"Poravnaj particije s 4 KB",
        "STR_WEB_COMMUNICATION_ERR":"Napaka pri komunikaciji:",
        "STR_WEB_REMOTE_ABNORMAL":"Napaka pri komunikaciji: Oddaljena napaka",
        "STR_WEB_REQUEST_TIMEOUT":"Napaka pri komunikaciji: Zahteva je potekla",
        "STR_WEB_SERVICE_UNAVAILABLE":"Napaka pri komunikaciji: Storitev ni na voljo",
        "STR_WEB_TOKEN_MISMATCH":"Stanje ozadnega procesa je bilo posodobljeno, poskusite pozneje.",
        "STR_WEB_SERVICE_BUSY":"Storitev je zasedena, prosimo poskusite kasneje.",
        "STR_MENU_VTSI_CREATE":"Ustvari datoteko VTSI",
        "STR_VTSI_CREATE_TIP":"Tokrat ne bo zapisoval na napravo, ampak bo samo ustvaril datoteko VTSI#@Nadaljuj?",
        "STR_VTSI_CREATE_SUCCESS":"Datoteka VTSI je bila uspešno ustvarjena!#@Za namestitev Ventoya jo lahko zapišete na napravo z uporabo programa Rufus (3.15+).",
        "STR_VTSI_CREATE_FAILED":"Ustvarjanje datoteke VTSI ni uspelo.",
        "STR_MENU_PART_RESIZE":"Neuničujoča namestitev",
        "STR_PART_RESIZE_TIP":"Ventoy bo poskusil z neuničujočo namestitvijo, če je le mogoče. #@Nadaljuj?",
        "STR_PART_RESIZE_SUCCESS":"Čestitamo!#@Ventoy neuničljiva namestitev je bila uspešno zaključena.",
        "STR_PART_RESIZE_FAILED":"Neuničljiva namestitev ni uspela, podrobnosti preverite v datoteki /var/log/ventoy.log.",
        "STR_PART_RESIZE_UNSUPPORTED":"Neuničljiva namestitev Ventoy se je ustavila, ker nekateri pogoji niso izpolnjeni. Podrobnosti preverite v datoteki /var/log/ventoy.log.",
        "STR_INSTALL_YES_TIP1":"Opozorilo: Podatki bodo izgubljeni!",
        "STR_INSTALL_YES_TIP2":"V spodnje besedilno polje vnesite DA, da potrdite, da res želite izvesti novo namestitev namesto nadgradnje.",
        "STR_PART_VENTOY_FS":"Datotečni sistem za particijo Ventoy",
        "STR_PART_FS":"Datotečni sistem",
        "STR_PART_CLUSTER":"Velikost gruče",
        "STR_PART_CLUSTER_DEFAULT":"Privzeta sistemska vrednost",
        "STR_DONATE":"Donirajte",
        "STR_4KN_UNSUPPORTED":"Trenutno Ventoy ne podpira naprav z ločljivostjo 4K.",
        
        "STRXXX":""
    },
    {
        "name":"Turkish (Türkçe)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Erhan Kültür,Tayfun Akkoyun, Oğuz Ersen",
        
        "STR_ERROR":"Hata",
        "STR_WARNING":"Uyarı",
        "STR_INFO":"Bilgi",
        "STR_INCORRECT_DIR":"Lütfen doğru dizin altında çalıştırın!",
        "STR_INCORRECT_TREE_DIR":"Dosyayı burada çalıştırmayın, lütfen yayınlanan en son kurulum paketini indirin ve oradan çalıştırın.",
        "STR_DEVICE":"Aygıt",
        "STR_LOCAL_VER":"Ventoy GÜNCEL Sürüm",
        "STR_DISK_VER":"Aygıttaki Ventoy Sürümü",
        "STR_STATUS":"Durum - HAZIR",
        "STR_INSTALL":"Kur",
        "STR_UPDATE":"Güncelle",
        "STR_UPDATE_TIP":"Yükseltme işlemi güvenlidir, ISO dosyalarınız bundan etkilenmeyecek.#@Devam edilsin mi?",
        "STR_INSTALL_TIP":"Disk biçimlendirilecek ve tüm veriler kaybolacak.#@Devam edilsin mi?",
        "STR_INSTALL_TIP2":"Disk biçimlendirilecek ve tüm veriler kaybolacak.#@Devam edilsin mi? (Son UYARI!)",
        "STR_INSTALL_SUCCESS":"Tebrikler!#@Ventoy, aygıta başarıyla kuruldu.",
        "STR_INSTALL_FAILED":"Kurulum sırasında bir hata oluştu. USB'yi yeniden takıp tekrar deneyebilirsiniz. Ayrıntılar için /var/log/ventoy.log dosyasına bakın.",
        "STR_UPDATE_SUCCESS":"Tebrikler!#@Ventoy, aygıtta başarıyla güncellendi.",
        "STR_UPDATE_FAILED":"Güncelleme sırasında bir hata oluştu. USB'yi yeniden takıp tekrar deneyebilirsiniz. Ayrıntılar için /var/log/ventoy.log dosyasına bakın.",
        "STR_WAIT_PROCESS":"Mevcut işlem çalışıyor, lütfen bekleyin...",
        "STR_MENU_OPTION":"Seçenek",
        "STR_MENU_SECURE_BOOT":"Güvenli Önyükleme Desteği",
        "STR_MENU_PART_CFG":"Bölüm Yapılandırması",
        "STR_BTN_OK":"Tamam",
        "STR_BTN_CANCEL":"İptal",
        "STR_PRESERVE_SPACE":"Diskin sonunda biraz ayrılmış alan bırak",
        "STR_SPACE_VAL_INVALID":"Ayrılmış alan için geçersiz değer girdiniz",
        "STR_MENU_CLEAR":"Ventoy'u USB Diskten Sil",
        "STR_CLEAR_SUCCESS":"Ventoy başarılı bir şekilde diskten silindi",
        "STR_CLEAR_FAILED":"Ventoy USB diskten temizlenirken bir hata oluştu. USB'yi yeniden takıp tekrar deneyebilirsiniz. Ayrıntılar için /var/log/ventoy.log dosyasına bakın",
        "STR_MENU_PART_STYLE":"Bölüm Yapısı",
        "STR_DISK_2TB_MBR_ERROR":"2TB üstündeki diskler için lütfen GPT disk yapısını seçin.",
        "STR_SHOW_ALL_DEV":"Tüm Aygıtları Göster",
        "STR_PART_ALIGN_4KB":"Tüm bölümleri 4KB düzeninde hizala",
        "STR_WEB_COMMUNICATION_ERR":"İletişim Hatası:",
        "STR_WEB_REMOTE_ABNORMAL":"İletişim Hatası:Uzaktan erişim hizmeti anormal durumda",
        "STR_WEB_REQUEST_TIMEOUT":"İletişim hatası: Uzaktan erişim isteği zaman aşımına uğradı",
        "STR_WEB_SERVICE_UNAVAILABLE":"İletişim hatası: Uzaktan web erişim hizmeti erişilemez durumda",
        "STR_WEB_TOKEN_MISMATCH":"Daemon durumu güncellendi,lütfen daha sonra tekrar deneyin.",
        "STR_WEB_SERVICE_BUSY":"Hizmet meşgul, lütfen daha sonra tekrar deneyin.",
        "STR_MENU_VTSI_CREATE":"VTSI Dosyası Oluştur",
        "STR_VTSI_CREATE_TIP":"Şu an aygıta yazılmayacak,sadece bir VTSI dosyası oluşturulacak#@Devam edilsin mi?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI dosyası başarılı bir şekilde oluşturuldu!#@Ventoy'un aygıta kurulumunu tamamlamak için Rufus(3.15+) programını kullanabilirsiniz.",
        "STR_VTSI_CREATE_FAILED":"VTSI dosyası oluşturma başarısız oldu!",
        "STR_MENU_PART_RESIZE":"Tahribatsız Kurulum",
        "STR_PART_RESIZE_TIP":"Ventoy,mümkünse tahribatsız kurulumu deneyecektir. . #@Devam edilsin mi?",
        "STR_PART_RESIZE_SUCCESS":"Tebrikler!#@Ventoy Tahribatsız Kurulum, başarıyla tamamlandı.",
        "STR_PART_RESIZE_FAILED":"Tahribatsız kurulum başarısız oldu, ayrıntılar için /var/log/ventoy.log dosyasını kontrol edin.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy Tahribatsız Kurulum, bazı koşullar sağlanmadığı için durduruldu. Ayrıntılı bilgi için /var/log/ventoy.log dosyasını kontrol edin.",
        "STR_INSTALL_YES_TIP1":"Uyarı: Tüm verileriniz kaybolacak!",
        "STR_INSTALL_YES_TIP2":"Güncelleme yapmak yerine,yeni bir kurulum yapmak istediğinizi doğrulamak için lütfen aşağıdaki metin kutusuna YES yazın.",
        "STR_PART_VENTOY_FS":"Ventoy bölümü Dosya Sistemi",
        "STR_PART_FS":"Dosya Sistemi",
        "STR_PART_CLUSTER":"Sektör Boyutu",
        "STR_PART_CLUSTER_DEFAULT":"Sistem varsayılan değeri",
        "STR_DONATE":"Ventoy'a Bağış yap",
        "STR_4KN_UNSUPPORTED":"Diskiniz 4K(4096byte) sektör yapısına sahip.Ventoy, 4K sektör yapısına sahip cihazlara şu anda destek vermemektedir.",
        
        "STRXXX":""
    },
    {
        "name":"German (Deutsch)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"luzea & u!^DEV, wiz64",
        
        "STR_ERROR":"Fehler",
        "STR_WARNING":"Warnung",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Bitte im richtigen Verzeichnis ausführen!",
        "STR_INCORRECT_TREE_DIR":"Bitte nicht hier ausführen. Laden Sie das Installationspaket herunter und von dort aus ausführen.",
        "STR_DEVICE":"Gerät",
        "STR_LOCAL_VER":"Ventoy (lokal)",
        "STR_DISK_VER":"Ventoy (Gerät)",
        "STR_STATUS":"Status - BEREIT",
        "STR_INSTALL":"Installieren",
        "STR_UPDATE":"Aktualisieren",
        "STR_UPDATE_TIP":"Die Durchführung der Aktualisierung ist sicher, ISO-Dateien bleiben unverändert.#@Fortfahren?",
        "STR_INSTALL_TIP":"Das Gerät wird formatiert und alle Daten gehen verloren.#@Fortfahren?",
        "STR_INSTALL_TIP2":"Das Gerät wird formatiert und alle Daten gehen verloren.#@Fortfahren? (erneute Überprüfung)",
        "STR_INSTALL_SUCCESS":"Herzlichen Glückwunsch!#@Ventoy wurde erfolgreich auf dem Gerät installiert.",
        "STR_INSTALL_FAILED":"Während der Installation ist ein Fehler aufgetreten. Stecken Sie das Gerät neu ein und versuchen Sie es erneut. Details finden Sie in /var/log/ventoy.log. Falls der Vorgang immer fehlschlägt, lesen Sie bitte das FAQ auf der offiziellen Website.",
        "STR_UPDATE_SUCCESS":"Herzlichen Glückwunsch!#@Ventoy wurde erfolgreich auf dem Gerät aktualisiert.",
        "STR_UPDATE_FAILED":"Während der Aktualisierung ist ein Fehler aufgetreten. Stecken Sie das Gerät neu ein und versuchen Sie es erneut. Details finden Sie in /var/log/ventoy.log. Falls der Vorgang immer fehlschlägt, lesen Sie bitte das FAQ auf der offiziellen Website.",
        "STR_WAIT_PROCESS":"Ein anderer Thread läuft, bitte warten...",
        "STR_MENU_OPTION":"Optionen",
        "STR_MENU_SECURE_BOOT":"Secure Boot",
        "STR_MENU_PART_CFG":"Partitionskonfiguration",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Abbrechen",
        "STR_PRESERVE_SPACE":"Etwas Speicher am Ende des Datenträgers reservieren.",
        "STR_SPACE_VAL_INVALID":"Ungültiger Wert für reservierten Speicherplatz!",
        "STR_MENU_CLEAR":"Ventoy entfernen",
        "STR_CLEAR_SUCCESS":"Ventoy wurde erfolgreich vom Gerät entfernt.",
        "STR_CLEAR_FAILED":"Beim Versuch, Ventoy zu entfernen, ist ein Fehler aufgetreten. Stecken Sie das Gerät neu ein und versuchen Sie es erneut. Überprüfen Sie die /var/log/ventoy.log auf Details.",
        "STR_MENU_PART_STYLE":"Partitionsstil",
        "STR_DISK_2TB_MBR_ERROR":"Wählen Sie GPT für Geräte > 2TB aus.",
        "STR_SHOW_ALL_DEV":"Alle Geräte anzeigen",
        "STR_PART_ALIGN_4KB":"Richten Sie Partitionen mit 4KB aus",
        "STR_WEB_COMMUNICATION_ERR":"Kommunikationsfehler:",
        "STR_WEB_REMOTE_ABNORMAL":"Kommunikationsfehler: Unnormales Verhalten",
        "STR_WEB_REQUEST_TIMEOUT":"Kommunikationsfehler: Zeitüberschreitung der Anforderung",
        "STR_WEB_SERVICE_UNAVAILABLE":"Kommunikationsfehler: Dienst nicht verfügbar",
        "STR_WEB_TOKEN_MISMATCH":"Daemon-Status aktualisiert, bitte später erneut versuchen.",
        "STR_WEB_SERVICE_BUSY":"Dienst ist ausgelastet, bitte später erneut versuchen.",
        "STR_MENU_VTSI_CREATE":"VTSI-Datei erstellen",
        "STR_VTSI_CREATE_TIP":"Es wird nur eine VTSI-Datei erstellt und nichts auf ein Gerät geschrieben. #@Fortfahren?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI-Datei erfolgreich erstellt!#@Sie können Rufus(3.15+) zum auf das Gerät schreiben verwenden um die Installation von Ventoy fertigzustellen.",
        "STR_VTSI_CREATE_FAILED":"VTSI-Datei konnte nicht erstellt werden.",
        "STR_MENU_PART_RESIZE":"Zerstörungsfreie Installation",
        "STR_PART_RESIZE_TIP":"Ventoy wird eine Installation ohne vorherige Formatierung versuchen. #@Fortfahren?",
        "STR_PART_RESIZE_SUCCESS":"Glückwunsch! #@Die Installation von Ventoy, ohne vorherige Formatierung, wurde erfolgreich abgeschlossen.",
        "STR_PART_RESIZE_FAILED":"Installation ohne vorherige Formatierung fehlgeschlagen. Details dazu finden Sie in /var/log/ventoy.log.",
        "STR_PART_RESIZE_UNSUPPORTED":"Aufgrund einiger nicht erfüllbarer Bedingungen wurde die Installation, ohne vorherige Formatierung, gestoppt. Details dazu finden Sie in /var/log/ventoy.log.",
        "STR_INSTALL_YES_TIP1":"Warnung: Die Daten gehen verloren!",
        "STR_INSTALL_YES_TIP2":"Bitte bestätigen Sie die untere Textbox, um eine Neuinstallation durchzuführen.",
        "STR_PART_VENTOY_FS":"Dateisystem für die Ventoy-Partition",
        "STR_PART_FS":"Dateisystem",
        "STR_PART_CLUSTER":"Clustergröße",
        "STR_PART_CLUSTER_DEFAULT":"Systemstandardwert",
        "STR_DONATE":"Spenden",
        "STR_4KN_UNSUPPORTED":"Ventoy unterstützt derzeit keine nativen 4K-Geräte.",
        
        "STRXXX":""
    },
    {
        "name":"Occitan (Occitan)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"quentin",
        
        "STR_ERROR":"Error",
        "STR_WARNING":"Avertiment",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Aqueste programa deu s’executar dins lo bon repertòri !",
        "STR_INCORRECT_TREE_DIR":"Lancetz pas aquò, mercés de telecargar lo programa d’installacion e relançatz-lo.",
        "STR_DEVICE":"Periferic",
        "STR_LOCAL_VER":"Ventoy en local",
        "STR_DISK_VER":"Ventoy sul periferic",
        "STR_STATUS":"Estat - prèst",
        "STR_INSTALL":"Installacion",
        "STR_UPDATE":"Mesa a jorn",
        "STR_UPDATE_TIP":"La mesa a jorn es segura, los fichièrs ISO seràn pas modificats.#@Continhar ?",
        "STR_INSTALL_TIP":"Lo disc serà formatat e totas sas donadas seràn perdudas.#@Contunhar ?",
        "STR_INSTALL_TIP2":"Lo disc serà formatat e totas sas donadas seràn perdudas.#@Contunhar ? (confirmacion)",
        "STR_INSTALL_SUCCESS":"Felicitacion  !#@Ventoy es estat corrèctament installat sul periferic.",
        "STR_INSTALL_FAILED":"Una error s’es producha pendent l’installacion. Podètz tornar brancar lo periferic USB e tornar ensajar. Agachatz lo fichièr /var/log/ventoy.log per ne saber mai.",
        "STR_UPDATE_SUCCESS":"Felicitacions !#@Ventoy es estat corrèctament mes a jorn sul periferic.",
        "STR_UPDATE_FAILED":"Una error s’es producha pendent la mesa a jorn. Podètz tornar brancar lo periferic USB e tornar ensajar. Agachatz lo fichièr /var/log/ventoy.log per ne saber mai.",
        "STR_WAIT_PROCESS":"Una operacion es en cors, esperatz...",
        "STR_MENU_OPTION":"Opcion",
        "STR_MENU_SECURE_BOOT":"Secure Boot",
        "STR_MENU_PART_CFG":"Configuracion de particion",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Anullar",
        "STR_PRESERVE_SPACE":"Servar d’espaci a la fin del disc",
        "STR_SPACE_VAL_INVALID":"Valor invalida per l’espaci reservat",
        "STR_MENU_CLEAR":"Escafar Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy es estat corrèctament tirat del periferic.",
        "STR_CLEAR_FAILED":"Una error s’es producha pendent l’esfaçament de Ventoy del disc. Podètz tornar brancar l’USB e tornar ensajar. Vejatz /var/log/ventoy.log pels detalhs.",
        "STR_MENU_PART_STYLE":"Estil de particion",
        "STR_DISK_2TB_MBR_ERROR":"Pels disques de mai de 2 To seleccionatz GPT",
        "STR_SHOW_ALL_DEV":"Mostrar totes los periferics",
        "STR_PART_ALIGN_4KB":"Alinhar las particions sus 4 Ko",
        "STR_WEB_COMMUNICATION_ERR":"Error de comunicacion :",
        "STR_WEB_REMOTE_ABNORMAL":"Error de comunicacion : remote abnormal",
        "STR_WEB_REQUEST_TIMEOUT":"Error de comunicacion : requèsta tardièra",
        "STR_WEB_SERVICE_UNAVAILABLE":"Error de comunicacion : servici pas disponible",
        "STR_WEB_TOKEN_MISMATCH":"Estat del Daemon actualizat, tornatz ensajar mai tard.",
        "STR_WEB_SERVICE_BUSY":"Lo servici es ocupat, tornatz ensajar mai tard.",
        "STR_MENU_VTSI_CREATE":"Generar lo fichièr VTSI",
        "STR_VTSI_CREATE_TIP":"Aqueste còp cap d’escritura al disc, generacion del fichiet VTSI sonque#@Contunhar ?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI file created successfully!#@You can use Rufus(3.15+) to write it to the device so as to complete the installation of Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Fracàs de la creacion del fichièr VTSI.",
        "STR_MENU_PART_RESIZE":"Non-destructive Install",
        "STR_PART_RESIZE_TIP":"Ventoy will try non-destructive installation if possible. #@Continue?",
        "STR_PART_RESIZE_SUCCESS":"Congratulations!#@Ventoy non-destructive installation successfully finished.",
        "STR_PART_RESIZE_FAILED":"Non-destructive installation failed, Check /var/log/ventoy.log for details.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy non-destructive installation stopped because some conditions cannot be met. Check /var/log/ventoy.log for details.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"Donate",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"French (Français)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"vboucard, Silejonu, wiz64",
        
        "STR_ERROR":"Erreur",
        "STR_WARNING":"Avertissement",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Ce programme doit s'exécuter dans le bon répertoire !",
        "STR_INCORRECT_TREE_DIR":"Ne me lancez pas d'ici, veuillez télécharger le programme d'installation et relancez-le.",
        "STR_DEVICE":"Périphérique",
        "STR_LOCAL_VER":"Ventoy en local",
        "STR_DISK_VER":"Ventoy sur le périphérique",
        "STR_STATUS":"État - prêt",
        "STR_INSTALL":"Installation",
        "STR_UPDATE":"Mise à jour",
        "STR_UPDATE_TIP":"La mise à jour est sûre, les fichiers ISO ne seront pas modifiés.#@Continuer ?",
        "STR_INSTALL_TIP":"Le disque va être formaté et toutes ses données seront perdues.#@Continuer ?",
        "STR_INSTALL_TIP2":"Le disque va être formaté et toutes ses données seront perdues.#@Continuer ? (Confirmation)",
        "STR_INSTALL_SUCCESS":"Félicitations !#@Ventoy a été correctement installé sur le périphérique.",
        "STR_INSTALL_FAILED":"Une erreur est survenue durant l'installation. Vous pouvez rebrancher le périphérique et réessayer. Consultez le fichier /var/log/ventoy.log pour plus de détails.",
        "STR_UPDATE_SUCCESS":"Félicitations !#@Ventoy a été correctement mis à jour sur le périphérique.",
        "STR_UPDATE_FAILED":"Une erreur est survenue durant la mise à jour. Vous pouvez rebrancher le périphérique et réessayer. Consultez le fichier /var/log/ventoy.log pour plus de détails.",
        "STR_WAIT_PROCESS":"Une opération est en cours, veuillez patienter...",
        "STR_MENU_OPTION":"Option",
        "STR_MENU_SECURE_BOOT":"Secure Boot",
        "STR_MENU_PART_CFG":"Configuration des partitions",
        "STR_BTN_OK":"Valider",
        "STR_BTN_CANCEL":"Annuler",
        "STR_PRESERVE_SPACE":"Réservez de l'espace à la fin du disque",
        "STR_SPACE_VAL_INVALID":"Valeur de l'espace réservé invalide",
        "STR_MENU_CLEAR":"Effacer Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy a été retiré du périphérique avec succès.",
        "STR_CLEAR_FAILED":"Une erreur est survenue pendant la suppression de Ventoy. Vous pouvez rebrancher le périphérique et réessayer. Consultez le fichier /var/log/ventoy.log pour plus de détails.",
        "STR_MENU_PART_STYLE":"Type de partition",
        "STR_DISK_2TB_MBR_ERROR":"Sélectionnez le type GPT pour les disques de plus de 2 To",
        "STR_SHOW_ALL_DEV":"Afficher tous les périphériques",
        "STR_PART_ALIGN_4KB":"Alignez les partitions sur 4 Ko",
        "STR_WEB_COMMUNICATION_ERR":"Erreur de communication :",
        "STR_WEB_REMOTE_ABNORMAL":"Erreur de communication : anomalie sur le serveur",
        "STR_WEB_REQUEST_TIMEOUT":"Erreur de communication : délai d'attente dépassé",
        "STR_WEB_SERVICE_UNAVAILABLE":"Erreur de communication : service inaccessible",
        "STR_WEB_TOKEN_MISMATCH":"Statut du démon mis à jour, veuillez réessayer plus tard.",
        "STR_WEB_SERVICE_BUSY":"Service surchargé, veuillez réessayer plus tard.",
        "STR_MENU_VTSI_CREATE":"Générer un fichier VTSI",
        "STR_VTSI_CREATE_TIP":"N'écrira pas sur le périphérique cette fois-ci, mais générera uniquement un fichier VTSI#@Continuer ?",
        "STR_VTSI_CREATE_SUCCESS":"Fichier VTSI créé avec succès!#@Vous pouvez utiliser Rufus (3.15+) pour l'écrire sur le périphérique afin de terminer l'installation de Ventoy.",
        "STR_VTSI_CREATE_FAILED":"La création du fichier VTSI a échoué.",
        "STR_MENU_PART_RESIZE":"Installation non destructive",
        "STR_PART_RESIZE_TIP":"Si possible, Ventoy essaiera une installation non destructive. #@Continuer ?",
        "STR_PART_RESIZE_SUCCESS":"Félicitations !#@L'installation non destructive de Ventoy s'est terminée avec succès.",
        "STR_PART_RESIZE_FAILED":"L'installation non destructive a échoué, consultez /var/log/ventoy.log pour plus de détails.",
        "STR_PART_RESIZE_UNSUPPORTED":"L'installation non destructive de Ventoy s'est arrêtée car certaines conditions ne peuvent pas être remplies. Consultez /var/log/ventoy.log pour plus de détails.",
        "STR_INSTALL_YES_TIP1":"Attention : les données seront perdues !",
        "STR_INSTALL_YES_TIP2":"Veuillez saisir YES dans la zone de texte ci-dessous pour confirmer que vous souhaitez bien effectuer une nouvelle installation au lieu d'une mise à niveau.",
        "STR_PART_VENTOY_FS":"Système de fichiers pour la partition Ventoy",
        "STR_PART_FS":"Système de fichiers",
        "STR_PART_CLUSTER":"Taille de cluster",
        "STR_PART_CLUSTER_DEFAULT":"Valeur par défaut du système",
        "STR_DONATE":"Faire un don",
        "STR_4KN_UNSUPPORTED":"Actuellement, Ventoy ne prend pas en charge les appareils natifs 4K.",
        
        "STRXXX":""
    },
    {
        "name":"Czech (Čeština)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"vavanade, lev741",
        
        "STR_ERROR":"Chyba",
        "STR_WARNING":"Varování",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Spusťte Ventoy ve správném adresáři!",
        "STR_INCORRECT_TREE_DIR":"Nespouštějte mě zde, stáhněte si prosím vydaný instalační balík a spusťte v něm.",
        "STR_DEVICE":"Zařízení",
        "STR_LOCAL_VER":"Ventoy v Balíčku",
        "STR_DISK_VER":"Ventoy v Zařízení",
        "STR_STATUS":"Status - PŘIPRAVENO",
        "STR_INSTALL":"Instalovat",
        "STR_UPDATE":"Aktualizovat",
        "STR_UPDATE_TIP":"Operace aktualizace je bezpečná, ISO soubory nebudou změněny.#@Pokračovat?",
        "STR_INSTALL_TIP":"Disk bude zformátován a všechna data budou ztracena!#@Pokračovat?",
        "STR_INSTALL_TIP2":"Disk bude zformátován a všechna data budou ztracena!#@Pokračovat? (druhé potvrzení)",
        "STR_INSTALL_SUCCESS":"Gratulujeme!#@Ventoy byla na zařízení úspěšně nainstalována.",
        "STR_INSTALL_FAILED":"V průběhu instalace se vyskytla chyba. Můžete vyjmout a znovu zastrčit USB zařízení a zkusit to znovu. Pro podrobnosti se podívejte do souboru /var/log/ventoy.log.",
        "STR_UPDATE_SUCCESS":"Gratulujeme!#@Ventoy byla na zařízení úspěšně aktualizována.",
        "STR_UPDATE_FAILED":"V průběhu aktualizace se vyskytla chyba. Můžete vyjmout a znovu zastrčit USB zařízení a zkusit to znovu. Pro podrobnosti se podívejte do souboru /var/log/ventoy.log.",
        "STR_WAIT_PROCESS":"Vlákno běží, prosíme vyčkejte...",
        "STR_MENU_OPTION":"Možnosti",
        "STR_MENU_SECURE_BOOT":"Podpora Secure Boot",
        "STR_MENU_PART_CFG":"Konfigurace Oddílu",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Zrušit",
        "STR_PRESERVE_SPACE":"Zachovejte místo na konci disku",
        "STR_SPACE_VAL_INVALID":"Neplatná hodnota pro vyhrazený prostor",
        "STR_MENU_CLEAR":"Odstranit Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy byl ze zařízení úspěšne odstraněn.",
        "STR_CLEAR_FAILED":"Nastala chyba při odstraňovaní Ventoy z disku. Múžete USB vytáhnout a znova zastrčit a vyskoušet to znovu. Pro podrobnosti se podívejte do souboru /var/log/ventoy.log",
        "STR_MENU_PART_STYLE":"Styl Oddílu",
        "STR_DISK_2TB_MBR_ERROR":"Vyberte GPT pro disk větší než 2TB",
        "STR_SHOW_ALL_DEV":"Zobrazit všechna zařízení",
        "STR_PART_ALIGN_4KB":"Zarovnat oddíly s velikostí 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Chyba komunikace:",
        "STR_WEB_REMOTE_ABNORMAL":"Chyba komunikace: remote abnormal",
        "STR_WEB_REQUEST_TIMEOUT":"Chyba komunikace: Request timed out",
        "STR_WEB_SERVICE_UNAVAILABLE":"Chyba komunikace: Service Unavailable",
        "STR_WEB_TOKEN_MISMATCH":"Daemon status byl aktualizován, prosím vyskoušejte to později.",
        "STR_WEB_SERVICE_BUSY":"Služba je zaneprázdnená, prosím vyskoušejte to později.",
        "STR_MENU_VTSI_CREATE":"Vygenerovat VTSI Soubor",
        "STR_VTSI_CREATE_TIP":"Tentokrát se do zařízení nebude zapisovat, jenom se vygeneruje VTSI soubor#@Pokračovat?",     
        "STR_VTSI_CREATE_SUCCESS":"VTSI soubor úspěšne vytvořen!#@Múžete použit Rufus(3.15+) na jeho zapsání do zařízení pro dokončení instalace Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Chyba při vytváření VTSI souboru.",
        "STR_MENU_PART_RESIZE":"Nedestruktivní Instalace",
        "STR_PART_RESIZE_TIP":"Ventoy se pokusí o nedestruktivní instalaci. #@Pokračovat?",
        "STR_PART_RESIZE_SUCCESS":"Gratulujeme!#@Nedestruktivní instalace Ventoy byla úspěšně dokončena.",
        "STR_PART_RESIZE_FAILED":"Nedestruktivní instalace selhala. Pro podrobnosti se podívejte do souboru /var/log/ventoy.log.",
        "STR_PART_RESIZE_UNSUPPORTED":"Nedestruktivní instalace Ventoy selhala, protože některé podmínky pro ni nemohly být splněny. Pro podrobnosti se podívejte do souboru /var/log/ventoy.log.",
        "STR_INSTALL_YES_TIP1":"Varování: Data budou ztracena!",
        "STR_INSTALL_YES_TIP2":"Do níže uvedeného textového pole zadejte YES a potvrďte, že skutečně chcete provést novou instalaci namísto aktualizace.",
        "STR_PART_VENTOY_FS":"Souborový systém pro Ventoy Partition",
        "STR_PART_FS":"Souborový systém",
        "STR_PART_CLUSTER":"Velikost clusteru",
        "STR_PART_CLUSTER_DEFAULT":"Výchozí hodnota systému",
        "STR_DONATE":"Darovat",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Slovak (Slovenčina)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"reggiiie/jose1711",

        "STR_ERR":"Chyba",
        "STR_WARNING":"Varovanie",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Spustite, prosím, v správnom adresári!",
        "STR_INCORRECT_TREE_DIR":"Nespúšťajte ma tu, stiahnite si, prosím, vydaný inštalačný balík a spustite ma v ňom.",
        "STR_DEVICE":"Zariadenie",
        "STR_LOCAL_VER":"Ventoy v balíčku",
        "STR_DISK_VER":"Ventoy v zariadení",
        "STR_STATUS":"Stav - PRIPRAVENÝ",
        "STR_INSTALL":"Inštalovať",
        "STR_UPDATE":"Aktualizovať",
        "STR_UPDATE_TIP":"Aktualizácia je bezpečná, ISO súbory zostanú nezmenené.#@Pokračovať?",
        "STR_INSTALL_TIP":"Disk bude formátovaný a všetky údaje sa stratia!#@Pokračovať?",
        "STR_INSTALL_TIP2":"Disk bude formátovaný a všetky údaje sa stratia!#@Pokračovať? (dvojitá kontrola)",
        "STR_INSTALL_SUCCESS":"Gratulujeme!#@Ventoy bol úspešne nainštalovaný na zariadenie.",
        "STR_INSTALL_FAILED":"V priebehu inštalácie sa vyskytla chyba. Môžete USB vytiahnuť, znova ho zasunúť a skúsiť to znovu. Podrobnosti nájdete v súbore /var/log/ventoy.log. Ak sa chyba stále opakuje, skontrolujte, prosím, FAQ na oficiálnej stránke.",
        "STR_UPDATE_SUCCESS":"Gratulujeme!#@Ventoy na zariadení bol úspešne aktualizovaný.",
        "STR_UPDATE_FAILED":"V priebehu aktualizácie sa vyskytla chyba. Môžete USB vytiahnuť, znova ho zasunúť a skúsiť to znovu. Podrobnosti nájdete v súbore /var/log/ventoy.log. Ak sa chyba stále opakuje, skontrolujte, prosím, FAQ na oficiálnej stránke.",
        "STR_MENU_OPTION":"Možnosti",
        "STR_MENU_SECURE_BOOT":"Podpora pre Secure Boot",
        "STR_MENU_PART_CFG":"Konfigurácia oddielu",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Zrušiť",
        "STR_PRESERVE_SPACE":"Zachovať nejaké miesto na konci disku",
        "STR_SPACE_VAL_INVALID":"Neplatná hodnota pre vyhradený priestor",
        "STR_MENU_CLEAR":"Vymazať Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy bol zo zariadenia úspešne odstránený.",
        "STR_CLEAR_FAILED":"Nastala chyba pri odstraňovaní Ventoy z disku. Môžete USB vytiahnuť, znova ho zasunúť a skúsiť to znovu. Podrobnosti nájdete v súbore /var/log/ventoy.log.",
        "STR_MENU_PART_STYLE":"Štýl oddielu",
        "STR_DISK_2TB_MBR_ERROR":"Prosím vyberte GPT pre disk väčší ako 2 TB",
        "STR_SHOW_ALL_DEV":"Zobraziť všetky zariadenia",
        "STR_PART_ALIGN_4KB":"Zarovnať oddiely na 4 KB",
        "STR_WEB_COMMUNICATION_ERR":"Chyba komunikácie:",
        "STR_WEB_REMOTE_ABNORMAL":"Chyba komunikácie: Chyba vzdialeného hostiteľa",
        "STR_WEB_REQUEST_TIMEOUT":"Chyba komunikácie: Požiadavka vypršala",
        "STR_WEB_SERVICE_UNAVAILABLE":"Chyba komunikácie: Služba je nedostupná",
        "STR_WEB_TOKEN_MISMATCH":"Stav démona bol aktualizovaný, skúste to, prosím, neskôr.",
        "STR_WEB_SERVICE_BUSY":"Služba je zaneprázdnená, skúste to, prosím, neskôr.",
        "STR_MENU_VTSI_CREATE":"Vygenerovať VTSI Súbor",
        "STR_VTSI_CREATE_TIP":"Tentokrát sa nebude na zariadenie zapisovať, len sa vygeneruje VTSI súbor#@Pokračovať?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI súbor bol úspešne vytvorený!#@Môžete použiť Rufus(3.15+) na jeho zapísanie na zariadenie, aby ste mohli dokončiť inštaláciu Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Chyba pri vytváraní VTSI súboru.",
        "STR_MENU_PART_RESIZE":"Nedeštruktívna inštalácia",
        "STR_PART_RESIZE_TIP":"Ventoy sa pokúsi o nedeštruktívnu inštaláciu. #@Pokračovať?",
        "STR_PART_RESIZE_SUCCESS":"Gratulujeme!#@Nedeštruktívna inštalácia Ventoy bola úspešne dokončená.",
        "STR_PART_RESIZE_FAILED":"Nedeštruktívna inštalácia zlyhala. Podrobnosti nájdete v súbore /var/log/ventoy.log.",
        "STR_PART_RESIZE_UNSUPPORTED":"Nedeštruktívna inštalácia Ventoy zlyhala, lebo niektoré podmienky nemohli byť splnené. Podrobnosti nájdete v súbore /var/log/ventoy.log.",
        "STR_INSTALL_YES_TIP1":"Varovanie: Dôjde k strate údajov!",
        "STR_INSTALL_YES_TIP2":"Prosím, zadajte YES v textovom poli nižšie, čím potvrdíte, že naozaj chcete vykonať novú inštaláciu miesto aktualizácie.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"Darovať",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Spanish (Latinoamérica)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"MELERIX, independent-arg",
        
        "STR_ERROR":"Error",
        "STR_WARNING":"Advertencia",
        "STR_INFO":"Información",
        "STR_INCORRECT_DIR":"¡Por favor, ejecuta el programa bajo el directorio correcto!",
        "STR_INCORRECT_TREE_DIR":"No me ejecutes aquí. Por favor descarga el paquete de instalación oficial y ejecútalo desde allí.",
        "STR_DEVICE":"Dispositivo",
        "STR_LOCAL_VER":"Ventoy En Paquete",
        "STR_DISK_VER":"Ventoy En Dispositivo",
        "STR_STATUS":"Estado - LISTO",
        "STR_INSTALL":"Instalar",
        "STR_UPDATE":"Actualizar",
        "STR_UPDATE_TIP":"La operación de actualización es segura, Los archivo ISO no se modificarán.#@¿Continuar?",
        "STR_INSTALL_TIP":"El dispositivo será formateado y todos los datos se perderán.#@¿Continuar?",
        "STR_INSTALL_TIP2":"El dispositivo será formateado y todos los datos se perderán.#@¿Continuar? (Doble Comprobación)",
        "STR_INSTALL_SUCCESS":"¡Felicitaciones!#@Ventoy ha sido instalado exitosamente en el dispositivo.",
        "STR_INSTALL_FAILED":"Ocurrió un error durante la instalación. Puedes reinsertar el dispositivo USB e intentar de nuevo. Comprueba /var/log/ventoy.log para más detalles. Si siempre falla, consulta las preguntas frecuentes (FAQ) en el sitio web oficial.",
        "STR_UPDATE_SUCCESS":"¡Felicitaciones!#@Ventoy ha sido actualizado exitosamente en el dispositivo.",
        "STR_UPDATE_FAILED":"Ocurrió un error durante la actualización. Puedes reinsertar el dispositivo USB e intentar de nuevo. Comprueba /var/log/ventoy.log para más detalles. Si siempre falla, consulta las preguntas frecuentes (FAQ) en el sitio web oficial.",
        "STR_WAIT_PROCESS":"Un hilo del sistema está ejecutándose, por favor espera...",
        "STR_MENU_OPTION":"Opción",
        "STR_MENU_SECURE_BOOT":"Soporte de arranque seguro",
        "STR_MENU_PART_CFG":"Configuración de partición",
        "STR_BTN_OK":"Aceptar",
        "STR_BTN_CANCEL":"Cancelar",
        "STR_PRESERVE_SPACE":"Preservar algo de espacio al final del dispositivo",
        "STR_SPACE_VAL_INVALID":"Valor inválido para el espacio reservado",
        "STR_MENU_CLEAR":"Limpiar Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy ha sido eliminado exitosamente del dispositivo.",
        "STR_CLEAR_FAILED":"Ocurrió un error al eliminar Ventoy del dispositivo. Puedes reinsertar el dispositivo USB e intentar de nuevo. Comprueba /var/log/ventoy.log para más detalles.",
        "STR_MENU_PART_STYLE":"Estilo de partición",
        "STR_DISK_2TB_MBR_ERROR":"Por favor selecciona GPT para dispositivos de más de 2TB",
        "STR_SHOW_ALL_DEV":"Mostrar todos los dispositivos",
        "STR_PART_ALIGN_4KB":"Alinear particiones a 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Error de comunicación:",
        "STR_WEB_REMOTE_ABNORMAL":"Error de comunicación: anomalía remoto",
        "STR_WEB_REQUEST_TIMEOUT":"Error de comunicación: tiempo de espera agotado",
        "STR_WEB_SERVICE_UNAVAILABLE":"Error de comunicación: servicio no disponible",
        "STR_WEB_TOKEN_MISMATCH":" Estado del daemon actualizado, por favor reintenta más tarde.",
        "STR_WEB_SERVICE_BUSY":" El servicio está ocupado, por favor reintenta más tarde.",
        "STR_MENU_VTSI_CREATE":"Generar archivo VTSI",
        "STR_VTSI_CREATE_TIP":"Esta acción no escribirá en el dispositivo, sólo generará un archivo VTSI.#@¿Continuar?",
        "STR_VTSI_CREATE_SUCCESS":"¡Archivo VTSI creado exitosamente!#@Puedes usar Rufus(3.15+) para escribirlo al dispositivo a fin de completar la instalación de Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Error al crear el archivo VTSI.",
        "STR_MENU_PART_RESIZE":"Instalación no destructiva",
        "STR_PART_RESIZE_TIP":"Ventoy intentará una instalación no destructiva si es posible. #@¿Continuar?",
        "STR_PART_RESIZE_SUCCESS":"¡Felicitaciones!#@La instalación no destructiva de Ventoy a finalizado exitosamente.",
        "STR_PART_RESIZE_FAILED":"Instalación no destructiva fallida, Comprueba /var/log/ventoy.log para detalles.",
        "STR_PART_RESIZE_UNSUPPORTED":"Instalación no destructiva de Ventoy detenida porque algunas condiciones no se pueden cumplir. Comprueba /var/log/ventoy.log para más detalles.",
        "STR_INSTALL_YES_TIP1":"Advertencia: ¡Los datos se perderán!",
        "STR_INSTALL_YES_TIP2":"Por favor ingresa YES en el cuadro de texto a continuación para confirmar que realmente quieres realizar una instalación nueva en vez de actualizar.",
        "STR_PART_VENTOY_FS":"Sistema de archivos para partición de Ventoy",
        "STR_PART_FS":"Sistema de archivos",
        "STR_PART_CLUSTER":"Tamaño del clúster",
        "STR_PART_CLUSTER_DEFAULT":"Valor predeterminado del sistema",
        "STR_DONATE":"Donar",
        "STR_4KN_UNSUPPORTED":"Actualmente Ventoy no es compatible con dispositivos nativos 4K.",
        
        "STRXXX":""
    },
    {
        "name":"Spanish (España)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Carlos Sánchez, MELERIX, shulkalala",
        
        "STR_ERROR":"Error",
        "STR_WARNING":"Advertencia",
        "STR_INFO":"Información",
        "STR_INCORRECT_DIR":"¡Por favor, ejecute Ventoy desde el directorio correcto!",
        "STR_INCORRECT_TREE_DIR":"No inicie Ventoy aquí, descargue el paquete de instalación publicado y ejecútelo allí.",
        "STR_DEVICE":"Dispositivo",
        "STR_LOCAL_VER":"Ventoy en paquete",
        "STR_DISK_VER":"Ventoy en dispositivo",
        "STR_STATUS":"Estado - LISTO",
        "STR_INSTALL":"Instalar",
        "STR_UPDATE":"Actualizar",
        "STR_UPDATE_TIP":"La operación de actualización es segura. Los archivos ISO no se modificarán.#@¿Continuar?",
        "STR_INSTALL_TIP":"Se va a formatear el dispositivo y a perder todos sus datos.#@¿Continuar?",
        "STR_INSTALL_TIP2":"Se va a formatear el dispositivo y a perder todos sus datos.#@¿Continuar? (Segunda confirmación)",
        "STR_INSTALL_SUCCESS":"¡Enhorabuena!#@Ventoy se ha instalado correctamente en el dispositivo.",
        "STR_INSTALL_FAILED":"Ocurrió un error durante la instalación. Puede reinsertar el dispositivo USB e intentarlo de nuevo.#@Compruebe el archivo /var/log/ventoy.log para más información.#@Si la instalación no deja de fallar, consulte el apartado de preguntas frecuentes en el sitio web oficial.",
        "STR_UPDATE_SUCCESS":"¡Enhorabuena!#@Ventoy se ha actualizado correctamente en el dispositivo.",
        "STR_UPDATE_FAILED":"Ocurrió un error durante la actualización. Puede reinsertar el dispositivo USB e intentarlo de nuevo.#@Compruebe el archivo /var/log/ventoy.log para más información.#@Si la instalación no deja de fallar, consulte el apartado de preguntas frecuentes en el sitio web oficial.",
        "STR_WAIT_PROCESS":"Un hilo está ejecutándose, por favor espere...",
        "STR_MENU_OPTION":"Opción",
        "STR_MENU_SECURE_BOOT":"Soporte de arranque seguro",
        "STR_MENU_PART_CFG":"Configuración de particiones",
        "STR_BTN_OK":"Aceptar",
        "STR_BTN_CANCEL":"Cancelar",
        "STR_PRESERVE_SPACE":"Reservar espacio de almacenamiento al final del dispositivo",
        "STR_SPACE_VAL_INVALID":"Valor inválido para el espacio reservado",
        "STR_MENU_CLEAR":"Limpiar Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy se ha eliminado correctamente del dispositivo.",
        "STR_CLEAR_FAILED":"Ocurrió un error al eliminar Ventoy del dispositivo. Puede reinsertar el dispositivo USB e intentarlo de nuevo.#@Compruebe el archivo /var/log/ventoy.log para más información.",
        "STR_MENU_PART_STYLE":"Estilo de partición",
        "STR_DISK_2TB_MBR_ERROR":"Por favor, seleccione GPT para dispositivos con capacidad mayor de 2 TB",
        "STR_SHOW_ALL_DEV":"Mostrar todos los dispositivos",
        "STR_PART_ALIGN_4KB":"Alinear particiones a 4 KB",
        "STR_WEB_COMMUNICATION_ERR":"Error de comunicación:",
        "STR_WEB_REMOTE_ABNORMAL":"Error de comunicación: anormal remoto",
        "STR_WEB_REQUEST_TIMEOUT":"Error de comunicación: tiempo de espera agotado",
        "STR_WEB_SERVICE_UNAVAILABLE":"Error de comunicación: servicio no disponible",
        "STR_WEB_TOKEN_MISMATCH":"Se ha actualizado el estado del servicio, por favor inténtelo de nuevo más tarde.",
        "STR_WEB_SERVICE_BUSY":" El servicio está ocupado, por favor inténtelo de nuevo más tarde.",
        "STR_MENU_VTSI_CREATE":"Generar archivo VTSI",
        "STR_VTSI_CREATE_TIP":"Esta vez no se escribirá en el dispositivo, tan sólo se generará un archivo VTSI#@¿Continuar?",
        "STR_VTSI_CREATE_SUCCESS":"¡El archivo VTSI se ha creado correctamente!#@Puede usar Rufus (3.15+) para escribirlo al dispositivo y completar la instalación de Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Se ha producido un error al intentar crear el archivo VTSI.",
        "STR_MENU_PART_RESIZE":"Instalación no destructiva",
        "STR_PART_RESIZE_TIP":"Ventoy intentará realizar una instalación no destructiva.#@¿Continuar?",
        "STR_PART_RESIZE_SUCCESS":"¡Enhorabuena!#@La instalación no destructiva de Ventoy se ha completado correctamente.",
        "STR_PART_RESIZE_FAILED":"La instalación no destructiva ha fallado.#@Compruebe el archivo /var/log/ventoy.log para más información.",
        "STR_PART_RESIZE_UNSUPPORTED":"La instalación no destructiva de Ventoy se ha detenido porque no se han cumplido todos los prerrequisitos.#@Compruebe el archivo /var/log/ventoy.log para más información.",
        "STR_INSTALL_YES_TIP1":"Advertencia: ¡Los datos se perderán!",
        "STR_INSTALL_YES_TIP2":"Por favor, escriba YES en el cuadro de texto a continuación para confirmar que realmente quiere realizar una nueva instalación de Ventoy en vez de actualizarlo.",
        "STR_PART_VENTOY_FS":"Sistema de archivos para partición de Ventoy",
        "STR_PART_FS":"Sistema de archivos",
        "STR_PART_CLUSTER":"Tamaño de cluster",
        "STR_PART_CLUSTER_DEFAULT":"Valor predeterminado del sistema",
        "STR_DONATE":"Donar",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Russian (Русский)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Dmitry Yerokhin, BL4CKH47H4CK3R, Teraskull, thehugonote, wiz64, dmitrygribenchuk, Темак",
        
        "STR_ERROR":"Ошибка",
        "STR_WARNING":"Внимание",
        "STR_INFO":"Информация",
        "STR_INCORRECT_DIR":"Запустите в другой папке!",
        "STR_INCORRECT_TREE_DIR":"Не запускайте приложение здесь, загрузите установочный пакет и запустите его в другом месте.",
        "STR_DEVICE":"Устройство",
        "STR_LOCAL_VER":"Ventoy в пакете",
        "STR_DISK_VER":"Ventoy на накопителе",
        "STR_STATUS":"Состояние — ГОТОВ",
        "STR_INSTALL":"Установить",
        "STR_UPDATE":"Обновить",
        "STR_UPDATE_TIP":"Обновление безопасно, ISO-образы останутся без изменений.#@Продолжить?",
        "STR_INSTALL_TIP":"Это действие отформатирует накопитель, все данные на нём будут уничтожены.#@Продолжить?",
        "STR_INSTALL_TIP2":"Это действие отформатирует накопитель, все данные на нём будут уничтожены.#@Вы действительно хотите продолжить?",
        "STR_INSTALL_SUCCESS":"Поздравляем!#@Установка Ventoy на устройстве успешно выполнена.",
        "STR_INSTALL_FAILED":"Ошибка при установке. Переподключите USB-накопитель и попробуйте снова. См. файл /var/log/ventoy.log с подробностями. Если ошибка не исчезает, см. «FAQ» на официальном сайте.",
        "STR_UPDATE_SUCCESS":"Поздравляем!#@Обновление Ventoy на устройстве успешно выполнено.",
        "STR_UPDATE_FAILED":"Ошибка при обновлении Ventoy. Переподключите USB-накопитель и попробуйте снова. См. файл /var/log/ventoy.log с подробностями. Если ошибка не исчезает, см. «FAQ» на официальном сайте.",
        "STR_WAIT_PROCESS":"Действие запущено, подождите...",
        "STR_MENU_OPTION":"Настройки",
        "STR_MENU_SECURE_BOOT":"Поддержка Secure Boot",
        "STR_MENU_PART_CFG":"Дополнительный раздел",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Отмена",
        "STR_PRESERVE_SPACE":"Создать дополнительный раздел в конце накопителя",
        "STR_SPACE_VAL_INVALID":"Недействительное значение размера раздела",
        "STR_MENU_CLEAR":"Удалить Ventoy",
        "STR_CLEAR_SUCCESS":"Удаление Ventoy с устройства успешно выполнено.",
        "STR_CLEAR_FAILED":"Ошибка при удалении Ventoy. Переподключите USB-накопитель и попробуйте снова. См. файл /var/log/ventoy.log с подробностями.",
        "STR_MENU_PART_STYLE":"Стиль разметки разделов",
        "STR_DISK_2TB_MBR_ERROR":"Для накопителей объёмом более 2 ТБ выберите GPT",
        "STR_SHOW_ALL_DEV":"Показывать все устройства",
        "STR_PART_ALIGN_4KB":"Выравнивать разделы с кластерами по 4 КБ",
        "STR_WEB_COMMUNICATION_ERR":"Ошибка связи:",
        "STR_WEB_REMOTE_ABNORMAL":"Ошибка связи: удалённое соединение недействительно",
        "STR_WEB_REQUEST_TIMEOUT":"Ошибка связи: истекло время ожидания запроса",
        "STR_WEB_SERVICE_UNAVAILABLE":"Ошибка связи: служба недоступна",
        "STR_WEB_TOKEN_MISMATCH":"Состояние службы обновлено, повторите попытку позже.",
        "STR_WEB_SERVICE_BUSY":"Служба занята, повторите попытку позже.",
        "STR_MENU_VTSI_CREATE":"Создать файл .vtsi",
        "STR_VTSI_CREATE_TIP":"Вместо установки Ventoy на накопитель будет создан только vtsi-файл.#@Продолжить?",
        "STR_VTSI_CREATE_SUCCESS":"Файл .vtsi успешно создан.#@Вы можете записать его на накопитель с помощью приложения Rufus (v3.15+), чтобы завершить установку Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Ошибка создания vtsi-файла.",
        "STR_MENU_PART_RESIZE":"Неразрушающая установка",
        "STR_PART_RESIZE_TIP":"Ventoy попробует выполнить неразрушающую установку. #@Продолжить?",
        "STR_PART_RESIZE_SUCCESS":"Поздравляем!#@Неразрушающая установка Ventoy успешно выполнена.",
        "STR_PART_RESIZE_FAILED":"Не удалось выполнить неразрушающую установку, см. файл /var/log/ventoy.log с подробностями.",
        "STR_PART_RESIZE_UNSUPPORTED":"Неразрушающая установка Ventoy остановлена из-за несоответствия некоторым требованиям. См. файл /var/log/ventoy.log с подробностями.",
        "STR_INSTALL_YES_TIP1":"ВНИМАНИЕ: все данные будут потеряны!",
        "STR_INSTALL_YES_TIP2":"Введите «YES» в текстовом поле ниже, чтобы подтвердить, что Вы действительно хотите выполнить новую установку, а не обновление.",
        "STR_PART_VENTOY_FS":"Файловая система для раздела Ventoy",
        "STR_PART_FS":"Тип:",
        "STR_PART_CLUSTER":"Размер кластера:",
        "STR_PART_CLUSTER_DEFAULT":"Системное значение по умолчанию",
        "STR_DONATE":"Пожертвовать",
        "STR_4KN_UNSUPPORTED":"В настоящее время Ventoy не поддерживает родные устройства 4K.",
        
        "STRXXX":""
    },
    {
        "name":"Bengali (বাংলা)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"BL4CKH47H4CK3R, wiz64",
        
        "STR_ERROR":"ত্রুটি",
        "STR_WARNING":"সতর্কতা",
        "STR_INFO":"তথ্য",
        "STR_INCORRECT_DIR":"দয়া করে সঠিক ডিরেক্টরিতে চালান!",
        "STR_INCORRECT_TREE_DIR":"আমাকে এখানে চালাবেন না, দয়া করে প্রকাশিত ইনস্টল প্যাকেজটি ডাউনলোড করুন এবং সেখানে চালান।",
        "STR_DEVICE":"ডিভাইস",
        "STR_LOCAL_VER":"Ventoy প্যাকেজে আছে",
        "STR_DISK_VER":"Ventoy ডিভাইসে আছে",
        "STR_STATUS":"স্থিতি - প্রস্তুত",
        "STR_INSTALL":"ইনস্টল করুন",
        "STR_UPDATE":"আপডেট করুন",
        "STR_UPDATE_TIP":"আপগ্রেড অপারেশন নিরাপদ, ISO ফাইলগুলি অপরিবর্তিত থাকবে।#@চালিয়ে যাবেন?",
        "STR_INSTALL_TIP":"ডিস্কটি ফর্ম্যাট করা হবে এবং সমস্ত ডেটা হারিয়ে যাবে।#@চালিয়ে যাবেন?",
        "STR_INSTALL_TIP2":"ডিস্কটি ফর্ম্যাট করা হবে এবং সমস্ত ডেটা হারিয়ে যাবে।#@চালিয়ে যাবেন? (পুনঃনিরীক্ষণ)",
        "STR_INSTALL_SUCCESS":"অভিনন্দন! #@Ventoy সফলভাবে ডিভাইসে ইনস্টল করা হয়েছে।",
        "STR_INSTALL_FAILED":"ইনস্টলেশন চলাকালীন একটি ত্রুটি ঘটেছে। আপনি USB পুনরায় প্লাগ করতে পারেন এবং আবার চেষ্টা করতে পারেন। বিস্তারিত জানার জন্য /var/log/ventoy.log পরীক্ষা করুন।",
        "STR_UPDATE_SUCCESS":"অভিনন্দন! #@Ventoy সফলভাবে ডিভাইসে আপডেট করা হয়েছে।",
        "STR_UPDATE_FAILED":"আপডেটের সময় একটি ত্রুটি ঘটেছে। আপনি USB পুনরায় প্লাগ করতে পারেন এবং আবার চেষ্টা করতে পারেন। বিস্তারিত জানার জন্য /var/log/ventoy.log পরীক্ষা করুন।",
        "STR_WAIT_PROCESS":"একটি থ্রেড চলছে, দয়া করে অপেক্ষা করুন ...",
        "STR_MENU_OPTION":"অপসন",
        "STR_MENU_SECURE_BOOT":"নিরাপদ বুট",
        "STR_MENU_PART_CFG":"পার্টিশন কনফিগারেশন",
        "STR_BTN_OK":"ঠিক আছে",
        "STR_BTN_CANCEL":"বাতিল",
        "STR_PRESERVE_SPACE":"ডিস্কের নীচে কিছু স্থান সংরক্ষণ করুন",
        "STR_SPACE_VAL_INVALID":"সংরক্ষিত জায়গার জন্য অবৈধ মান",
        "STR_MENU_CLEAR":"Ventoy সাফ",
        "STR_CLEAR_SUCCESS":"Ventoy সফলভাবে ডিভাইস থেকে সরানো হয়েছে।",
        "STR_CLEAR_FAILED":"ডিস্ক থেকে Ventoy সাফ করার সময় একটি ত্রুটি ঘটেছে। আপনি USB পুনরায় প্লাগ করতে পারেন এবং আবার চেষ্টা করতে পারেন। বিস্তারিত জানার জন্য /var/log/ventoy.log পরীক্ষা করুন।",
        "STR_MENU_PART_STYLE":"পার্টিশন স্টাইল",
        "STR_DISK_2TB_MBR_ERROR":"2TB এর বেশি ডিস্কের জন্য দয়া করে GPT নির্বাচন করুন",
        "STR_SHOW_ALL_DEV":"সমস্ত ডিভাইস দেখান",
        "STR_PART_ALIGN_4KB":"4KB দিয়ে পার্টিশন সারিবদ্ধ করুন",
        "STR_WEB_COMMUNICATION_ERR":"যোগাযোগে ত্রুটি:",
        "STR_WEB_REMOTE_ABNORMAL":"যোগাযোগ ত্রুটি: দূরবর্তী অস্বাভাবিক",
        "STR_WEB_REQUEST_TIMEOUT":"যোগাযোগ ত্রুটি: অনুরোধের সময় শেষ হয়েছে৷",
        "STR_WEB_SERVICE_UNAVAILABLE":"যোগাযোগ ত্রুটি: পরিষেবা অনুপলব্ধ৷",
        "STR_WEB_TOKEN_MISMATCH":"ডেমন স্ট্যাটাস আপডেট হয়েছে, অনুগ্রহ করে পরে আবার চেষ্টা করুন।",
        "STR_WEB_SERVICE_BUSY":"পরিষেবা ব্যস্ত, পরে আবার চেষ্টা করুন।",
        "STR_MENU_VTSI_CREATE":"VTSI ফাইল তৈরি করুন",
        "STR_VTSI_CREATE_TIP":"এই সময় ডিভাইসে লিখবেন না, তবে শুধুমাত্র একটি VTSI ফাইল তৈরি করবেন#@চালিয়ে যান?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI ফাইলটি সফলভাবে তৈরি হয়েছে!#@ আপনি Rufus(3.15+) ব্যবহার করে এটিকে ডিভাইসে লিখতে পারেন যাতে Ventoy-এর ইনস্টলেশন সম্পূর্ণ করা যায়।",
        "STR_VTSI_CREATE_FAILED":"VTSI ফাইল তৈরি করা ব্যর্থ হয়েছে৷",
        "STR_MENU_PART_RESIZE":"অ-ধ্বংসাত্মক ইনস্টল",
        "STR_PART_RESIZE_TIP":"Ventoy সম্ভব হলে অ-ধ্বংসাত্মক ইনস্টলেশন চেষ্টা করবে। #@চালিয়ে যান?",
        "STR_PART_RESIZE_SUCCESS":"অভিনন্দন!#@Ventoy অ-ধ্বংসাত্মক ইনস্টলেশন সফলভাবে শেষ হয়েছে।",
        "STR_PART_RESIZE_FAILED":"অ-ধ্বংসাত্মক ইনস্টলেশন ব্যর্থ হয়েছে, বিস্তারিত জানার জন্য /var/log/ventoy.log চেক করুন।",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy অ-ধ্বংসাত্মক ইনস্টলেশন বন্ধ করা হয়েছে কারণ কিছু শর্ত পূরণ করা যাবে না। বিস্তারিত জানার জন্য /var/log/ventoy.log দেখুন।",
        "STR_INSTALL_YES_TIP1":"সতর্কতা: ডেটা হারিয়ে যাবে!",
        "STR_INSTALL_YES_TIP2":"আপনি আপগ্রেড করার পরিবর্তে একটি নতুন ইনস্টল করতে চান তা নিশ্চিত করতে দয়া করে নীচের পাঠ্য বাক্সে Yes লিখুন৷।",
        "STR_PART_VENTOY_FS":"Ventoy পার্টিশনের জন্য ফাইল সিস্টেম",
        "STR_PART_FS":"ফাইল সিস্টেম",
        "STR_PART_CLUSTER":"ক্লাস্টারের আকার",
        "STR_PART_CLUSTER_DEFAULT":"সিস্টেমের ডিফল্ট মান",
        "STR_DONATE":"দান করুন",
        "STR_4KN_UNSUPPORTED":"বর্তমানে Ventoy 4K নেটিভ ডিভাইস সমর্থন করে না।",
        
        "STRXXX":""
    },
    {
        "name":"Hindi (हिन्दी)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"BL4CKH47H4CK3R, wiz64",
        
        "STR_ERROR":"त्रुटि",
        "STR_WARNING":"चेतावनी",
        "STR_INFO":"जानकारी",
        "STR_INCORRECT_DIR":"कृपया सही निर्देशिका के तहत चलाएं!",
        "STR_INCORRECT_TREE_DIR":"मुझे यहां न चलाएं, कृपया जारी किए गए इंस्टॉल पैकेज को डाउनलोड करें, और वहां चलाएं।",
        "STR_DEVICE":"डिवाइस",
        "STR_LOCAL_VER":"पैकेज में Ventoy",
        "STR_DISK_VER":"डिवाइस में Ventoy",
        "STR_STATUS":"स्थिति - तैयार",
        "STR_INSTALL":"इंस्टॉल",
        "STR_UPDATE":"अपडेट करें",
        "STR_UPDATE_TIP":"नवीनीकरण ऑपरेशन सुरक्षित है, ISO फाइल अपरिवर्तित रहेंगी।#@जारी रखें?",
        "STR_INSTALL_TIP":"डिस्क को स्वरूपित किया जाएगा और सभी डेटा खो जाएगा।#@जारी रखें?",
        "STR_INSTALL_TIP2":"डिस्क को स्वरूपित किया जाएगा और सभी डेटा खो जाएगा।#@जारी रखें? (दोहरी जाँच)",
        "STR_INSTALL_SUCCESS":"बधाई! #@Ventoy डिवाइस में सफलतापूर्वक स्थापित किया गया है।",
        "STR_INSTALL_FAILED":"स्थापना के दौरान एक त्रुटि हुई। आप USB को पुन: स्थापित कर सकते हैं और पुनः प्रयास कर सकते हैं। विस्तार के लिए /var/log/ventoy.log की जाँच करें।",
        "STR_UPDATE_SUCCESS":"बधाई! #@Ventoy डिवाइस में सफलतापूर्वक अपडेट हो गई है।",
        "STR_UPDATE_FAILED":"अपडेट के दौरान एक त्रुटि हुई। आप USB को पुन: भर सकते हैं और पुनः प्रयास कर सकते हैं। विस्तार के लिए /var/log/ventoy.log की जाँच करें।",
        "STR_WAIT_PROCESS":"एक धागा चल रहा है, कृपया प्रतीक्षा करें ...",
        "STR_MENU_OPTION":"विकल्प",
        "STR_MENU_SECURE_BOOT":"सुरक्षित बूट",
        "STR_MENU_PART_CFG":"विभाजन विन्यास",
        "STR_BTN_OK":"ठीक है",
        "STR_BTN_CANCEL":"रद्द करना",
        "STR_PRESERVE_SPACE":"डिस्क के निचले भाग में कुछ स्थान सुरक्षित रखें",
        "STR_SPACE_VAL_INVALID":"आरक्षित स्थान के लिए अमान्य मान",
        "STR_MENU_CLEAR":"Ventoy को हटा दें",
        "STR_CLEAR_SUCCESS":"डिवाइस से Ventoy को सफलतापूर्वक हटा दिया गया है।",
        "STR_CLEAR_FAILED":"डिस्क से Ventoy को साफ़ करते समय एक त्रुटि हुई। आप USB को पुन: भर सकते हैं और पुनः प्रयास कर सकते हैं। विस्तार के लिए /var/log/ventoy.log की जाँच करें।",
        "STR_MENU_PART_STYLE":"विभाजन शैली",
        "STR_DISK_2TB_MBR_ERROR":"कृपया 2TB से अधिक डिस्क के लिए GPT का चयन करें",
        "STR_SHOW_ALL_DEV":"सभी डिवाइस दिखाएं",
        "STR_PART_ALIGN_4KB":"4KB के साथ partitions संरेखित करें",
        "STR_WEB_COMMUNICATION_ERR":"संपर्क त्रुटि:",
        "STR_WEB_REMOTE_ABNORMAL":"संचार त्रुटि: remote असामान्य",
        "STR_WEB_REQUEST_TIMEOUT":"संचार त्रुटि: अनुरोध समयबाह्य (request timeout)",
        "STR_WEB_SERVICE_UNAVAILABLE":"संचार त्रुटि: सेवा अनुपलब्ध",
        "STR_WEB_TOKEN_MISMATCH":"Daemon की स्थिति अपडेट की गई, कृपया बाद में पुन: प्रयास करें।",
        "STR_WEB_SERVICE_BUSY":"सेवा व्यस्त है, कृपया बाद में पुनः प्रयास करें।",
        "STR_MENU_VTSI_CREATE":"VTSI File उत्पन्न करें",
        "STR_VTSI_CREATE_TIP":"इस बार डिवाइस पर नहीं लिखा जाएगा, लेकिन केवल एक VTSI File उत्पन्न होगी#@जारी रखें?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI फ़ाइल सफलतापूर्वक बनाई गई!#@ आप इसे डिवाइस पर लिखने के लिए Rufus(3.15+) का उपयोग कर सकते हैं ताकि Ventoy की स्थापना को पूरा किया जा सके।",
        "STR_VTSI_CREATE_FAILED":"बनाई गई VTSI file विफल रही।",
        "STR_MENU_PART_RESIZE":"गैर-विनाशकारी इंस्टॉल",
        "STR_PART_RESIZE_TIP":"यदि संभव हो तो Ventoy गैर-विनाशकारी स्थापना का प्रयास करेगा। #@जारी रखना?",
        "STR_PART_RESIZE_SUCCESS":"बधाई हो!#@Ventoy गैर-विनाशकारी स्थापना सफलतापूर्वक समाप्त हो गई।",
        "STR_PART_RESIZE_FAILED":"गैर-विनाशकारी स्थापना विफल, विवरण के लिए /var/log/ventoy.log की जाँच करें।",
        "STR_PART_RESIZE_UNSUPPORTED":"वेंटोय गैर-विनाशकारी स्थापना रोक दी गई, कुछ शर्तों को पूरा नहीं किया जा सकता है। विवरण के लिए /var/log/ventoy.log की जाँच करें।",
        "STR_INSTALL_YES_TIP1":"चेतावनी: डेटा खो जाएगा!",
        "STR_INSTALL_YES_TIP2":"यह पुष्टि करने के लिए कि आप वास्तव में अपग्रेड के बजाय एक नया इंस्टॉल करना चाहते हैं, कृपया नीचे दिए टेक्स्ट बॉक्स में Yes दर्ज करें।",
        "STR_PART_VENTOY_FS":"Ventoy विभाजन के लिए फाइल सिस्टम",
        "STR_PART_FS":"फाइल सिस्टम",
        "STR_PART_CLUSTER":"क्लस्टर साइज",
        "STR_PART_CLUSTER_DEFAULT":"सिस्टम डिफ़ॉल्ट मान",
        "STR_DONATE":"दान करें",
        "STR_4KN_UNSUPPORTED":"वर्तमान में Ventoy 4K नेटिव डिवाइस को सपोर्ट नहीं करता है।",
        
        "STRXXX":""
    },
    {
        "name":"Dutch (Nederlands)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"UmitCanbolat, RockyTDR",
        
        "STR_ERROR":"Fout",
        "STR_WARNING":"Waarschuwing",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Voer Ventoy uit in de juiste map!",
        "STR_INCORRECT_TREE_DIR":"Ventoy kan hier niet worden uitgevoerd. Download het installatiepakket en probeer Ventoy daarmee te starten.",
        "STR_DEVICE":"Apparaat",
        "STR_LOCAL_VER":"Ventoy in pakket",
        "STR_DISK_VER":"Ventoy op apparaat",
        "STR_STATUS":"Status - KLAAR",
        "STR_INSTALL":"Installeren",
        "STR_UPDATE":"Bijwerken",
        "STR_UPDATE_TIP":"Bijwerken is veilig: ISO-bestanden blijven ongewijzigd.#@Doorgaan?",
        "STR_INSTALL_TIP":"Het apparaat wordt geformatteerd en alle gegevens gaan verloren.#@Doorgaan?",
        "STR_INSTALL_TIP2":"Het apparaat wordt geformatteerd en alle gegevens gaan verloren.#@Doorgaan? (Dubbelcheck)",
        "STR_INSTALL_SUCCESS":"Gefeliciteerd!#@Ventoy is met succes op het apparaat geïnstalleerd.",
        "STR_INSTALL_FAILED":"Er is een fout opgetreden tijdens de installatie. U kunt het USB-apparaat opnieuw aansluiten en het nogmaals proberen. Controleer /var/log/ventoy.log voor details. Raadpleeg de FAQ op de officiële website als het blijft mislukken.",
        "STR_UPDATE_SUCCESS":"Gefeliciteerd!#@Ventoy is met succes bijgewerkt op het apparaat.",
        "STR_UPDATE_FAILED":"Er is een fout opgetreden tijdens de update. U kunt het USB-apparaat opnieuw aansluiten en het nogmaals proberen. Controleer /var/log/ventoy.log voor details. Raadpleeg de FAQ op de officiële website als het blijft mislukken.",
        "STR_WAIT_PROCESS":"Ventoy is nog bezig, even geduld...",
        "STR_MENU_OPTION":"Opties",
        "STR_MENU_SECURE_BOOT":"Ondersteuning voor Secure Boot",
        "STR_MENU_PART_CFG":"Partitieconfiguratie",
        "STR_BTN_OK":"Ok",
        "STR_BTN_CANCEL":"Annuleren",
        "STR_PRESERVE_SPACE":"Ruimte aan het einde van de schijf reserveren",
        "STR_SPACE_VAL_INVALID":"Ongeldige waarde voor gereserveerde ruimte",
        "STR_MENU_CLEAR":"Ventoy verwijderen",
        "STR_CLEAR_SUCCESS":"Ventoy is met succes verwijderd van het apparaat.",
        "STR_CLEAR_FAILED":"Er is een fout opgetreden bij het verwijderen van Ventoy. U kunt het USB-apparaat opnieuw aansluiten en het nogmaals proberen. Controleer /var/log/ventoy.log voor details.",
        "STR_MENU_PART_STYLE":"Partitietabel",
        "STR_DISK_2TB_MBR_ERROR":"Selecteer GPT voor apparaten groter dan 2 TB",
        "STR_SHOW_ALL_DEV":"Alle apparaten weergeven",
        "STR_PART_ALIGN_4KB":"Partities uitlijnen met 4 KB",
        "STR_WEB_COMMUNICATION_ERR":"Communicatiefout:",
        "STR_WEB_REMOTE_ABNORMAL":"Communicatiefout: extern abnormaal",
        "STR_WEB_REQUEST_TIMEOUT":"Communicatiefout: time-out van verzoek",
        "STR_WEB_SERVICE_UNAVAILABLE":"Communicatiefout: service niet beschikbaar",
        "STR_WEB_TOKEN_MISMATCH":"Daemon-status bijgewerkt, probeer het later opnieuw.",
        "STR_WEB_SERVICE_BUSY":"Service is bezet. Probeer het later opnieuw.",
        "STR_MENU_VTSI_CREATE":"VTSI-bestand genereren",
        "STR_VTSI_CREATE_TIP":"Deze keer wordt er niet naar het apparaat geschreven, maar wordt er alleen een VTSI-bestand gegenereerd#@Doorgaan?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI-bestand met succes aangemaakt!#@U kunt Rufus(3.15+) gebruiken om het bestand naar het apparaat te schrijven om de installatie van Ventoy te voltooien.",
        "STR_VTSI_CREATE_FAILED":"Aanmaken van VTSI-bestand mislukt.",
        "STR_MENU_PART_RESIZE":"Niet-destructieve installatie",
        "STR_PART_RESIZE_TIP":"Ventoy zal indien mogelijk een niet-destructieve installatie proberen. #@Doorgaan?",
        "STR_PART_RESIZE_SUCCESS":"Gefeliciteerd!#@De niet-destructieve installatie van Ventoy is met succes voltooid.",
        "STR_PART_RESIZE_FAILED":"Niet-destructieve installatie mislukt. Controleer /var/log/ventoy.log voor details.",
        "STR_PART_RESIZE_UNSUPPORTED":"De niet-destructieve installatie van Ventoy is gestopt omdat aan sommige voorwaarden niet kan worden voldaan. Controleer /var/log/ventoy.log voor details.",
        "STR_INSTALL_YES_TIP1":"Waarschuwing: alle gegevens zullen verloren gaan!",
        "STR_INSTALL_YES_TIP2":"Vul in het onderstaande tekstvak YES in om te bevestigen dat u inderdaad een nieuwe installatie wilt uitvoeren in plaats van een upgrade.",
        "STR_PART_VENTOY_FS":"Bestandssysteem voor Ventoy-partitie",
        "STR_PART_FS":"Bestandssysteem",
        "STR_PART_CLUSTER":"Clustergrootte",
        "STR_PART_CLUSTER_DEFAULT":"Standaardwaarde systeem",
        "STR_DONATE":"Doneer",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Romanian (Română)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"DorinMol, cezarlacatus, wiz64",
        
        "STR_ERROR":"Eroare",
        "STR_WARNING":"Avertisment",
        "STR_INFO":"Informare",
        "STR_INCORRECT_DIR":"Vă rugăm executați în directorul corect!",
        "STR_INCORRECT_TREE_DIR":"Nu rulați aici, vă rugăm să descărcați pachetul de instalare și executati acolo.",
        "STR_DEVICE":"Dispozitiv",
        "STR_LOCAL_VER":"Ventoy În Pachet",
        "STR_DISK_VER":"Ventoy În Dispozitiv",
        "STR_STATUS":"Stare - PREGĂTIT",
        "STR_INSTALL":"Instalare",
        "STR_UPDATE":"Actualizare",
        "STR_UPDATE_TIP":"Operația de actualizare este sigură, fișierele ISO nu vor fi alterate / modificate.#@Continuați?",
        "STR_INSTALL_TIP":"Unitatea disc va fi formatată și toate datele vor fi pierdute.#@Continuați?",
        "STR_INSTALL_TIP2":"Unitatea disc va fi formatată și toate datele vor fi pierdute.#@Continuați? (Verificare Dublă)",
        "STR_INSTALL_SUCCESS":"Felicitări!#@Ventoy a fost instalat cu succes pe dispozitiv.",
        "STR_INSTALL_FAILED":"A apărut o eroare în timpul instalării. Reintroduceți dispozitivul USB și încercați din nou. Verificați /var/log/ventoy.log pentru detalii.",
        "STR_UPDATE_SUCCESS":"Felicitări!#@Ventoy a fost actualizat cu succes pe dispozitiv.",
        "STR_UPDATE_FAILED":"A apărut o eroare în timpul actualizării. Reintroduceți dispozitivul USB și încercați din nou. Verificați /var/log/ventoy.log pentru detalii.",
        "STR_WAIT_PROCESS":"Rulează un fir de execuție, vă rugăm așteptați...",
        "STR_MENU_OPTION":"Opțiune",
        "STR_MENU_SECURE_BOOT":"Încărcare Sigură",
        "STR_MENU_PART_CFG":"Configurare partiție",
        "STR_BTN_OK":"O.K",
        "STR_BTN_CANCEL":"Anulare",
        "STR_PRESERVE_SPACE":"Păstrați puțin spațiu în partea de jos a discului",
        "STR_SPACE_VAL_INVALID":"Valoare nevalidă pentru spațiul rezervat",
        "STR_MENU_CLEAR":"Curata Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy a fost eliminat cu succes de pe dispozitiv.",
        "STR_CLEAR_FAILED":"A apărut o eroare la ștergerea Ventoy de pe disc. Puteți reconecta USB-ul și încerca din nou. Verificați /var/log/ventoy.log pentru detalii.",
        "STR_MENU_PART_STYLE":"Stil partiție",
        "STR_DISK_2TB_MBR_ERROR":"Vă rugăm să selectați GPT pentru disc de peste 2 TB",
        "STR_SHOW_ALL_DEV":"Afișați toate dispozitivele",
        "STR_PART_ALIGN_4KB":"Aliniați partițiile cu 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Eroare de comunicare:",
        "STR_WEB_REMOTE_ABNORMAL":"Eroare de comunicare: remote abnormal",
        "STR_WEB_REQUEST_TIMEOUT":"Eroare de comunicare: Solicitarea a expirat",
        "STR_WEB_SERVICE_UNAVAILABLE":"Eroare de comunicare: Serviciu indisponibil",
        "STR_WEB_TOKEN_MISMATCH":"Starea demonului a fost actualizată, vă rugăm să reîncercați mai târziu.",
        "STR_WEB_SERVICE_BUSY":"Serviciul este ocupat, vă rugăm să reîncercați mai târziu.",
        "STR_MENU_VTSI_CREATE":"Generați fișierul VTSI",
        "STR_VTSI_CREATE_TIP":"De data aceasta nu va scrie pe dispozitiv, ci va genera doar un fișier VTSI#@Continuați?",
        "STR_VTSI_CREATE_SUCCESS":"Fișierul VTSI creat cu succes!#@Puteți folosi Rufus(3.15+) pentru a-l scrie pe dispozitiv, astfel încât să finalizați instalarea Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Fișierul VTSI creat a eșuat.",
        "STR_MENU_PART_RESIZE":"Instalare nedistructivă",
        "STR_PART_RESIZE_TIP":"Ventoy va încerca instalarea nedistructivă, dacă este posibil. #@Continuă?",
        "STR_PART_RESIZE_SUCCESS":"Felicitări!#@Ventoy instalarea nedistructivă s-a încheiat cu succes.",
        "STR_PART_RESIZE_FAILED":"Instalarea nedistructivă a eșuat. Verificați /var/log/ventoy.log pentru detalii.",
        "STR_PART_RESIZE_UNSUPPORTED":"Instalarea nedistructivă Ventoy a fost oprită deoarece unele condiții nu pot fi îndeplinite. Verificați /var/log/ventoy.log pentru detalii.",
        "STR_INSTALL_YES_TIP1":"Atenție: Datele se vor pierde!",
        "STR_INSTALL_YES_TIP2":"Introduceți DA în caseta de text de mai jos pentru a confirma că într-adevăr doriți să faceți o nouă instalare în loc să faceți upgrade.",
        "STR_PART_VENTOY_FS":"Sistem de fișiere pentru partiția Ventoy",
        "STR_PART_FS":"Sistemul de fișiere",
        "STR_PART_CLUSTER":"Dimensiunea clusterului",
        "STR_PART_CLUSTER_DEFAULT":"Valoarea implicită a sistemului",
        "STR_DONATE":"Donează",
        "STR_4KN_UNSUPPORTED":"Momentan, Ventoy nu suporta dispozitive native 4k.",
        
        "STRXXX":""
    },
    {
        "name":"Japanese (日本語)",
        "FontFamily":"Yu Gothic UI",
        "FontSize":16,
        "Author":"taichi eto, Bentnand, cmplstofB, Re*Index.(ot_inc)",
        
        "STR_ERROR":"エラー",
        "STR_WARNING":"警告",
        "STR_INFO":"情報",
        "STR_INCORRECT_DIR":"正しいディレクトリで実行してください!",
        "STR_INCORRECT_TREE_DIR":"ここで実行をしないでください。リリースされたインストールパッケージをダウンロードして、実行をしてください。",
        "STR_DEVICE":"デバイス",
        "STR_LOCAL_VER":"パッケージ内のVentoy",
        "STR_DISK_VER":"デバイス内のVentoy",
        "STR_STATUS":"状態 - 準備完了",
        "STR_INSTALL":"インストール",
        "STR_UPDATE":"アップデート",
        "STR_UPDATE_TIP":"アップグレードの操作は安全に行なわれ、ISOファイルは変更もされません。#@続行しますか?",
        "STR_INSTALL_TIP":"デバイスはフォーマットされ、すべてのデータが失われます。#@続行しますか?",
        "STR_INSTALL_TIP2":"デバイスはフォーマットされ、すべてのデータが失われます。#@続行しますか? (再確認)",
        "STR_INSTALL_SUCCESS":"おめでとうございます!#@Ventoyがデバイスにインストールされました。",
        "STR_INSTALL_FAILED":"インストール中にエラーが発生しました。USBデバイスを再接続をして再試行ができます。詳細については、/var/log/ventoy.logを確認してください。何度も失敗する場合は、公式サイト内のFAQをご参照ください。",
        "STR_UPDATE_SUCCESS":"おめでとうございます!#@Ventoyがデバイスで正常にアップデートされました。",
        "STR_UPDATE_FAILED":"アップデート中にエラーが発生しました。USBデバイスを再接続して再試行ができます。詳細については、/var/log/ventoy.logを確認してください。何度も失敗する場合は、公式サイトのFAQをご参照ください。",
        "STR_WAIT_PROCESS":"スレッドが実行中です。お待ちください...",
        "STR_MENU_OPTION":"オプション",
        "STR_MENU_SECURE_BOOT":"セキュアブートのサポート",
        "STR_MENU_PART_CFG":"パーティションの設定",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"キャンセル",
        "STR_PRESERVE_SPACE":"ディスクの最後にある程度の容量を確保する",
        "STR_SPACE_VAL_INVALID":"システム予約済みパーティションの値が無効です",
        "STR_MENU_CLEAR":"Ventoyを削除する",
        "STR_CLEAR_SUCCESS":"Ventoyがデバイスから正常に削除されました。",
        "STR_CLEAR_FAILED":"Ventoyをディスクから削除するときにエラーが発生しました。USBデバイスを再接続して再試行ができます。詳細については、/var/log/ventoy.logを確認してください。",
        "STR_MENU_PART_STYLE":"パーティションのスタイル",
        "STR_DISK_2TB_MBR_ERROR":"2TBを超えるデバイスの場合はGPTを選択してください",
        "STR_SHOW_ALL_DEV":"すべてのデバイスを表示",
        "STR_PART_ALIGN_4KB":"パーティションを4KBに揃える",
        "STR_WEB_COMMUNICATION_ERR":"コミュニケーションエラー:",
        "STR_WEB_REMOTE_ABNORMAL":"コミュニケーションエラー: リモート先が異常になっています",
        "STR_WEB_REQUEST_TIMEOUT":"コミュニケーションエラー: リクエストがタイムアウトしました",
        "STR_WEB_SERVICE_UNAVAILABLE":"コミュニケーションエラー: サービスが利用できません",
        "STR_WEB_TOKEN_MISMATCH":"デーモンのステータスが更新されました。後で再試行してください。",
        "STR_WEB_SERVICE_BUSY":"サービスがビジーの状態です。後で再試行してください。",
        "STR_MENU_VTSI_CREATE":"VTSIファイルを作成する",
        "STR_VTSI_CREATE_TIP":"今回はデバイスへの書き込みを行なわず、VTSIファイルの作成のみを行ないます。#@続行しますか?",
        "STR_VTSI_CREATE_SUCCESS":"VTSIファイルが正常に作成されました!#@Rufus(3.15以降)を使用してデバイスに書き込むことでVentoyのインストールが完了します。",
        "STR_VTSI_CREATE_FAILED":"VTSIファイルの作成に失敗しました。",
        "STR_MENU_PART_RESIZE":"非破壊なインストール",
        "STR_PART_RESIZE_TIP":"Ventoyは、可能であれば非破壊なインストールを試みます。#@続行しますか?",
        "STR_PART_RESIZE_SUCCESS":"おめでとうございます!#@Ventoyの非破壊なインストールが正常に完了しました。",
        "STR_PART_RESIZE_FAILED":"非破壊なインストールが失敗しました。詳細については、/var/log/ventoy.logを確認してください。",
        "STR_PART_RESIZE_UNSUPPORTED":"いくつかの条件を満たしていないため、Ventoyの非破壊なインストールは停止されました。詳細については、/var/log/ventoy.logを確認してください。",
        "STR_INSTALL_YES_TIP1":"警告: データが失われます!",
        "STR_INSTALL_YES_TIP2":"アップグレードではなく新規インストールを実行するには、下のテキストボックスに「YES」と入力してください。",
        "STR_PART_VENTOY_FS":"Ventoyパーティションのファイルシステム",
        "STR_PART_FS":"ファイルシステム",
        "STR_PART_CLUSTER":"クラスタサイズ",
        "STR_PART_CLUSTER_DEFAULT":"システムの既定値",
        "STR_DONATE":"寄付",
        "STR_4KN_UNSUPPORTED":"現在、Ventoyは4Kネイティブデバイスをサポートしていません。",
        
        "STRXXX":""
    },
    {
        "name":"Italian (Italiano)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"AverageUser2,Albano Battistella,bovirus,passero",
        
        "STR_ERROR":"Errore",
        "STR_WARNING":"Attenzione",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Eseguilo nella cartella corretta",
        "STR_INCORRECT_TREE_DIR":"Non eseguirlo qui.#@Scarica il pacchetto di installazione ed avvialo.",
        "STR_DEVICE":"Dispositivo",
        "STR_LOCAL_VER":"Versione installata",
        "STR_DISK_VER":"Versione nel dispositivo",
        "STR_STATUS":"Stato - PRONTO",
        "STR_INSTALL":"Installa",
        "STR_UPDATE":"Aggiorna",
        "STR_UPDATE_TIP":"L'aggiornamento è sicuro, i file presenti nel dispositivo rimarranno invariati.#@#@Vuoi continuare?",
        "STR_INSTALL_TIP":"Il dispositivo verrà formattato e tutti i dati saranno persi.#@#@Vuoi continuare?",
        "STR_INSTALL_TIP2":"Il dispositivo verrà formattato e tutti i dati saranno persi.#@#@Vuoi continuare (seconda verifica)?",
        "STR_INSTALL_SUCCESS":"Congratulazioni!#@#@Installazione di Ventoy nel dispositivo completata.",
        "STR_INSTALL_FAILED":"Si è verificato un errore durante l'installazione.#@#@Riconnetti il dispositivo e riprova.#@#@Per i dettagli controlla il file /var/log/ventoy.log .",
        "STR_UPDATE_SUCCESS":"Congratulazioni!#@#@Aggiornamento di Ventoy nel dispositivo completato",
        "STR_UPDATE_FAILED":"Si è verificato un errore durante l'aggiornamento.#@È possibile riconnettere il dispositivo USB e riprovare.#@Per i dettagli controlla il file /var/log/ventoy.log.#@Se l'aggiornamento non va a buon fine, consulta le FAQ sul sito ufficiale",
        "STR_WAIT_PROCESS":"Un processo è in esecuzione. Attendi...",
        "STR_MENU_OPTION":"Opzioni",
        "STR_MENU_SECURE_BOOT":"Avvio protetto (secure boot)",
        "STR_MENU_PART_CFG":"Configurazione partizione",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Annulla",
        "STR_PRESERVE_SPACE":"Riserva un po' di spazio alla fine del dispositivo",
        "STR_SPACE_VAL_INVALID":"Quantità di spazio da riservare non valida",
        "STR_MENU_CLEAR":"Rimuovi Ventoy dal dispositivo",
        "STR_CLEAR_SUCCESS":"Rimozione di Ventoy dal dispositivo completata.",
        "STR_CLEAR_FAILED":"Si è verificato un errore durante la rimozione di Ventoy dal dispositivo.#@È possibile riconnettere il dispositivo USB e riprovare.#@Per i dettagli controlla il file /var/log/ventoy.log ",
        "STR_MENU_PART_STYLE":"Stile tabella partizioni",
        "STR_DISK_2TB_MBR_ERROR":"Seleziona GPT per i dispositivi più grandi di 2TB",
        "STR_SHOW_ALL_DEV":"Visualizza tutti i dispositivi",
        "STR_PART_ALIGN_4KB":"Allinea le partizioni con 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Errore di comunicazione:",
        "STR_WEB_REMOTE_ABNORMAL":"Errore di comunicazione: remoto non normale",
        "STR_WEB_REQUEST_TIMEOUT":"Errore di comunicazione: richiesta scaduta",
        "STR_WEB_SERVICE_UNAVAILABLE":"Errore di comunicazione: servizio non disponibile",
        "STR_WEB_TOKEN_MISMATCH":"Stato del demon aggiornato. Riprova più tardi.",
        "STR_WEB_SERVICE_BUSY":"Il servizio è occupato. Riprova più tardi.",
        "STR_MENU_VTSI_CREATE":"Genera file VTSI",
        "STR_VTSI_CREATE_TIP":"Questa volta non scriverà sul dispositivo, ma genererà solo un file VTSI#@#@Vuoi continuare?",
        "STR_VTSI_CREATE_SUCCESS":"Creazione file VTSI completata!#@Per scriverlo nel dispositivo in modo da completare l'installazione di Ventoy puoi usare Rufus (versione 3.15 o successiva).",
        "STR_VTSI_CREATE_FAILED":"Creazione file VTSI non riuscita.",
        "STR_MENU_PART_RESIZE":"Installazione non distruttiva",
        "STR_PART_RESIZE_TIP":"Ventoy proverà, se possibile, l'installazione non distruttiva.#@#@Vuoi continuare?",
        "STR_PART_RESIZE_SUCCESS":"Congratulazioni!#@Installazione non distruttiva di Ventoy completata.",
        "STR_PART_RESIZE_FAILED":"Installazione non distruttiva fallita.#@Controlla il file /var/log/ventoy.log per i dettagli.",
        "STR_PART_RESIZE_UNSUPPORTED":"L'installazione non distruttiva di Ventoy è stata interrotta perché alcune condizioni non possono essere soddisfatte.#@Controlla il file /var/log/ventoy.log per i dettagli.",
        "STR_INSTALL_YES_TIP1":"Attenzione: i dati verranno persi!",
        "STR_INSTALL_YES_TIP2":"Per confermare che vuoi davvero eseguire un'installazione pulita anziché un aggiornamento digita YES nella casella di testo qui sotto .",
        "STR_PART_VENTOY_FS":"File system partizione Ventoy",
        "STR_PART_FS":"File system",
        "STR_PART_CLUSTER":"Dimensione cluster",
        "STR_PART_CLUSTER_DEFAULT":"Valore predefinito sistema",
        "STR_DONATE":"Dona",
        "STR_4KN_UNSUPPORTED":"Attualmente Ventoy non supporta dispositivi nativi 4K.",
        
        "STRXXX":""
    },
    {
        "name":"Croatian (Hrvatski)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Valnjes",
        
        "STR_ERROR":"Pogreška",
        "STR_WARNING":"Upozorenje",
        "STR_INFO":"Informacija",
        "STR_INCORRECT_DIR":"Molim vas pokrenite unutar pravog direktorija!",
        "STR_INCORRECT_TREE_DIR":"Ne pokrećite me ovdje, molim vas downloadajte poslijednju verziju, te ponovno pokrenite!",
        "STR_DEVICE":"Uređaj",
        "STR_LOCAL_VER":"Ventoy (lokalni)",
        "STR_DISK_VER":"Ventoy (na uređaju)",
        "STR_STATUS":"Status - SPREMAN",
        "STR_INSTALL":"Instaliraj",
        "STR_UPDATE":"Ažuriraj",
        "STR_UPDATE_TIP":"Nadogradnja je sigurna, ISO datoteke neće biti promjenjene.#@Nastaviti?",
        "STR_INSTALL_TIP":"USB disk će biti formatiran i svi podatci će biti izgubljeni!#@Nastaviti?",
        "STR_INSTALL_TIP2":"USB disk će biti formatiran i svi podatci će biti izgubljeni!#@Nastaviti? (Dodatna provjera)",
        "STR_INSTALL_SUCCESS":"Čestitam!#@Ventoy je uspješno instaliran na vaš uređaj.",
        "STR_INSTALL_FAILED":"Dogodila se pogreška tokom instalacije. Pokušajte ponovno spojiti USB i pokušati ponovno. Provjerite /var/log/ventoy.log za više detalja o nastaloj pogrešci.",
        "STR_UPDATE_SUCCESS":"Čestitam!#@Ventoy je uspješno ažuriran na vašem uređaju.",
        "STR_UPDATE_FAILED":"Dogodila se pogreška tokom ažuriranja. Pokušajte ponovno spojiti USB i pokušati ponovno. Provjerite /var/log/ventoy.log za više detalja o nastaloj pogrešci.",
        "STR_WAIT_PROCESS":"Instanca je vec pokrenuta, molimo vas pričekajte...",
        "STR_MENU_OPTION":"Opcije",
        "STR_MENU_SECURE_BOOT":"Secure Boot",
        "STR_MENU_PART_CFG":"Konfiguracija particije",
        "STR_BTN_OK":"u redu",
        "STR_BTN_CANCEL":"Otkazati",
        "STR_PRESERVE_SPACE":"Sačuvajte malo prostora na dnu diska",
        "STR_SPACE_VAL_INVALID":"Nevažeća vrijednost rezerviranog prostora",
        "STR_MENU_CLEAR":"Obriši Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy je uspesno obrisan sa uređaja",
        "STR_CLEAR_FAILED":"Dogodila se greška tokom brisanja Ventoy sa diska. Možeš ponovo ubacit USB i pokušati opet. Pogledaj /var/log/ventoy.log za još detalja.",
        "STR_MENU_PART_STYLE":"Stil particija",
        "STR_DISK_2TB_MBR_ERROR":"Molim te, koristi GPT za diskove preko 2TB",
        "STR_SHOW_ALL_DEV":"Show All Devices",
        "STR_PART_ALIGN_4KB":"Align partitions with 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Communication error:",
        "STR_WEB_REMOTE_ABNORMAL":"Communication error: remote abnormal",
        "STR_WEB_REQUEST_TIMEOUT":"Communication error: Request timed out",
        "STR_WEB_SERVICE_UNAVAILABLE":"Communication error: Service Unavailable",
        "STR_WEB_TOKEN_MISMATCH":"Daemon status updated, please retry later.",
        "STR_WEB_SERVICE_BUSY":"Service is busy, please retry later.",
        "STR_MENU_VTSI_CREATE":"Generate VTSI File",
        "STR_VTSI_CREATE_TIP":"This time will not write to the device, but only generate a VTSI file#@Continue?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI file created successfully!#@You can use Rufus(3.15+) to write it to the device so as to complete the installation of Ventoy.",
        "STR_VTSI_CREATE_FAILED":"VTSI file created failed.",
        "STR_MENU_PART_RESIZE":"Non-destructive Install",
        "STR_PART_RESIZE_TIP":"Ventoy will try non-destructive installation if possible. #@Continue?",
        "STR_PART_RESIZE_SUCCESS":"Congratulations!#@Ventoy non-destructive installation successfully finished.",
        "STR_PART_RESIZE_FAILED":"Non-destructive installation failed, Check /var/log/ventoy.log for details.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy non-destructive installation stopped because some conditions cannot be met. Check /var/log/ventoy.log for details.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"Donirajte",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Hungarian (Magyar)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Bitfarago, gidano, meskobalazs",
        
        "STR_ERROR":"Hiba",
        "STR_WARNING":"Figyelmeztetés",
        "STR_INFO":"Információ",
        "STR_INCORRECT_DIR":"Futtassa a megfelelő könyvtárból.",
        "STR_INCORRECT_TREE_DIR":"Ne innen futtassa, töltse le a kiadott telepítőcsomagot, és futtassa onnan.",
        "STR_DEVICE":"Eszköz",
        "STR_LOCAL_VER":"A csomagban lévő Ventoy",
        "STR_DISK_VER":"Az eszközön lévő Ventoy",
        "STR_STATUS":"Állapot – KÉSZ",
        "STR_INSTALL":"Telepítés",
        "STR_UPDATE":"Frissítés",
        "STR_UPDATE_TIP":"A frissítés biztonságos, az ISO-fájlok nem változnak.#@Folytatja?",
        "STR_INSTALL_TIP":"A meghajtó formázva lesz, és minden rajta lévő adat elvész.#@Folytatja?",
        "STR_INSTALL_TIP2":"A meghajtó formázva lesz, és minden rajta lévő adat elvész.#@Folytatja? (Második jóváhagyás)",
        "STR_INSTALL_SUCCESS":"Gratulálunk!#@A Ventoy sikeresen telepítve lett az eszközön.",
        "STR_INSTALL_FAILED":"Hiba történt a telepítés során. Csatlakoztassa újra az USB-eszközt, és próbálja újra.#@A részleteket lásd a /var/log/ventoy.log fájlban.",
        "STR_UPDATE_SUCCESS":"Gratulálunk!#@A Ventoy sikeresen frissítve lett az eszközön.",
        "STR_UPDATE_FAILED":"Hiba történt a frissítés során. Csatlakoztassa újra az USB-eszközt, és próbálja újra.#@A részleteket lásd a /var/log/ventoy.log fájlban.",
        "STR_WAIT_PROCESS":"A feldolgozás még folyamatban van, kis türelmet…",
        "STR_MENU_OPTION":"Beállítás",
        "STR_MENU_SECURE_BOOT":"Secure Boot támogatás",
        "STR_MENU_PART_CFG":"Partíció konfiguráció",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Mégse",
        "STR_PRESERVE_SPACE":"Szabad terület maradjon a lemez végén",
        "STR_SPACE_VAL_INVALID":"Érvénytelen érték a fenntartott helynél",
        "STR_MENU_CLEAR":"A Ventoy eltávolítása",
        "STR_CLEAR_SUCCESS":"A Ventoy sikeresen eltávolítva az eszközről.",
        "STR_CLEAR_FAILED":"Hiba történt a Ventoy lemezről törlése során. Újra csatlakoztathatja az USB-t, és megpróbálhatja újra. A részleteket lásd a /var/log/ventoy.log fájlban.",
        "STR_MENU_PART_STYLE":"Partíció fajtája",
        "STR_DISK_2TB_MBR_ERROR":"2 TB-nál nagyobb lemezhez válassza a GPT-t",
        "STR_SHOW_ALL_DEV":"Az összes eszköz megjelenítése",
        "STR_PART_ALIGN_4KB":"A partíciók 4 kB-hoz igazítása",
        "STR_WEB_COMMUNICATION_ERR":"Kommunikációs hiba:",
        "STR_WEB_REMOTE_ABNORMAL":"Kommunikációs hiba: Távoli hiba",
        "STR_WEB_REQUEST_TIMEOUT":"Kommunikációs hiba: Kérés időtúllépése",
        "STR_WEB_SERVICE_UNAVAILABLE":"Kommunikációs hiba: A szolgáltatás nem érhető el",
        "STR_WEB_TOKEN_MISMATCH":"A démon állapota frissítve, próbálja újra később.",
        "STR_WEB_SERVICE_BUSY":"A szolgáltatás foglalt, próbálja újra később.",
        "STR_MENU_VTSI_CREATE":"VTSI fájl előállítása",
        "STR_VTSI_CREATE_TIP":"Ez alkalommal nem ír az eszközre, csak egy VTSI fájlt állít elő.#@Folytatja?",
        "STR_VTSI_CREATE_SUCCESS":"A VTSI fájl sikeresen létrehozva!#@A Rufus(3.15+) segítségével az eszközre írhatja a Ventoy telepítésének befejezéséhez.",
        "STR_VTSI_CREATE_FAILED":"A VTSI fájl létrehozása nem sikerült.",
        "STR_MENU_PART_RESIZE":"Rombolásmentes telepítés",
        "STR_PART_RESIZE_TIP":"A Ventoy lehetőség szerint megpróbálja a rombolásmentes telepítést. #@Folytatja?",
        "STR_PART_RESIZE_SUCCESS":"Gratulálunk!#@A Ventoy rombolásmentes telepítése sikeresen befejeződött.",
        "STR_PART_RESIZE_FAILED":"A rombolásmentes telepítés nem sikerült, a részleteket lásd a /var/log/ventoy.log fájlban.",
        "STR_PART_RESIZE_UNSUPPORTED":"A Ventoy rombolásmentes telepítése leállt, mert bizonyos feltételek nem teljesíthetők. A részleteket lásd a /var/log/ventoy.log fájlban.",
        "STR_INSTALL_YES_TIP1":"Figyelmeztetés: Az adatok elvesznek!",
        "STR_INSTALL_YES_TIP2":"Írja a YES szót az alábbi szövegmezőbe, hogy megerősítse, valóban új telepítést szeretne végrehajtani a frissítés helyett.",
        "STR_PART_VENTOY_FS":"Fájlrendszer a Ventoy partícióhoz",
        "STR_PART_FS":"Fájlrendszer",
        "STR_PART_CLUSTER":"Klaszter méret",
        "STR_PART_CLUSTER_DEFAULT":"Rendszer alapérték",
        "STR_DONATE":"Adományoz",
        "STR_4KN_UNSUPPORTED":"Jelenleg a Ventoy nem támogat natív 4K eszközt.",
        
        "STRXXX":""
    },
    {
        "name":"Traditional Chinese（正體中文）",
        "FontFamily":"新細明體",
        "FontSize":14,
        "Author":"penut85420,tony8077616",
        
        "STR_ERROR":"錯誤",
        "STR_WARNING":"警告",
        "STR_INFO":"提醒",
        "STR_INCORRECT_DIR":"請在正確的資料夾下開啟！",
        "STR_INCORRECT_TREE_DIR":"請下載並使用發行版本的安裝包",
        "STR_DEVICE":"裝置",
        "STR_LOCAL_VER":"安裝包內 Ventoy 版本",
        "STR_DISK_VER":"裝置內 Ventoy 版本",
        "STR_STATUS":"狀態 - 準備就緒",
        "STR_INSTALL":"安裝",
        "STR_UPDATE":"升級",
        "STR_UPDATE_TIP":"升級操作是安全的，磁碟內的 ISO 檔案不會被清除#@請問是否繼續？",
        "STR_INSTALL_TIP":"磁碟將會被格式化，所有內容將會被清除！#@請問是否繼續？",
        "STR_INSTALL_TIP2":"磁碟將會被格式化，所有內容將會被清除！#@請再次確認是否繼續？",
        "STR_INSTALL_SUCCESS":"恭喜，Ventoy 已經成功安裝到此裝置中！",
        "STR_INSTALL_FAILED":"安裝 Ventoy 的過程中發生錯誤，請再次重新插入磁碟重試，詳細訊息請檢視 /var/log/ventoy.log 檔案。",
        "STR_UPDATE_SUCCESS":"恭喜，新版本的 Ventoy 已經成功更新到此裝置中！",
        "STR_UPDATE_FAILED":"更新 Ventoy 的過程中發生錯誤，請再次重新插入磁碟重試，詳細訊息請檢視 /var/log/ventoy.log 檔案。",
        "STR_WAIT_PROCESS":"目前仍有執行緒正在運作中，請稍候...",
        "STR_MENU_OPTION":"選項",
        "STR_MENU_SECURE_BOOT":"支援安全開機",
        "STR_MENU_PART_CFG":"設定分割磁區",
        "STR_BTN_OK":"確定",
        "STR_BTN_CANCEL":"取消",
        "STR_PRESERVE_SPACE":"在磁區最後保留一部分空間",
        "STR_SPACE_VAL_INVALID":"保留空間的容量不正確",
        "STR_MENU_CLEAR":"清除 Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy 已成功從裝置中清除",
        "STR_CLEAR_FAILED":"清除 Ventoy 的過程中發生錯誤，請再次重新插入磁碟重試，詳細訊息請檢視 /var/log/ventoy.log 檔案。",
        "STR_MENU_PART_STYLE":"分割表格式",
        "STR_DISK_2TB_MBR_ERROR":"超過 2TB 的磁碟請選擇 GPT 格式分割表",
        "STR_SHOW_ALL_DEV":"顯示所有裝置",
        "STR_PART_ALIGN_4KB":"磁碟分割區按照 4KB 對齊",
        "STR_WEB_COMMUNICATION_ERR":"通訊錯誤:",
        "STR_WEB_REMOTE_ABNORMAL":"通訊錯誤: 伺服器端發生異常",
        "STR_WEB_REQUEST_TIMEOUT":"通訊錯誤: 請求逾時",
        "STR_WEB_SERVICE_UNAVAILABLE":"通訊錯誤: 伺服器無法使用",
        "STR_WEB_TOKEN_MISMATCH":"伺服器狀態已更新",
        "STR_WEB_SERVICE_BUSY":"伺服器忙碌中，請稍後再試",
        "STR_MENU_VTSI_CREATE":"建立 VTSI 檔案",
        "STR_VTSI_CREATE_TIP":"這項操作將只會在原地目錄建立一個 VTSI 的檔案#@請問是否繼續？",
        "STR_VTSI_CREATE_SUCCESS":"VTSI 檔案建立完成！#@您可以使用 Rufus(3.15+) 將 VTSI 檔案寫入指定的裝置以完成 Ventoy 的安裝。",
        "STR_VTSI_CREATE_FAILED":"VTSI 檔案建立失敗。",
        "STR_MENU_PART_RESIZE":"無損安裝",
        "STR_PART_RESIZE_TIP":"Ventoy 將嘗試進行無損安裝。#@請注意：任何對於磁碟的任何操作都存在一定的風險，對於重要的資料建議您做好備份！#@請問您是否繼續？",
        "STR_PART_RESIZE_SUCCESS":"恭喜！#@Ventoy 已完成無損安裝。",
        "STR_PART_RESIZE_FAILED":"Ventoy 無損安裝失敗。詳細訊息請檢視 /var/log/ventoy.log 檔案。",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy 無損安裝已中止因為未滿足部分條件。詳細訊息請檢視 /var/log/ventoy.log 檔案。",
        "STR_INSTALL_YES_TIP1":"警告：資料將會遺失！",
        "STR_INSTALL_YES_TIP2":"請在下方的文字輸入框輸入 YES 來證實您想要進行全新安裝取代升級。",
        "STR_PART_VENTOY_FS":"Ventoy 磁區的檔案系統",
        "STR_PART_FS":"檔案系統",
        "STR_PART_CLUSTER":"磁區單位大小",
        "STR_PART_CLUSTER_DEFAULT":"系統預設值",
        "STR_DONATE":"贊助",
        "STR_4KN_UNSUPPORTED":"目前 Ventoy 不支援單位磁區格式為4K的硬碟。",
        
        "STRXXX":""
    },
    {
        "name":"Serbian Latin (Srpski)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Bojan Maksimović, panickingkernel, Zoran Jankov, Stefan Šućur, Kosta Stojiljković",
        
        "STR_ERROR":"Greška",
        "STR_WARNING":"Upozorenje",
        "STR_INFO":"Informacija",
        "STR_INCORRECT_DIR":"Molim Vas, pokrenite me u pravom direktorijumu!",
        "STR_INCORRECT_TREE_DIR":"Nemojtе me pokretati ovde, molim Vas preuzmite objavljeni instalacioni paket i pokrenite me tamo.",
        "STR_DEVICE":"Uređaj",
        "STR_LOCAL_VER":"Ventoy u paketu",
        "STR_DISK_VER":"Ventoy u uređaju",
        "STR_STATUS":"Status - SPREMAN",
        "STR_INSTALL":"Instaliraj",
        "STR_UPDATE":"Nadogradi",
        "STR_UPDATE_TIP":"Nadogradnja je sigurna, ISO datoteke neće biti promenjene.#@Nastavi?",
        "STR_INSTALL_TIP":"Disk će biti formatiran i svi podaci će biti izgubljeni.#@Nastavi?",
        "STR_INSTALL_TIP2":"Disk će biti formatiran i svi podaci će biti izgubljeni.#@Nastavi? (Dvostruka provera)",
        "STR_INSTALL_SUCCESS":"Čestitam!#@Ventoy je uspešno instaliran na ovom uređaju.",
        "STR_INSTALL_FAILED":"Dogodila se greška tokom instalacije. Možete izvaditi i ponovo ubaciti USB, pa pokušati opet. Pogledajte /var/log/ventoy.log za još detalja.",
        "STR_UPDATE_SUCCESS":"Čestitam!#@Ventoy je uspešno nadograđen na ovom uređaju.",
        "STR_UPDATE_FAILED":"Dogodila se greška prilikom nadogradnje. Možeте izvaditi i ponovo ubaciti USB, pa pokušati opet. Pogledajte /var/log/ventoy.log za još detalja.",
        "STR_WAIT_PROCESS":"Odvija se operacija, molim Vas sačekajte...",
        "STR_MENU_OPTION":"Opcije",
        "STR_MENU_SECURE_BOOT":"Sigurno uključivanje je podržano",
        "STR_MENU_PART_CFG":"Konfiguracija particija",
        "STR_BTN_OK":"U redu",
        "STR_BTN_CANCEL":"Otkaži",
        "STR_PRESERVE_SPACE":"Sačuvaj nešto prostora na kraju diska",
        "STR_SPACE_VAL_INVALID":"Neispravna vrednost za prezervirani prostor",
        "STR_MENU_CLEAR":"Obriši Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy je uspešno obrisan sa uređaja",
        "STR_CLEAR_FAILED":"Dogodila se greška tokom brisanja Ventoy sa diska. Možete ponovo ubaciti USB i pokušati opet. Pogledajte /var/log/ventoy.log za još detalja.",
        "STR_MENU_PART_STYLE":"Stil particija",
        "STR_DISK_2TB_MBR_ERROR":"Molim Vas, izaberite GPT za diskove preko 2TB",
        "STR_SHOW_ALL_DEV":"Pokaži sve uređaje",
        "STR_PART_ALIGN_4KB":"Poravnajte sve particije sa 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Greška u komunikaciji:",
        "STR_WEB_REMOTE_ABNORMAL":"Greška u komunikaciji: nenormalno daljinsko upravljanje",
        "STR_WEB_REQUEST_TIMEOUT":"Greška u komunikaciji: Zahtev je istekao",
        "STR_WEB_SERVICE_UNAVAILABLE":"Greška u komunikaciji: Servis nije dostupan",
        "STR_WEB_TOKEN_MISMATCH":"Status demona ažuriran, pokušajte ponovo kasnije.",
        "STR_WEB_SERVICE_BUSY":"Servis je zauzet, pokušajte ponovo kasnije.",
        "STR_MENU_VTSI_CREATE":"Stvori VTSI datoteku",
        "STR_VTSI_CREATE_TIP":"Ovaj put ne pisati na uređaj, nego samo stvoriti VTSI datoteku#@Nastavi?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI datoteka uspešno stvorena!#@Možete koristiti Rufus(3.15+) da je zapišete na uređaj da biste završili instalaciju Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Greška u stvaranju VTSI datoteke.",
        "STR_MENU_PART_RESIZE":"Nedestruktivna instalacija",
        "STR_PART_RESIZE_TIP":"Ventoy će pokušati nedestruktivnu instalaciju ako je moguće. #@Nastavi?",
        "STR_PART_RESIZE_SUCCESS":"Čestitam!#@Ventoy nedestruktivna instalacija je bila uspešna.",
        "STR_PART_RESIZE_FAILED":"Nedestruktivna instalacija je bila neuspešna. Pogledajte /var/log/ventoy.log za još detalja.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy nedestruktivna instalacija se zaustavila jer nisu postignuti neki uslovi. Pogledajte /var/log/ventoy.log za još detalja.",
        "STR_INSTALL_YES_TIP1":"Upozorenje: podaci će biti izgubljeni!",
        "STR_INSTALL_YES_TIP2":"Molim Vas, unesite YES u polje ispod da biste potvrdili da zaista želite da uradite novu instalaciju umesto nadogradnje.",
        "STR_PART_VENTOY_FS":"Datotečni sistem za Ventoy particiju",
        "STR_PART_FS":"Datotoečni sistem",
        "STR_PART_CLUSTER":"Veličina klastera",
        "STR_PART_CLUSTER_DEFAULT":"Podrazumevana sistemska vrednost",
        "STR_DONATE":"Donirajte",
        "STR_4KN_UNSUPPORTED":"Ventoy trenutno ne podržava 4K izvorne uređaje.",
        
        "STRXXX":""
    },
    {
        "name":"Serbian Cyrillic (Српски)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Bojan Maksimović, panickingkernel, Zoran Jankov, Stefan Šućur, Kosta Stojiljković",

        "STR_ERROR":"Грешка",
        "STR_WARNING":"Упозорење",
        "STR_INFO":"Информација",
        "STR_INCORRECT_DIR":"Молим Вас, покрените ме у правом директоријуму!",
        "STR_INCORRECT_TREE_DIR":"Немојте ме покретати овде, молим Вас преузмите објављени инсталациони пакет и покрените ме тамо.",
        "STR_DEVICE":"Уређај",
        "STR_LOCAL_VER":"Ventoy у пакету",
        "STR_DISK_VER":"Ventoy у уређају",
        "STR_STATUS":"Статус - СПРЕМАН",
        "STR_INSTALL":"Инсталирај",
        "STR_UPDATE":"Надогради",
        "STR_UPDATE_TIP":"Надоградња је сигурна, ISO датотеке неће бити промењене.#@Настави?",
        "STR_INSTALL_TIP":"Диск ће бити форматиран и сви подаци ће бити изгубљени.#@Настави?",
        "STR_INSTALL_TIP2":"Диск ће бити форматиран и сви подаци ће бити изгубљени.#@Настави? (Двострука провера)",
        "STR_INSTALL_SUCCESS":"Честитам!#@Ventoy је успешно инсталиран на овом уређају.",
        "STR_INSTALL_FAILED":"Догодила се грешка током инсталације. Можете извадити и поново убацити USB, па покушати опет. Погледајте /var/log/ventoy.log за још детаља.",
        "STR_UPDATE_SUCCESS":"Честитам!#@Ventoy је био успешно надограђен на овом уређају.",
        "STR_UPDATE_FAILED":"Догодила се грешка током надоградње. Можете извадити и поново убацити USB, па покушати опет. Погледајте /var/log/ventoy.log за још детаља.",
        "STR_WAIT_PROCESS":"Одвија се операција, молим Вас сачекајте...",
        "STR_MENU_OPTION":"Опције",
        "STR_MENU_SECURE_BOOT":"Сигурно укључивање је подржано",
        "STR_MENU_PART_CFG":"Конфигурација партиција",
        "STR_BTN_OK":"У реду",
        "STR_BTN_CANCEL":"Откажи",
        "STR_PRESERVE_SPACE":"Сачувај нешто простора на крају диска",
        "STR_SPACE_VAL_INVALID":"Неисправна вредност за презервирани простор",
        "STR_MENU_CLEAR":"Обриши Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy је успешно обрисан са уређаја",
        "STR_CLEAR_FAILED":"Догодила се грешка током брисања Ventoy са диска. Можете поново убацити USB и покушати опет. Погледајте /var/log/ventoy.log за још детаља.",
        "STR_MENU_PART_STYLE":"Стил партиција",
        "STR_DISK_2TB_MBR_ERROR":"Молим Вас, изаберите GPT за дискове преко 2ТB",
        "STR_SHOW_ALL_DEV":"Прикажи све уређаје",
        "STR_PART_ALIGN_4KB":"Поравнајте партиције са 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Комуникациона грешка:",
        "STR_WEB_REMOTE_ABNORMAL":"Комуникациона грешка: Ненормално даљинско управљање",
        "STR_WEB_REQUEST_TIMEOUT":"Комуникациона грешка: Захтев је истекао",
        "STR_WEB_SERVICE_UNAVAILABLE":"Комуникациона грешка: Сервис је недоступан",
        "STR_WEB_TOKEN_MISMATCH":"Статус демона ажуриран, покушајте поново касније.",
        "STR_WEB_SERVICE_BUSY":"Севис је заузет, покушајте поново касније.",
        "STR_MENU_VTSI_CREATE":"Створи VTSI датотеку",
        "STR_VTSI_CREATE_TIP":"Овај пут не писати на уређај, него само створити VTSI датотеку#@Настави?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI датотека успешно створена!#@Можете користити Rufus(3.15+) да је запишете на уређај да бисте завршили инсталацију Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Грешка у стварању VTSI датотеке.",
        "STR_MENU_PART_RESIZE":"Недеструктивна инсталација",
        "STR_PART_RESIZE_TIP":"Ventoy ће покушати недеструктивну инсталацију ако је могуће. #@Настави?",
        "STR_PART_RESIZE_SUCCESS":"Честитам!#@Ventoy недеструктивна инсталација је била успешна.",
        "STR_PART_RESIZE_FAILED":"Недеструктивна инсталација је била неуспешна. Погледајте /var/log/ventoy.log за више детаља.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy недеструктивна инсталација се зауставила јер нису постигнути неки услови. Погледајте /var/log/ventoy.log за још детаља.",
        "STR_INSTALL_YES_TIP1":"Упозорење: подаци ће бити изгубљени!",
        "STR_INSTALL_YES_TIP2":"Молим Вас, унесите YES у поље испод да бисте потврдили да заиста желите да урадите нову инсталацију уместо надоградње.",
        "STR_PART_VENTOY_FS":"Датотечни систем за Ventoy партицију",
        "STR_PART_FS":"Датотечни систем",
        "STR_PART_CLUSTER":"Величина кластера",
        "STR_PART_CLUSTER_DEFAULT":"Подразумевана системска вредност",
        "STR_DONATE":"Донирајте",
        "STR_4KN_UNSUPPORTED":"Ventoy тренутно не подржава 4К изворне уређаје.",
        
        "STRXXX":""
    },
    {
        "name":"Thai (ไทย)",
        "FontFamily":"Courier New",
        "FontSize":17,
        "Author":"longpanda",
        
        "STR_ERROR":"ผิดพลาด",
        "STR_WARNING":"คำเตือน",
        "STR_INFO":"ข้อมูล",
        "STR_INCORRECT_DIR":"กรุณาเรียกใช้ในอุปกรณ์ที่ถูกต้อง!",
        "STR_INCORRECT_TREE_DIR":"อย่าติดตั้งในนี้, โปรดดาวน์โหลดและใช้แพ็คเกจการติดตั้งของรุ่นที่วางจำหน่าย",
        "STR_DEVICE":"อุปกรณ์",
        "STR_LOCAL_VER":"เวอร์ชั่น Ventoy ",
        "STR_DISK_VER":"ลง Ventoy ในอุปกรณ์",
        "STR_STATUS":"สถานะ - พร้อม",
        "STR_INSTALL":"ติดตั้ง",
        "STR_UPDATE":"ปรับปรุง",
        "STR_UPDATE_TIP":"การดำเนินการอัปเกรดปลอดภัย, ไฟล์ ISO จะไม่มีการเปลี่ยนแปลง.#@ต่อเนื่อง?",
        "STR_INSTALL_TIP":"ดิสก์จะถูกฟอร์แมตและข้อมูลทั้งหมดจะสูญหาย.#@ต่อเนื่อง?",
        "STR_INSTALL_TIP2":"ดิสก์จะถูกฟอร์แมตและข้อมูลทั้งหมดจะสูญหาย#@ต่อเนื่อง? (ตรวจสอบอีกครั้ง)",
        "STR_INSTALL_SUCCESS":"ขอแสดงความยินดี!#@Ventoy ติดตั้งไปยังอุปกรณ์สำเร็จแล้ว",
        "STR_INSTALL_FAILED":"เกิดข้อผิดพลาดระหว่างการติดตั้ง. คุณสามารถถอด USB และลองอีกครั้ง. ตรวจสอบรายละเอียดได้ที่ /var/log/ventoy.log.",
        "STR_UPDATE_SUCCESS":"ขอแสดงความยินดี!#@Ventoy อัปเดตไปยังอุปกรณ์สำเร็จแล้ว.",
        "STR_UPDATE_FAILED":"เกิดข้อผิดพลาดระหว่างการอัพเดต. คุณสามารถถอด USB และลองอีกครั้ง. ตรวจสอบรายละเอียดได้ที่ /var/log/ventoy.log.",
        "STR_WAIT_PROCESS":"CPU กำลังทำงานอยู่,ได้โปรดรอ...",
        "STR_MENU_OPTION":"ตัวเลือก",
        "STR_MENU_SECURE_BOOT":"สนับสนุนการบูตที่ปลอดภัย",
        "STR_MENU_PART_CFG":"กำหนดค่าพาร์ติชัน",
        "STR_BTN_OK":"ตกลง",
        "STR_BTN_CANCEL":"ยกเลิก",
        "STR_PRESERVE_SPACE":"เก็บพื้นที่บางส่วนของอุปกรณ์ไว้",
        "STR_SPACE_VAL_INVALID":"มีปัญหาสำหรับพื้นที่ที่สงวนไว้(กรุณา ฟอร์แมต และติดตั้งใหม่)",
        "STR_MENU_CLEAR":"Clear Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy has been successfully removed from the device.",
        "STR_CLEAR_FAILED":"An error occurred when clear Ventoy from disk. You can replug the USB and try again. Check /var/log/ventoy.log for detail.",
        "STR_MENU_PART_STYLE":"Partition Style",
        "STR_DISK_2TB_MBR_ERROR":"Please select GPT for disk over 2TB",
        "STR_SHOW_ALL_DEV":"Show All Devices",
        "STR_PART_ALIGN_4KB":"Align partitions with 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Communication error:",
        "STR_WEB_REMOTE_ABNORMAL":"Communication error: remote abnormal",
        "STR_WEB_REQUEST_TIMEOUT":"Communication error: Request timed out",
        "STR_WEB_SERVICE_UNAVAILABLE":"Communication error: Service Unavailable",
        "STR_WEB_TOKEN_MISMATCH":"Daemon status updated, please retry later.",
        "STR_WEB_SERVICE_BUSY":"Service is busy, please retry later.",
        "STR_MENU_VTSI_CREATE":"Generate VTSI File",
        "STR_VTSI_CREATE_TIP":"This time will not write to the device, but only generate a VTSI file#@Continue?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI file created successfully!#@You can use Rufus(3.15+) to write it to the device so as to complete the installation of Ventoy.",
        "STR_VTSI_CREATE_FAILED":"VTSI file created failed.",
        "STR_MENU_PART_RESIZE":"Non-destructive Install",
        "STR_PART_RESIZE_TIP":"Ventoy will try non-destructive installation if possible. #@Continue?",
        "STR_PART_RESIZE_SUCCESS":"Congratulations!#@Ventoy non-destructive installation successfully finished.",
        "STR_PART_RESIZE_FAILED":"Non-destructive installation failed, Check /var/log/ventoy.log for details.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy non-destructive installation stopped because some conditions cannot be met. Check /var/log/ventoy.log for details.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"บริจาค",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Norwegian (Norsk)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Stein-Ove Bøthun",

        "STR_ERROR":"Feil",
        "STR_WARNING":"Advarsel",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Vennligst kjør fra den riktige mappen!",
        "STR_INCORRECT_TREE_DIR":"Ikke kjør meg her, vennligst last ned den utgitte installasjonspakken og kjør den der.",
        "STR_DEVICE":"Enhet",
        "STR_LOCAL_VER":"Ventoy i Pakken",
        "STR_DISK_VER":"Ventoy på Enheten",
        "STR_STATUS":"Status - KLAR",
        "STR_INSTALL":"Installer",
        "STR_UPDATE":"Oppdater",
        "STR_UPDATE_TIP":"Oppgraderingen er trygg, ISO-filene vil være uendret.#@Fortsette?",
        "STR_INSTALL_TIP":"Disken blir formatert og alle dataene vil gå tapt.#@Fortsette?",
        "STR_INSTALL_TIP2":"Disken blir formatert og alle dataene vil gå tapt.#@Fortsette? (Dobbelsjekk)",
        "STR_INSTALL_SUCCESS":"Gratulerer!#@Ventoy er blitt installert på enheten.",
        "STR_INSTALL_FAILED":"Det oppstod en feil under installasjonen. Du kan koble til USB-en på nytt og prøve igjen. Check /var/log/ventoy.log for flere detaljer.",
        "STR_UPDATE_SUCCESS":"Gratulerer!#@Ventoy har blitt oppdatert på enheten.",
        "STR_UPDATE_FAILED":"Det oppstod en feil under oppdateringen. Du kan koble til USB-en på nytt og prøve igjen. Se /var/log/ventoy.log for flere detaljer.",
        "STR_WAIT_PROCESS":"En prosess kjører, vennligst vent...",
        "STR_MENU_OPTION":"Alternativ",
        "STR_MENU_SECURE_BOOT":"Støtte for sikker oppstart",
        "STR_MENU_PART_CFG":"Partisjonskonfigirasjon",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Avbryt",
        "STR_PRESERVE_SPACE":"Bevar litt plass på bunnen av disken",
        "STR_SPACE_VAL_INVALID":"Ugyldig verdi for reservert plass",
        "STR_MENU_CLEAR":"Fjern Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy er blitt fjernet fra enheten.",
        "STR_CLEAR_FAILED":"Det oppstod en feil da Ventoy bler fjernet fra disken. Du kan koble til USB-en på nytt og prøve igjen. Sjekk /var/log/ventoy.log for flere detaljer.",
        "STR_MENU_PART_STYLE":"Partisjonsstil",
        "STR_DISK_2TB_MBR_ERROR":"Velg GPT for disker over 2 TB",
        "STR_SHOW_ALL_DEV":"Vis alle enheter",
        "STR_PART_ALIGN_4KB":"Juster partisjoner med 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Kommunikasjonsfeil:",
        "STR_WEB_REMOTE_ABNORMAL":"Kommunokasjonsfeil: unormalt fjerntliggende",
        "STR_WEB_REQUEST_TIMEOUT":"Kommunikasjonsfeil: Forespørselen ble tidsavbrutt",
        "STR_WEB_SERVICE_UNAVAILABLE":"Kommunikasjonsfeil: Tjeneste utilgjengelig",
        "STR_WEB_TOKEN_MISMATCH":"Daemon status er oppdatert, vennligst prøv igjen senere.",
        "STR_WEB_SERVICE_BUSY":"Tjenesten er opptatt, vennligst prøv igjen senere.",
        "STR_MENU_VTSI_CREATE":"Generate VTSI File",
        "STR_VTSI_CREATE_TIP":"This time will not write to the device, but only generate a VTSI file#@Continue?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI file created successfully!#@You can use Rufus(3.15+) to write it to the device so as to complete the installation of Ventoy.",
        "STR_VTSI_CREATE_FAILED":"VTSI file created failed.",
        "STR_MENU_PART_RESIZE":"Non-destructive Install",
        "STR_PART_RESIZE_TIP":"Ventoy will try non-destructive installation if possible. #@Continue?",
        "STR_PART_RESIZE_SUCCESS":"Congratulations!#@Ventoy non-destructive installation successfully finished.",
        "STR_PART_RESIZE_FAILED":"Non-destructive installation failed, Check /var/log/ventoy.log for details.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy non-destructive installation stopped because some conditions cannot be met. Check /var/log/ventoy.log for details.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"donere",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Vietnamese (Tiếng Việt)",
        "FontFamily":"Tahoma",
        "FontSize":14,
        "Author":"Nguyen Quoc Hoang - cuumay.win, HoangNPV - @isoamyl50",
        
        "STR_ERROR":"Lỗi",
        "STR_WARNING":"Cảnh báo",
        "STR_INFO":"Thông tin",
        "STR_INCORRECT_DIR":"Vui lòng chạy Ventoy2Disk tại đúng thư mục của nó !",
        "STR_INCORRECT_TREE_DIR":"Không được chạy Ventoy2Disk ở đây, vui lòng tải gói cài đặt đã phát hành và chạy ở đó.",
        "STR_DEVICE":"Thiết bị",
        "STR_LOCAL_VER":"Phiên bản Ventoy",
        "STR_DISK_VER":"Phiên bản Ventoy ở thiết bị",
        "STR_STATUS":"Trạng thái - SẴN SÀNG",
        "STR_INSTALL":"Cài đặt",
        "STR_UPDATE":"Cập nhật",
        "STR_UPDATE_TIP":"Việc cập nhật Ventoy là an toàn, các tập tin ISO của bạn sẽ được giữ nguyên trạng.#@Bạn thật sự muốn tiếp tục?",
        "STR_INSTALL_TIP":"Thiết bị sẽ được định dạng và do đó TẤT CẢ DỮ LIỆU trên thiết bị sẽ bị mất.#@Bạn thật sự muốn tiếp tục?",
        "STR_INSTALL_TIP2":"Thiết bị sẽ được định dạng và do đó TẤT CẢ DỮ LIỆU trên thiết bị sẽ bị mất.#@Bạn thật sự muốn tiếp tục? (Xác nhận lần hai)",
        "STR_INSTALL_SUCCESS":"Chúc mừng bạn!#@Thiết bị đã được cài Ventoy thành công.",
        "STR_INSTALL_FAILED":"Đã xảy ra lỗi trong quá trình cài đặt Ventoy. Bạn có thể cắm lại USB và thử lại. Xem chi tiết ở tệp /var/log/ventoy.log. Nếu lỗi tiếp tục xảy ra, vui lòng đọc FAQ tại trang web chính thức.",
        "STR_UPDATE_SUCCESS":"Chúc mừng bạn!#@Thiết bị đã được cập nhật Ventoy thành công.",
        "STR_UPDATE_FAILED":"Đã xảy ra lỗi trong quá trình cập nhật Ventoy. Bạn có thể cắm lại USB và thử lại. Xem chi tiết ở tệp /var/log/ventoy.log. Nếu lỗi tiếp tục xảy ra, vui lòng đọc FAQ tại trang web chính thức.",
        "STR_WAIT_PROCESS":"Một luồng xử lý đang chạy, vui lòng chờ...",
        "STR_MENU_OPTION":"Tùy chọn",
        "STR_MENU_SECURE_BOOT":"Bật hỗ trợ Secure Boot",
        "STR_MENU_PART_CFG":"Cấu hình phân vùng",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Hủy",
        "STR_PRESERVE_SPACE":"Giữ lại phần dung lượng ở cuối thiết bị",
        "STR_SPACE_VAL_INVALID":"Giá trị dung lượng giữ lại không hợp lệ.",
        "STR_MENU_CLEAR":"Gỡ bỏ Ventoy",
        "STR_CLEAR_SUCCESS":"Chúc mừng bạn!#@Thiết bị đã được gỡ bỏ Ventoy thành công.",
        "STR_CLEAR_FAILED":"Đã xảy ra lỗi trong quá trình gỡ bỏ Ventoy. Bạn có thể cắm lại USB và thử lại. Xem chi tiết ở tệp /var/log/ventoy.log.",
        "STR_MENU_PART_STYLE":"Kiểu phân vùng",
        "STR_DISK_2TB_MBR_ERROR":"Thiết bị có dung lượng lớn hơn 2TB. Vui lòng chọn Kiểu phân vùng là GPT.",
        "STR_SHOW_ALL_DEV":"Hiện tất cả Thiết bị",
        "STR_PART_ALIGN_4KB":"Căn chỉnh phân vùng với 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Lỗi giao tiếp:",
        "STR_WEB_REMOTE_ABNORMAL":"Lỗi giao tiếp: Remote bất thường",
        "STR_WEB_REQUEST_TIMEOUT":"Lỗi giao tiếp: Yêu cầu đã hết thời gian chờ",
        "STR_WEB_SERVICE_UNAVAILABLE":"Lỗi giao tiếp: Dịch vụ không sẵn có",
        "STR_WEB_TOKEN_MISMATCH":"Đã cập nhật trạng thái Daemon, vui lòng thử lại sau.",
        "STR_WEB_SERVICE_BUSY":"Dịch vụ bận, vui lòng thử lại sau.",
        "STR_MENU_VTSI_CREATE":"Tạo tệp VTSI",
        "STR_VTSI_CREATE_TIP":"Lần này sẽ không ghi vào thiết bị, chỉ tạo một tệp VTSI#@Bạn muốn tiếp tục?",
        "STR_VTSI_CREATE_SUCCESS":"Tạo tệp VTSI thành công!#@Để hoàn thành cài đặt Ventoy, bạn có thể dùng Rufus (3.15+) để ghi tệp này vào thiết bị.",
        "STR_VTSI_CREATE_FAILED":"Tạo tệp VTSI đã gặp lỗi.",
        "STR_MENU_PART_RESIZE":"Cài đặt không phá hủy",
        "STR_PART_RESIZE_TIP":"Ventoy sẽ thử cài đặt không phá hủy nếu có thể. #@Bạn muốn tiếp tục?",
        "STR_PART_RESIZE_SUCCESS":"Chúc mừng bạn!#@Thiết bị đã được cài Ventoy thành công theo cách cài đặt không phá hủy.",
        "STR_PART_RESIZE_FAILED":"Đã xảy ra lỗi trong quá trình cài Ventoy theo cách cài đặt không phá hủy. Xem chi tiết ở tệp /var/log/ventoy.log.",
        "STR_PART_RESIZE_UNSUPPORTED":"Cài Ventoy theo cách cài đặt không phá hủy đã bị dừng lại vì chưa đúng điều kiện. Xem chi tiết ở tệp /var/log/ventoy.log.",
        "STR_INSTALL_YES_TIP1":"Cảnh báo: Dữ liệu sẽ bị mất !",
        "STR_INSTALL_YES_TIP2":"Vui lòng nhập YES vào ô bên dưới để xác nhận bạn muốn cài mới thay vì cài nâng cấp.",
        "STR_PART_VENTOY_FS":"Hệ thống tập tin cho Phân vùng Ventoy",
        "STR_PART_FS":"Hệ thống tập tin",
        "STR_PART_CLUSTER":"Kích thước Cluster",
        "STR_PART_CLUSTER_DEFAULT":"Theo giá trị mặc định của hệ thống",
        "STR_DONATE":"Ủng hộ tác giả",
        "STR_4KN_UNSUPPORTED":"Hiện tại Ventoy không hỗ trợ các thiết bị 4K native.",
        
        "STRXXX":""
    },
    {
        "name":"Lithuanian (Lietuvių)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"r0manas",
        
        "STR_ERROR ":"Klaida",
        "STR_WARNING ":"Įspėjimas",
        "STR_INFO ":"Informacija",
        "STR_INCORRECT_DIR ":"Prašome paleisti teisingame kataloge!",
        "STR_INCORRECT_TREE_DIR ":"Nepaleiskite manęs čia, atsisiųskite išleistą diegimo paketą ir paleiskite ten.",
        "STR_DEVICE ":"Įrenginys",
        "STR_LOCAL_VER ":"„Ventoy“ versija pakuotėje",
        "STR_DISK_VER ":"„Ventoy“ versija įrenginyje",
        "STR_STATUS ":"Būsena - PASIRENGĘS",
        "STR_INSTALL ":"Įdiegti",
        "STR_UPDATE ":"Atnaujinti",
        "STR_UPDATE_TIP ":"Atnaujinti - saugu, ISO failai liks nepažeisti.#@Tęsti?",
        "STR_INSTALL_TIP ":"Įrenginys bus suformatuotas ir visi duomenys bus prarasti.#@Tęsti?",
        "STR_INSTALL_TIP2 ":"Įrenginys bus suformatuotas ir visi duomenys bus prarasti.#@Tęsti? (TIKRAI TĘSTI?)",
        "STR_INSTALL_SUCCESS ":"Sveikinu!#@Ventoy sėkmingai įdiegtas įrenginyje.",
        "STR_INSTALL_FAILED ":"Diegimo metu įvyko klaida. Galite iš naujo prijungti USB ir bandyti dar kartą. Patikrinkite /var/log/ventoy.log, jei norite sužinoti daugiau.",
        "STR_UPDATE_SUCCESS ":"Sveikinu!#@Ventoy sėkmingai atnaujintas įrenginyje.",
        "STR_UPDATE_FAILED ":"Atnaujinant įvyko klaida. Galite iš naujo prijungti USB ir bandyti dar kartą. Patikrinkite /var/log/ventoy.log, jei norite sužinoti daugiau.",
        "STR_WAIT_PROCESS ":"Pradėtas procesas, palaukite ...",
        "STR_MENU_OPTION ":"Nustatymai",
        "STR_MENU_SECURE_BOOT ":"Secure Boot palaikymas",
        "STR_MENU_PART_CFG ":"Skirsnio konfigūracija",
        "STR_BTN_OK ":"Gerai",
        "STR_BTN_CANCEL ":"Atšaukti",
        "STR_PRESERVE_SPACE ":"Rezervuoti vietos įrenginyje",
        "STR_SPACE_VAL_INVALID ":"Neteisinga rezervuotos vietos vertė",
        "STR_MENU_CLEAR ":"Pašalinti „Ventoy“",
        "STR_CLEAR_SUCCESS ":"„Ventoy“ sėkmingai pašalintas iš įrenginio.",
        "STR_CLEAR_FAILED ":"Įvyko klaida pašalinant „Ventoy“ iš įrenginio. USB ir bandyti dar kartą. Patikrinkite /var/log/ventoy.log, jei norite sužinoti daugiau.",
        "STR_MENU_PART_STYLE ":"Skirsnio formatas",
        "STR_DISK_2TB_MBR_ERROR ":"Prašome pasirinkti GPT, jei įrenginys didesnis nei 2 TB.",
        "STR_SHOW_ALL_DEV":"Show All Devices",
        "STR_PART_ALIGN_4KB":"Align partitions with 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Communication error:",
        "STR_WEB_REMOTE_ABNORMAL":"Communication error: remote abnormal",
        "STR_WEB_REQUEST_TIMEOUT":"Communication error: Request timed out",
        "STR_WEB_SERVICE_UNAVAILABLE":"Communication error: Service Unavailable",
        "STR_WEB_TOKEN_MISMATCH":"Daemon status updated, please retry later.",
        "STR_WEB_SERVICE_BUSY":"Service is busy, please retry later.",
        "STR_MENU_VTSI_CREATE":"Generate VTSI File",
        "STR_VTSI_CREATE_TIP":"This time will not write to the device, but only generate a VTSI file#@Continue?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI file created successfully!#@You can use Rufus(3.15+) to write it to the device so as to complete the installation of Ventoy.",
        "STR_VTSI_CREATE_FAILED":"VTSI file created failed.",
        "STR_MENU_PART_RESIZE":"Non-destructive Install",
        "STR_PART_RESIZE_TIP":"Ventoy will try non-destructive installation if possible. #@Continue?",
        "STR_PART_RESIZE_SUCCESS":"Congratulations!#@Ventoy non-destructive installation successfully finished.",
        "STR_PART_RESIZE_FAILED":"Non-destructive installation failed, Check /var/log/ventoy.log for details.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy non-destructive installation stopped because some conditions cannot be met. Check /var/log/ventoy.log for details.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"paaukoti",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Macedonian (Македонски)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Влатко Стојанов",
        
        "STR_ERROR":"Грешка",
        "STR_WARNING":"Предупредување",
        "STR_INFO":"Инфо",
        "STR_INCORRECT_DIR":"Ве молам покренете ме од коректната папка!",
        "STR_INCORRECT_TREE_DIR":"Не покренувајте ме овде, ве молам превземете ја последната верзија и покренете ме таму.",
        "STR_DEVICE":"УСБ уред",
        "STR_LOCAL_VER":"Ventoy во пакетот",
        "STR_DISK_VER":"Ventoy на УСБ уредот",
        "STR_STATUS":"Статус - Спремен",
        "STR_INSTALL":"Инсталирај",
        "STR_UPDATE":"Ажурирај",
        "STR_UPDATE_TIP":"Операцијата ажурирање е безбедна, ISO датотеките нема да бидат променети.#@Продолжи?",
        "STR_INSTALL_TIP":"УСБ уредот ќе биде форматиран и сите податоци ќе бидат уништени.#@Продолжи?",
        "STR_INSTALL_TIP2":"УСБ уредот ќе биде форматиран и сите податоци ќе бидат уништени.#@Продолжи? (Двојна проверка)",
        "STR_INSTALL_SUCCESS":"Честитки!#@Ventoy е успешно инсталиран на УСБ уредот.",
        "STR_INSTALL_FAILED":"Се појави грешка при инсталацијата. Отстранете го УСБ уредот и пробајте повторно. Проверете го /var/log/ventoy.log за детали.",
        "STR_UPDATE_SUCCESS":"Честитки!#@Ventoy е успешно ажуриран на УСБ уредот.",
        "STR_UPDATE_FAILED":"Се појави грешка при ажурирањето. Отстранете го УСБ уредот и пробајте повторно. Проверете го /var/log/ventoy.log за детали.",
        "STR_WAIT_PROCESS":"Процесот е вклучен, ве молиме почекајте...",
        "STR_MENU_OPTION":"Опции",
        "STR_MENU_SECURE_BOOT":"Secure Boot Поддршка",
        "STR_MENU_PART_CFG":"Конфигурирање на партиции",
        "STR_BTN_OK":"ОК",
        "STR_BTN_CANCEL":"Излез",
        "STR_PRESERVE_SPACE":"Резервирај место на крајот на дискот",
        "STR_SPACE_VAL_INVALID":"Невалидна вредност за резервираното место",
        "STR_MENU_CLEAR":"Отстрани го Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy е успешно отстранет од УСБ уредот.",
        "STR_CLEAR_FAILED":"Се појави грешка при отстранувањето на Ventoy од уредот. Отстранете го УСБ уредот и пробајте повторно. Проверете го /var/log/ventoy.log за детали.",
        "STR_MENU_PART_STYLE":"Тип на партиција",
        "STR_DISK_2TB_MBR_ERROR":"Ве молиме изберете GPT за уреди поголеми од 2TB",
        "STR_SHOW_ALL_DEV":"Прикажи ги сите уреди",
        "STR_PART_ALIGN_4KB":"Порамнете ги партициите со 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Комуникациска грешка:",
        "STR_WEB_REMOTE_ABNORMAL":"Комуникациска грешка: Абнормално далечинско управување",
        "STR_WEB_REQUEST_TIMEOUT":"Комуникациска грешка: Барањето е истечено",
        "STR_WEB_SERVICE_UNAVAILABLE":"Комуникациска грешка: Недостапен сервис",
        "STR_WEB_TOKEN_MISMATCH":"Ажуриран статус на даемонот, молиме обидете се подоцна.",
        "STR_WEB_SERVICE_BUSY":"Сервисот е зафатен, молиме обидете се подоцна.",
        "STR_MENU_VTSI_CREATE":"Генерирај VTSI датотека",
        "STR_VTSI_CREATE_TIP":"Овој пат ништо нема да се запише на УСБ уредот туку само ќе се генерира VTSI датотека#@Продолжи?",
        "STR_VTSI_CREATE_SUCCESS":"Успешно креирана VTSI датотека!#@Можете да го користите Rufus(3.15+) за да ја запишете на уредот како и да ја комплетирате инсталацијата на Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Креирањето на VTSI датотека е неуспешно.",
        "STR_MENU_PART_RESIZE":"Недеструктивна инсталација",
        "STR_PART_RESIZE_TIP":"Ventoy ќе се обиде со недеструктивна инсталација доколку е возможно. #@Продолжи?",
        "STR_PART_RESIZE_SUCCESS":"Честитки! Недеструктивната инсталација е успешно завршена.",
        "STR_PART_RESIZE_FAILED":"Недеструктивната инсталација не успеа. Проверете го /var/log/ventoy.log за детали.",
        "STR_PART_RESIZE_UNSUPPORTED":"Недеструктивната инсталација на Ventoy е запрена бидејќи некои услови не се исполнети. Проверете го /var/log/ventoy.log за детали.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"донира",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Hebrew (עברית)",
        "FontFamily":"tahoma",
        "FontSize":16,
        "Author":"chaim-chv",
        
        "STR_ERROR":"תקלה",
        "STR_WARNING":"אזהרה",
        "STR_INFO":"מידע",
        "STR_INCORRECT_DIR":"הרץ בבקשה בתיקייה הנכונה!",
        "STR_INCORRECT_TREE_DIR":"אל תריץ אותי כאן. בבקשה הורד את חבילת ההתקנה ששוחררה, ותריץ אותי שם.",
        "STR_DEVICE":"התקן",
        "STR_LOCAL_VER":"גרסת Ventoy מקומית",
        "STR_DISK_VER":"גרסת Ventoy בהתקן",
        "STR_STATUS":"סטטוס - מוכן",
        "STR_INSTALL":"התקנה",
        "STR_UPDATE":"עדכון",
        "STR_UPDATE_TIP":"אופציית העדכון היא בטוחה לשימוש. קבצי ה-ISO לא ייפגעו.#@להמשיך?",
        "STR_INSTALL_TIP":"הדיסק יפורמט וכל המידע שבו ייעלם.#@להמשיך?",
        "STR_INSTALL_TIP2":"הדיסק יפורמט וכל המידע שבו ייעלפ ויימחק לחלוטין.#@להמשיך? (בדיקה לווידוא)",
        "STR_INSTALL_SUCCESS":"הצלחה!#@Ventoy הותקנה בהצלחה על הדיסק הנבחר",
        "STR_INSTALL_FAILED":"אירעה תקלה בניסיון ההתקנה. אפשר לנסות לחבר את הדיסק מחדש ולבצע ניסיון התקנה נוסף. עיין בקובץ ה-/var/log/ventoy.log לפרטים נוספים.",
        "STR_UPDATE_SUCCESS":"הצלחה!#@Ventoy הותקנה בהצלחה על הדיסק הנבחר",
        "STR_UPDATE_FAILED":"אירעה תקלה בניסיון העדכון. אפשר לנסות לחבר את הדיסק מחדש ולבצע ניסיון נוסף לעדכון. עיין בקובץ ה-/var/log/ventoy.log לפרטים נוספים.",
        "STR_WAIT_PROCESS":"עובד... נא המתן",
        "STR_MENU_OPTION":"אפשרויות",
        "STR_MENU_SECURE_BOOT":"תמיכה ב-Secure Boot",
        "STR_MENU_PART_CFG":"הגדרת מחיצות",
        "STR_BTN_OK":"אישור",
        "STR_BTN_CANCEL":"ביטול",
        "STR_PRESERVE_SPACE":"הגדר מקום לשמירה בדיסק",
        "STR_SPACE_VAL_INVALID":"ערך לא חוקי עבור שטח לשמירה",
        "STR_MENU_CLEAR":"נקה את Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy נמחקה בהצלחה מהדיסק הנבחר.",
        "STR_CLEAR_FAILED":"אירעה תקלה בניסיון למחוק את Ventoy מהדיסק הנבחר. אפשר לנסות לחבר את הדיסק מחדש ולבצע שוב מחיקה. עיין בקובץ ה-/var/log/ventoy.log לפרטים נוספים.",
        "STR_MENU_PART_STYLE":"סוג מחיצה",
        "STR_DISK_2TB_MBR_ERROR":"בחר ב-GPT עבור דיסק שגדול מ-2TB",
        "STR_SHOW_ALL_DEV":"Show All Devices",
        "STR_PART_ALIGN_4KB":"Align partitions with 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Communication error:",
        "STR_WEB_REMOTE_ABNORMAL":"Communication error: remote abnormal",
        "STR_WEB_REQUEST_TIMEOUT":"Communication error: Request timed out",
        "STR_WEB_SERVICE_UNAVAILABLE":"Communication error: Service Unavailable",
        "STR_WEB_TOKEN_MISMATCH":"Daemon status updated, please retry later.",
        "STR_WEB_SERVICE_BUSY":"Service is busy, please retry later.",
        "STR_MENU_VTSI_CREATE":"Generate VTSI File",
        "STR_VTSI_CREATE_TIP":"This time will not write to the device, but only generate a VTSI file#@Continue?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI file created successfully!#@You can use Rufus(3.15+) to write it to the device so as to complete the installation of Ventoy.",
        "STR_VTSI_CREATE_FAILED":"VTSI file created failed.",
        "STR_MENU_PART_RESIZE":"Non-destructive Install",
        "STR_PART_RESIZE_TIP":"Ventoy will try non-destructive installation if possible. #@Continue?",
        "STR_PART_RESIZE_SUCCESS":"Congratulations!#@Ventoy non-destructive installation successfully finished.",
        "STR_PART_RESIZE_FAILED":"Non-destructive installation failed, Check /var/log/ventoy.log for details.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy non-destructive installation stopped because some conditions cannot be met. Check /var/log/ventoy.log for details.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"לִתְרוֹם",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Portuguese (Português de Portugal)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Eskiso, Hugo Carvalho",
        
        "STR_ERROR":"Erro",
        "STR_WARNING":"Aviso",
        "STR_INFO":"Informação",
        "STR_INCORRECT_DIR":"Execute na pasta correta!",
        "STR_INCORRECT_TREE_DIR":"Não me execute aqui, transfira o pacote de instalação e execute lá.",
        "STR_DEVICE":"Dispositivo",
        "STR_LOCAL_VER":"Ventoy em pacote",
        "STR_DISK_VER":"Ventoy no dispositivo",
        "STR_STATUS":"Estado - PRONTO",
        "STR_INSTALL":"Instalar",
        "STR_UPDATE":"Atualizar",
        "STR_UPDATE_TIP":"A operação de atualização é segura, os ficheiros ISO não serão alterados.#@Continuar?",
        "STR_INSTALL_TIP":"O disco será formatado e todos os dados serão perdidos.#@Continuar?",
        "STR_INSTALL_TIP2":"O disco será formatado e todos os dados serão perdidos.#@Continuar? (Confirmação)",
        "STR_INSTALL_SUCCESS":"Parabéns!#@Ventoy foi instalado com sucesso no dispositivo.",
        "STR_INSTALL_FAILED":"Ocorreu um erro durante a instalação. Pode voltar a ligar o dispositivo USB e tentar novamente. Verifique o ficheiro /var/log/ventoy.log para mais detalhes.",
        "STR_UPDATE_SUCCESS":"Parabéns!#@Ventoy foi atualizado com sucesso no dispositivo.",
        "STR_UPDATE_FAILED":"Ocorreu um erro durante a atualização. Pode voltar a ligar o dispositivo USB e tentar novamente. Verifique o ficheiro /var/log/ventoy.log para mais detalhes.",
        "STR_WAIT_PROCESS":"Uma thread está em execução, aguarde...",
        "STR_MENU_OPTION":"Opção",
        "STR_MENU_SECURE_BOOT":"Arranque seguro",
        "STR_MENU_PART_CFG":"Configuração da partição",
        "STR_BTN_OK":"Aceitar",
        "STR_BTN_CANCEL":"Cancelar",
        "STR_PRESERVE_SPACE":"Reservar algum espaço no final do disco",
        "STR_SPACE_VAL_INVALID":"Valor inválido para o espaço reservado",
        "STR_MENU_CLEAR":"Remover o Ventoy",
        "STR_CLEAR_SUCCESS":"O Ventoy foi removido deste dispositivo com sucesso.",
        "STR_CLEAR_FAILED":"Um erro ocorreu ao remover o Ventoy do disco. Pode voltar a ligar o dispositivo USB e tentar novamente. Verifique o ficheiro /var/log/ventoy.log para mais detalhes.",
        "STR_MENU_PART_STYLE":"Estilo de partição",
        "STR_DISK_2TB_MBR_ERROR":"Selecione GPT para discos maiores que 2TB",
        "STR_SHOW_ALL_DEV":"Mostrar todos os dispositivos",
        "STR_PART_ALIGN_4KB":"Alinhar as partições com 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Erro de comunicação:",
        "STR_WEB_REMOTE_ABNORMAL":"Erro de comunicação: Anomalia remota",
        "STR_WEB_REQUEST_TIMEOUT":"Erro de comunicação: Pedido de tempo limite",
        "STR_WEB_SERVICE_UNAVAILABLE":"Erro de comunicação: Serviço indisponível",
        "STR_WEB_TOKEN_MISMATCH":"Estado do daemon atualizado, tente novamente mais tarde.",
        "STR_WEB_SERVICE_BUSY":"Serviço está ocupado, tente novamente mais tarde.",
        "STR_MENU_VTSI_CREATE":"Gerar ficheiro VTSI",
        "STR_VTSI_CREATE_TIP":"Desta vez não irá gravar no dispositivo, mas apenas gerar um ficheiro VTSI#@Continuar?",
        "STR_VTSI_CREATE_SUCCESS":"Ficheiro VTSI criado com sucesso!#@Pode utilizar o Rufus(3.15+) para o gravar no dispositivo de modo a completar a instalação do Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Falha no ficheiro VTSI criado.",
        "STR_MENU_PART_RESIZE":"Instalação não destrutiva",
        "STR_PART_RESIZE_TIP":"O Ventoy tentará uma instalação não destrutiva, se possível. #@Continuar?",
        "STR_PART_RESIZE_SUCCESS":"Parabéns!#@Instalação não destrutiva do Ventoy concluída com sucesso.",
        "STR_PART_RESIZE_FAILED":"Falha na instalação não destrutiva. Leia o ficheiro /var/log/ventoy.log para mais detalhes.",
        "STR_PART_RESIZE_UNSUPPORTED":"A instalação não destrutiva do Ventoy foi interrompida porque algumas condições não podem ser cumpridas. Leia o ficheiro /var/log/ventoy.log para mais detalhes.",
        "STR_INSTALL_YES_TIP1":"Aviso: Os dados serão perdidos!",
        "STR_INSTALL_YES_TIP2":"Escreva YES na caixa de texto abaixo para confirmar que deseja de facto fazer uma nova instalação em vez de uma atualização.",
        "STR_PART_VENTOY_FS":"Sistema de ficheiros para a partição Ventoy",
        "STR_PART_FS":"Sistema de ficheiros",
        "STR_PART_CLUSTER":"Tamanho do cluster",
        "STR_PART_CLUSTER_DEFAULT":"Valor predefinido do sistema",
        "STR_DONATE":"Doar",
        "STR_4KN_UNSUPPORTED":"Atualmente, o Ventoy não é compatível com dispositivos nativos 4K.",
        
        "STRXXX":""
    },
    {
        "name":"Indonesian (Bahasa Indonesia)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Ida Bagus Anom Sanjaya",
        
        "STR_ERROR":"Kesalahan",
        "STR_WARNING":"Peringatan",
        "STR_INFO":"Informasi",
        "STR_INCORRECT_DIR":"Silakan jalankan pada direktori yang benar!",
        "STR_INCORRECT_TREE_DIR":"Jangan jalankan di sini, silakan unduh paket pemasangan yang diliris, dan jalankan di sana.",
        "STR_DEVICE":"Perangkat",
        "STR_LOCAL_VER":"Ventoy pada Paket",
        "STR_DISK_VER":"Ventoy pada Perangkat",
        "STR_STATUS":"Status - SIAP",
        "STR_INSTALL":"Pasang",
        "STR_UPDATE":"Perbarui",
        "STR_UPDATE_TIP":"Operasi pembaruan ini aman, tidak akan ada perubahan pada berkas ISO.#@Lanjutkan?",
        "STR_INSTALL_TIP":"Disk akan di-format dan semua data akan hilang.#@Lanjutkan?",
        "STR_INSTALL_TIP2":"Disk akan di-format dan semua data akan hilang.#@Lanjutkan? (YAKIN)",
        "STR_INSTALL_SUCCESS":"Selamat!#@Ventoy telah berhasil terpasang di perangkat ini.",
        "STR_INSTALL_FAILED":"Terjadi kesalahan ketika pemasangan berlangsung. Anda perlu mencabut-pasang ulang USB dan coba lagi. Cek /var/log/ventoy.log untuk detil.",
        "STR_UPDATE_SUCCESS":"Selamat!#@Ventoy telah berhasil diperbarui di perangkat ini.",
        "STR_UPDATE_FAILED":"Terjadi kesalahan ketika pembaruan berlangsung. Anda perlu mencabut-pasang ulang USB dan coba lagi. Cek /var/log/ventoy.log untuk detil.",
        "STR_WAIT_PROCESS":"Tugas sedang berjalan, silakan tunggu...",
        "STR_MENU_OPTION":"Pilihan",
        "STR_MENU_SECURE_BOOT":"Dukungan Secure Boot",
        "STR_MENU_PART_CFG":"Konfigurasi Partisi",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Batal",
        "STR_PRESERVE_SPACE":"Pertahankan sisa ruang penyimpanan pada disk ini",
        "STR_SPACE_VAL_INVALID":"Nilai sisa ruang tidak valid",
        "STR_MENU_CLEAR":"Bersihkan Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy telah berhasil dihapus pada perangkat ini.",
        "STR_CLEAR_FAILED":"Terjadi kesalahan ketika penghapusan berlangsung. Anda perlu mencabut-pasang ulang USB dan coba lagi. Periksa berkas /var/log/ventoy.log untuk detil.",
        "STR_MENU_PART_STYLE":"Model Partisi",
        "STR_DISK_2TB_MBR_ERROR":"Silakan pilih GPT untuk disk yang lebih dari 2TB",
        "STR_SHOW_ALL_DEV":"Tampilkan semua perangkat",
        "STR_PART_ALIGN_4KB":"Mensejajarkan dengan partisi 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Kesalahan komunikasi:",
        "STR_WEB_REMOTE_ABNORMAL":"Kesalahan komunikasi: pengendali bermasalah",
        "STR_WEB_REQUEST_TIMEOUT":"Kesalahan komunikasi: Waktu permintaan habis",
        "STR_WEB_SERVICE_UNAVAILABLE":"Kesalahan komunikasi: Layanan tidak tersedia",
        "STR_WEB_TOKEN_MISMATCH":"Status daemon diperbarui, silakan coba lagi nanti.",
        "STR_WEB_SERVICE_BUSY":"Layanan sedang sibuk, silakan coba lagi nanti.",
        "STR_MENU_VTSI_CREATE":"Membuat berkas VTSI",
        "STR_VTSI_CREATE_TIP":"Saat ini tidak akan menuliskan di perangkat, tetapi hanya membuat berkas VTSI #@Lanjutkan?",
        "STR_VTSI_CREATE_SUCCESS":"Berkas VTSI berhasil dibuat#@Anda bisa menggunakan Rufus(3.15+) untuk menuliskannya ke perangkat untuk menyelesaikan pemasangan Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Berkas VTSI gagal dibuat.",
        "STR_MENU_PART_RESIZE":"Pemasangan tanpa merusakkan",
        "STR_PART_RESIZE_TIP":"Ventoy akan mencoba pemasangan tanpa merusakkan apabila memungkinkan. #@Lanjutkan?",
        "STR_PART_RESIZE_SUCCESS":"Selamat!#@Pemasangan tanpa merusakkan Ventoy berhasil.",
        "STR_PART_RESIZE_FAILED":"Pemasangan tanpa merusakkan gagal, periksa berkas /var/log/ventoy.log untuk detil.",
        "STR_PART_RESIZE_UNSUPPORTED":"Pemasangan tanpa merusakkan Ventoy terhenti karena beberapa kondisi yang tidak mendukung. Periksa berkas /var/log/ventoy.log untuk detil.",
        "STR_INSTALL_YES_TIP1":"Peringatan: Data akan hilang!",
        "STR_INSTALL_YES_TIP2":"Silakan ketik YES pada kotak tulis di bawah untuk mengonfirmasikan bahwa anda memang yakin melakukan pemasangan baru ketimbang memperbaruinya",
        "STR_PART_VENTOY_FS":"Sistem Berkas Untuk Partisi Ventoy",
        "STR_PART_FS":"Sistem Berkas",
        "STR_PART_CLUSTER":"Ukuran Kluster",
        "STR_PART_CLUSTER_DEFAULT":"Nilai Standar Sistem",
        "STR_DONATE":"Donasi sukarela",
        "STR_4KN_UNSUPPORTED":"Saat ini Ventoy tidak mendukung perangkat 4K yang asli.",
        
        "STRXXX":""
    },
    {
        "name":"Norwegian Bokmål (Norsk Bokmål)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Allan Nordhøy",
        
        "STR_ERROR":"Feil",
        "STR_WARNING":"Advarsel",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Start programmet i rett mappe først.",
        "STR_INCORRECT_TREE_DIR":"Ikke kjør meg her. Last ned utgitt installasjonspakke og kjør det der.",
        "STR_DEVICE":"Enhet",
        "STR_LOCAL_VER":"Ventoy i pakke",
        "STR_DISK_VER":"Ventoy i enhet",
        "STR_STATUS":"Status — Klar",
        "STR_INSTALL":"Installer",
        "STR_UPDATE":"Updater",
        "STR_UPDATE_TIP":"Oppgradering er trygt. ISO-filer vil forbli uendret.#@Fortsett?",
        "STR_INSTALL_TIP":"Enheten vil bli formatert og all data på den vil gå tapt.#@Fortsett?",
        "STR_INSTALL_TIP2":"Enheten vil bli formatert og all data på den vil gå tapt.#@Fortsett? (Dobbeltsjekk)",
        "STR_INSTALL_SUCCESS":"Gratulerer!#@Ventoy har blitt installert på enheten.",
        "STR_INSTALL_FAILED":"Noe gikk galt under installasjonen. Du kan plugge ut og inn USB-enheten og prøve igjen. Sjekk /var/log/ventoy.log for detaljer. Hvis feilen vedvarer kan du sjekke O-S-S på nettsiden.",
        "STR_UPDATE_SUCCESS":"Gratulerer!#@Ventoy har blitt installert på enheten.",
        "STR_UPDATE_FAILED":"Noe gikk galt under oppdateringen. Du kan plugge ut og inn USB-enheten og prøve igjen. Sjekk /var/log/ventoy.log for detaljer. Hvis feilen vedvarer kan du sjekke O-S-S på nettsiden.",
        "STR_WAIT_PROCESS":"Vent mens en tråd kjører …",
        "STR_MENU_OPTION":"Alternativ",
        "STR_MENU_SECURE_BOOT":"Secure Boot-støtte",
        "STR_MENU_PART_CFG":"Partisjonsoppsett",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Avbryt",
        "STR_PRESERVE_SPACE":"Hold av litt plass på slutten av disken",
        "STR_SPACE_VAL_INVALID":"Ugyldig verdi for avholdt plass",
        "STR_MENU_CLEAR":"Tøm Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy har blitt fjernet fra enheten.",
        "STR_CLEAR_FAILED":"Noe gikk galt ved fjerning av Ventoy fra disken. Du kan plugge ut og inn USB-enheten og prøve igjen. Sjekk /var/log/ventoy.log for detaljer.",
        "STR_MENU_PART_STYLE":"Partisjonsstil",
        "STR_DISK_2TB_MBR_ERROR":"Velg «GPT» for enheter over 2 TB",
        "STR_SHOW_ALL_DEV":"Vis alle enheter",
        "STR_PART_ALIGN_4KB":"Juster partisjoner til 4 KB",
        "STR_WEB_COMMUNICATION_ERR":"Kommunikasjonsfeil:",
        "STR_WEB_REMOTE_ABNORMAL":"Kommunikasjonsfeil: Noe er galt annensteds hen",
        "STR_WEB_REQUEST_TIMEOUT":"Kommunikasjonsfeil: Tidsavbrudd for forespørsel",
        "STR_WEB_SERVICE_UNAVAILABLE":"Kommunikasjonsfeil: Utilgjengelig tjeneste",
        "STR_WEB_TOKEN_MISMATCH":"Nisse-status oppdatert. Prøv igjen senere.",
        "STR_WEB_SERVICE_BUSY":"Tjensten er opptatt. Prøv igjen senere.",
        "STR_MENU_VTSI_CREATE":"Generer VTSI-fil",
        "STR_VTSI_CREATE_TIP":"Dette vil ikke skrive til enheten, men kun generere en VTSIfil#@Fortsett?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI-fil opprettet.#@Du kan bruke Rufus(3.15+) til å skrive den til enheten for å fullføre installasjonen av Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Noe gikk galt under opprettelse av VTSI-fil.",
        "STR_MENU_PART_RESIZE":"Non-destructive Install",
        "STR_PART_RESIZE_TIP":"Ventoy will try non-destructive installation if possible. #@Continue?",
        "STR_PART_RESIZE_SUCCESS":"Congratulations!#@Ventoy non-destructive installation successfully finished.",
        "STR_PART_RESIZE_FAILED":"Non-destructive installation failed, Check /var/log/ventoy.log for details.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy non-destructive installation stopped because some conditions cannot be met. Check /var/log/ventoy.log for details.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"Donere",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Ukrainian (Українська)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Teraskull, thehugonote, dmitrygribenchuk",

        "STR_ERROR":"Помилка",
        "STR_WARNING":"Попередження",
        "STR_INFO":"Інформація",
        "STR_INCORRECT_DIR":"Будь ласка, запустіть у правильному каталозі!",
        "STR_INCORRECT_TREE_DIR":"Не запускайте мене тут, завантажте реліз інсталяційного пакету і запустіть там.",
        "STR_DEVICE":"Пристрій",
        "STR_LOCAL_VER":"Ventoy в пакеті",
        "STR_DISK_VER":"Ventoy на пристрої",
        "STR_STATUS":"Стан - ГОТОВИЙ",
        "STR_INSTALL":"Встановити",
        "STR_UPDATE":"Оновити",
        "STR_UPDATE_TIP":"Процес оновлення безпечний, файли ISO залишаться незмінними.#@Продовжити?",
        "STR_INSTALL_TIP":"Диск буде відформатовано, і всі дані будуть втрачені.#@Продовжити?",
        "STR_INSTALL_TIP2":"Диск буде відформатовано, і всі дані будуть втрачені.#@Ви дійсно хочете продовжити?",
        "STR_INSTALL_SUCCESS":"Вітаємо!#@Ventoy успішно встановлено на пристрій.",
        "STR_INSTALL_FAILED":"Під час встановлення сталася помилка. Ви можете ще раз підключити USB і повторити спробу. Перевірте /var/log/ventoy.log для отримання детальної інформації.",
        "STR_UPDATE_SUCCESS":"Вітаємо!#@Ventoy на пристрої успішно оновлено.",
        "STR_UPDATE_FAILED":"Під час оновлення сталася помилка. Ви можете ще раз підключити USB і повторити спробу. Перевірте /var/log/ventoy.log для отримання детальної інформації.",
        "STR_WAIT_PROCESS":"Потік запущено, зачекайте...",
        "STR_MENU_OPTION":"Опції",
        "STR_MENU_SECURE_BOOT":"Підтримка Secure Boot",
        "STR_MENU_PART_CFG":"Конфігурація розділів",
        "STR_BTN_OK":"ОК",
        "STR_BTN_CANCEL":"Скасувати",
        "STR_PRESERVE_SPACE":"Зарезервувати простір в кінці диска",
        "STR_SPACE_VAL_INVALID":"Недійсне значення для зарезервованого простору",
        "STR_MENU_CLEAR":"Видалити Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy успішно видалено з пристрою.",
        "STR_CLEAR_FAILED":"Під час видалення Ventoy сталася помилка. Ви можете ще раз підключити USB і повторити спробу. Перевірте /var/log/ventoy.log для отримання детальної інформації.",
        "STR_MENU_PART_STYLE":"Стиль розмітки розділів",
        "STR_DISK_2TB_MBR_ERROR":"Будь ласка, виберіть GPT для дисків понад 2TB",
        "STR_SHOW_ALL_DEV":"Показати всі пристрої",
        "STR_PART_ALIGN_4KB":"Вирівняти розділи з розміром 4КБ",
        "STR_WEB_COMMUNICATION_ERR":"Помилка зв'язку: ",
        "STR_WEB_REMOTE_ABNORMAL":"Помилка зв'язку: віддалене з'єднання недійсне",
        "STR_WEB_REQUEST_TIMEOUT":"Помилка зв'язку: Час очікування запиту минув",
        "STR_WEB_SERVICE_UNAVAILABLE":"Помилка зв'язку: Служба недоступна",
        "STR_WEB_TOKEN_MISMATCH":"Статус демона оновлено. Повторіть спробу пізніше.",
        "STR_WEB_SERVICE_BUSY":"Служба зайнята, повторіть спробу пізніше.",
        "STR_MENU_VTSI_CREATE":"Створити файл VTSI",
        "STR_VTSI_CREATE_TIP":"Цього разу Ventoy не буде встановлено на пристрій, а лише буде створено файл VTSI#@Продовжити?",
        "STR_VTSI_CREATE_SUCCESS":"Файл VTSI створено успішно!#@Ви можете використати Rufus(3.15+), щоб записати його на пристрій, для завершення встановлення Ventoy",
        "STR_VTSI_CREATE_FAILED":"Не вдалося створити файл VTSI.",
        "STR_MENU_PART_RESIZE":"Неруйнівна установка",
        "STR_PART_RESIZE_TIP":"Ventoy спробує виконати неруйнівну установку, якщо це можливо. #@Продовжити?",
        "STR_PART_RESIZE_SUCCESS":"Вітаємо!#@Неруйнівна установка Ventoy успішно завершена.",
        "STR_PART_RESIZE_FAILED":"Збій неруйнівної установки, перевірте /var/log/ventoy.log для отримання детальної інформації.",
        "STR_PART_RESIZE_UNSUPPORTED":"Неруйнівна установка Ventoy зупинена через неможливість виконання деяких умов. Перевірте /var/log/ventoy.log для отримання детальної інформації.",
        "STR_INSTALL_YES_TIP1":"Попередження: дані будуть втрачені!",
        "STR_INSTALL_YES_TIP2":"Будь ласка, введіть ТАК у текстове поле нижче, щоб підтвердити, що ви справді бажаєте виконати нове встановлення замість оновлення.",
        "STR_PART_VENTOY_FS":"Файлова система для розділу Ventoy",
        "STR_PART_FS":"Файлова система",
        "STR_PART_CLUSTER":"Розмір кластера",
        "STR_PART_CLUSTER_DEFAULT":"Системне значення за умовчанням",
        "STR_DONATE":"Пожертвуйте",
        "STR_4KN_UNSUPPORTED":"Наразі Ventoy не підтримує рідні пристрої 4K.",
        
        "STRXXX":""
    },
    {
        "name":"Greek (Ελληνικά)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"grmasa, Vasilis Kosmidis",
        
        "STR_ERROR":"Σφάλμα",
        "STR_WARNING":"Προειδοποίηση",
        "STR_INFO":"Πληροφορίες",
        "STR_INCORRECT_DIR":"Παρακαλώ εκτελέστε στον σωστό κατάλογο!",
        "STR_INCORRECT_TREE_DIR":"Μην με εκτελείτε εδώ, κατεβάστε το πακέτο εγκατάστασης και εκτελέστε εκεί.",
        "STR_DEVICE":"Συσκευή",
        "STR_LOCAL_VER":"Ventoy στο πρόγραμμα",
        "STR_DISK_VER":"Ventoy στη συσκευή",
        "STR_STATUS":"Κατάσταση - ΕΤΟΙΜΟ",
        "STR_INSTALL":"Εγκατάσταση",
        "STR_UPDATE":"Αναβάθμιση",
        "STR_UPDATE_TIP":"Η λειτουργία αναβάθμισης είναι ασφαλής, τα αρχεία ISO δεν θα αλλάξουν.#@Συνέχεια;",
        "STR_INSTALL_TIP":"Ο δίσκος θα μορφοποιηθεί και όλα τα δεδομένα θα χαθούν.#@Συνέχεια;",
        "STR_INSTALL_TIP2":"Ο δίσκος θα μορφοποιηθεί και όλα τα δεδομένα θα χαθούν.#@Συνέχεια; (Επανελέγξτε)",
        "STR_INSTALL_SUCCESS":"Συγχαρητήρια!#@Ventoy έχει εγκατασταθεί με επιτυχία στη συσκευή.",
        "STR_INSTALL_FAILED":"Παρουσιάστηκε σφάλμα κατά την εγκατάσταση. Μπορείτε να επανασυνδέσετε το USB και να δοκιμάσετε ξανά. Ελέγξτε το αρχείο /var/log/ventoy.log για λεπτομέρειες.",
        "STR_UPDATE_SUCCESS":"Συγχαρητήρια!#@Ventoy ενημερώθηκε με επιτυχία στη συσκευή.",
        "STR_UPDATE_FAILED":"Παρουσιάστηκε σφάλμα κατά την ενημέρωση. Μπορείτε να επανασυνδέσετε το USB και να δοκιμάσετε ξανά. Ελέγξτε το αρχείο /var/log/ventoy.log για λεπτομέρειες.",
        "STR_WAIT_PROCESS":"Μια εργασία εκτελείται, παρακαλώ περιμένετε...",
        "STR_MENU_OPTION":"Επιλογές",
        "STR_MENU_SECURE_BOOT":"Υποστήριξη ασφαλούς εκκίνησης (Secure Boot)",
        "STR_MENU_PART_CFG":"Διαμόρφωση κατατμήσεων",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Ακύρωση",
        "STR_PRESERVE_SPACE":"Διατηρήστε λίγο χώρο στο κάτω μέρος του δίσκου",
        "STR_SPACE_VAL_INVALID":"Μη έγκυρη τιμή για τον δεσμευμένο χώρο",
        "STR_MENU_CLEAR":"Κατάργηση του Ventoy",
        "STR_CLEAR_SUCCESS":"Το Ventoy καταργήθηκε με επιτυχία από τη συσκευή.",
        "STR_CLEAR_FAILED":"Παρουσιάστηκε σφάλμα κατά την εκκαθάριση του Ventoy από το δίσκο. Μπορείτε να επανασυνδέσετε το USB και να δοκιμάσετε ξανά. Ελέγξτε το αρχείο /var/log/ventoy.log για λεπτομέρειες.",
        "STR_MENU_PART_STYLE":"Στυλ κατατμήσεων",
        "STR_DISK_2TB_MBR_ERROR":"Παρακαλώ επιλέξτε GPT για δίσκο άνω των 2 TB",
        "STR_SHOW_ALL_DEV":"Προβολή όλων των συσκευών",
        "STR_PART_ALIGN_4KB":"Ευθυγράμμιση κατατμήσεων με 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Σφάλμα επικοινωνίας:",
        "STR_WEB_REMOTE_ABNORMAL":"Σφάλμα επικοινωνίας: απομακρυσμένη ανωμαλία",
        "STR_WEB_REQUEST_TIMEOUT":"Σφάλμα επικοινωνίας: χρονική λήξη αιτήματος",
        "STR_WEB_SERVICE_UNAVAILABLE":"Σφάλμα επικοινωνίας: υπηρεσία μη διαθέσιμη",
        "STR_WEB_TOKEN_MISMATCH":"Η κατάσταση του δαίμονα επικαιροποιήθηκε. Παρακαλώ δοκιμάστε αργότερα.",
        "STR_WEB_SERVICE_BUSY":"Η υπηρεσία είναι απασχολημένη. Παρακαλώ δοκιμάστε αργότερα.",
        "STR_MENU_VTSI_CREATE":"Δημιουργία αρχείου VTSI",
        "STR_VTSI_CREATE_TIP":"Αυτή τη φορά δε θα γράψει στη συσκευή, αλλά θα δημιουργήσει μόνο ένα αρχείο VTSI#@Συνέχεια;",
        "STR_VTSI_CREATE_SUCCESS":"Το αρχείο VTSI δημιουργήθηκε με επιτυχία!#@Μπορείτε να χρησιμοποιήσετε το Rufus(3.15+) για να το γράψετε στη συσκευή ώστε να ολοκληρώσετε την εγκατάσταση του Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Αποτυχία δημιουργίας αρχείου VTSI.",
        "STR_MENU_PART_RESIZE":"Μη καταστροφική εγκατάσταση",
        "STR_PART_RESIZE_TIP":"Το Ventoy θα δοκιμάσει μη καταστροφική εγκατάσταση, αν είναι δυνατόν. #@Συνέχεια;",
        "STR_PART_RESIZE_SUCCESS":"Συγχαρητήρια!#@Η μη καταστροφική εγκατάσταση του Ventoy ολοκληρώθηκε με επιτυχία.",
        "STR_PART_RESIZE_FAILED":"Η μη καταστροφική εγκατάσταση απέτυχε. Ελέγξτε το /var/log/ventoy.log για λεπτομέρειες.",
        "STR_PART_RESIZE_UNSUPPORTED":"Η μη καταστρεπτική εγκατάσταση του Ventoy σταμάτησε επειδή δεν μπορούν να τηρηθούν ορισμένες προϋποθέσεις. Ελέγξτε το /var/log/ventoy.log για λεπτομέρειες.",
        "STR_INSTALL_YES_TIP1":"Προειδοποίηση: Τα δεδομένα θα χαθούν!",
        "STR_INSTALL_YES_TIP2":"Εισάγετε ΝΑΙ στο παρακάτω πλαίσιο κειμένου για να επιβεβαιώσετε ότι όντως θέλετε να κάνετε μια νέα εγκατάσταση αντί για αναβάθμιση.",
        "STR_PART_VENTOY_FS":"Σύστημα αρχείων για το διαμέρισμα Ventoy",
        "STR_PART_FS":"Σύστημα αρχείων",
        "STR_PART_CLUSTER":"Μέγεθος συμπλέγματος",
        "STR_PART_CLUSTER_DEFAULT":"Προεπιλεγμένη τιμή συστήματος",
        "STR_DONATE":"Δωρεά",
        "STR_4KN_UNSUPPORTED":"Επί του παρόντος, το Ventoy δεν υποστηρίζει εγγενείς συσκευές 4K.",
        
        "STRXXX":""
    },
    
    {
        "name":"Swedish (Svenska)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Sopor",
        
        "STR_ERROR":"Fel",
        "STR_WARNING":"Varning",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Kör programmet från korrekt katalog!",
        "STR_INCORRECT_TREE_DIR":"Kör inte programmet här. Ladda ner installationspaketet och kör programmet där.",
        "STR_DEVICE":"Enhet",
        "STR_LOCAL_VER":"Ventoy i paketet",
        "STR_DISK_VER":"Ventoy på enheten",
        "STR_STATUS":"Status - REDO",
        "STR_INSTALL":"Installera",
        "STR_UPDATE":"Uppdatera",
        "STR_UPDATE_TIP":"Uppdateringen är säker, ISO-filerna kommer att vara oförändrade.#@Fortsätta?",
        "STR_INSTALL_TIP":"Enheten kommer att formateras och all data kommer att gå förlorad.#@Fortsätta?",
        "STR_INSTALL_TIP2":"Enheten kommer att formateras och all data kommer att gå förlorad.#@Fortsätta? (Sista varningen!)",
        "STR_INSTALL_SUCCESS":"Gratulerar!#@Ventoy har installerats på enheten.",
        "STR_INSTALL_FAILED":"Ett fel inträffade under installationen. Prova att återansluta USB-enheten och försök igen. Läs i /var/log/ventoy.log för mer information.",
        "STR_UPDATE_SUCCESS":"Gratulerar!#@Ventoy har uppdaterats på enheten.",
        "STR_UPDATE_FAILED":"Ett fel inträffade under uppdateringen. Prova att återansluta USB-enheten och försök igen. Läs i /var/log/ventoy.log för mer information.",
        "STR_WAIT_PROCESS":"En tråd körs redan, vänta...",
        "STR_MENU_OPTION":"Alternativ",
        "STR_MENU_SECURE_BOOT":"Stöd för säker start",
        "STR_MENU_PART_CFG":"Partitionskonfiguration",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Avbryt",
        "STR_PRESERVE_SPACE":"Spara lite utrymme i slutet av enheten",
        "STR_SPACE_VAL_INVALID":"Ogiltigt värde för reserverat utrymme",
        "STR_MENU_CLEAR":"Ta bort Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy har tagits bort från enheten.",
        "STR_CLEAR_FAILED":"Ett fel inträffade när Ventoy skulle tas bort från enheten. Prova att återansluta USB-enheten och försök igen. Läs i /var/log/ventoy.log för mer information.",
        "STR_MENU_PART_STYLE":"Partitionsstil",
        "STR_DISK_2TB_MBR_ERROR":"Välj GPT för enhet över 2 TB",
        "STR_SHOW_ALL_DEV":"Visa alla enheter",
        "STR_PART_ALIGN_4KB":"Justera partitioner med 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Kommunikationsfel:",
        "STR_WEB_REMOTE_ABNORMAL":"Kommunikationsfel: onormal fjärr",
        "STR_WEB_REQUEST_TIMEOUT":"Kommunikationsfel: Begäran tog för lång tid",
        "STR_WEB_SERVICE_UNAVAILABLE":"Kommunikationsfel: Tjänsten är inte tillgänglig",
        "STR_WEB_TOKEN_MISMATCH":"Daemon-status uppdaterad. Försök igen senare.",
        "STR_WEB_SERVICE_BUSY":"Tjänsten är upptagen. Försök igen senare.",
        "STR_MENU_VTSI_CREATE":"Skapa VTSI-fil",
        "STR_VTSI_CREATE_TIP":"Den här gången skrivs det inget till enheten, utan det skapas endast en VTSI-fil#@Fortsätta?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI-filen skapad!#@Använd Rufus (3.15+) till att skriva till enheten för att slutföra installationen av Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Misslyckades med att skapa VTSI-filen.",
        "STR_MENU_PART_RESIZE":"Icke-förstörande installation",
        "STR_PART_RESIZE_TIP":"Om det är möjligt kommer Ventoy att försöka göra en icke-förstörande installation. #@Fortsätta?",
        "STR_PART_RESIZE_SUCCESS":"Gratulerar!#@Ventoy lyckades slutföra en icke-förstörande installation.",
        "STR_PART_RESIZE_FAILED":"Ventoy misslyckades slutföra en icke-förstörande installation. Mer information finns i /var/log/ventoy.log.",
        "STR_PART_RESIZE_UNSUPPORTED":"Icke-förstörande installation stoppades eftersom vissa villkor inte kunde uppfyllas. Mer information finns i /var/log/ventoy.log.",
        "STR_INSTALL_YES_TIP1":"Varning: Data kommer att gå förlorad!",
        "STR_INSTALL_YES_TIP2":"Skriv YES i textrutan nedan för att bekräfta att du verkligen vill göra en nyinstallation i stället för att uppgradera.",
        "STR_PART_VENTOY_FS":"Filsystem för Ventoy-partition",
        "STR_PART_FS":"Filsystem",
        "STR_PART_CLUSTER":"Klusterstorlek",
        "STR_PART_CLUSTER_DEFAULT":"Systemets standardvärde",
        "STR_DONATE":"Donera",
        "STR_4KN_UNSUPPORTED":"För närvarande stöder inte Ventoy 4K-enheter.",
        
        "STRXXX":""
    },    
    {
        "name":"Bulgarian (Български)",
        "FontFamily":"Segoe",
        "FontSize":16,
        "Author":"jekovcar",

        "STR_ERROR":"Грешка",
        "STR_WARNING":"Предупреждение",
        "STR_INFO":"Информация",
        "STR_INCORRECT_DIR":"Стартирайте от правилната папка!",
        "STR_INCORRECT_TREE_DIR":"Не стартирайте оттук, изтеглете инсталационен пакет и стартирайте него.",
        "STR_DEVICE":"Устройство",
        "STR_LOCAL_VER":"Ventoy в пакета",
        "STR_DISK_VER":"Ventoy на устройството",
        "STR_STATUS":"Състояние - В готовност",
        "STR_INSTALL":"Инсталиране",
        "STR_UPDATE":"Обновяване",
        "STR_UPDATE_TIP":"Обновяването е безопасно, файловете на ISO няма да бъдат променени.#@Продължаване?",
        "STR_INSTALL_TIP":"Устройството ще бъде форматирано, а всички данни - премахнати.#@Продължаване?",
        "STR_INSTALL_TIP2":"Устройството ще бъде форматирано, а всички данни - премахнати.#@Продължаване? (повторно потвърждаване)",
        "STR_INSTALL_SUCCESS":"Поздравления!#@Ventoy е успешно инсталиран на устройството.",
        "STR_INSTALL_FAILED":"По време на инсталацията е възникнала грешка. Можете да поставите отново USB устройството и да опитате отново. Проверете /var/log/ventoy.log за подробности. Ако инсталацията винаги е неуспешна, направете справка с често задаваните въпроси на официалната страница.",
        "STR_UPDATE_SUCCESS":"Поздравление!#@Ventoy е успешно обновен на устройството.",
        "STR_UPDATE_FAILED":"По време на обновяването е възникнала грешка. Можете да поставите отново USB устройството и да опитате отново. Проверете /var/log/ventoy.log за подробности. Ако обновяването винаги е неуспешно, направете справка с често задаваните въпроси на официалната страница.",
        "STR_WAIT_PROCESS":"В процес, изчакайте…",
        "STR_MENU_OPTION":"Настройки",
        "STR_MENU_SECURE_BOOT":"Поддръжка на Secure Boot",
        "STR_MENU_PART_CFG":"Допълнителен дял",
        "STR_BTN_OK":"Добре",
        "STR_BTN_CANCEL":"Отказ",
        "STR_PRESERVE_SPACE":"Създаване на допълнителен дял в края на диска",
        "STR_SPACE_VAL_INVALID":"Невярна стойност за запазено пространство",
        "STR_MENU_CLEAR":"Премахване на Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy е успешно премахнат от устройството.",
        "STR_CLEAR_FAILED":"По време на премахването е възникнала грешка. Можете да поставите отново USB устройството и да опитате отново. Проверете /var/log/ventoy.log за подробности.",
        "STR_MENU_PART_STYLE":"Вид на дяловете",
        "STR_DISK_2TB_MBR_ERROR":"Изберете GPT за устройства по-големи от 2ТБ",
        "STR_SHOW_ALL_DEV":"Показване на всички устройства",
        "STR_PART_ALIGN_4KB":"Подравняване на дяловете по 4КБ",
        "STR_WEB_COMMUNICATION_ERR":"Грешка при свързване:",
        "STR_WEB_REMOTE_ABNORMAL":"Грешка при свързване: Отдалечената връзка е недействителна",
        "STR_WEB_REQUEST_TIMEOUT":"Грешка при свързване: Времето за изчакване на заявката е просрочено",
        "STR_WEB_SERVICE_UNAVAILABLE":"Грешка при свързване: Услугата е недостъпна",
        "STR_WEB_TOKEN_MISMATCH":"Състоянието на услугата е обновено, опитайте по-късно.",
        "STR_WEB_SERVICE_BUSY":"Услугата е заета, опитайте по-късно.",
        "STR_MENU_VTSI_CREATE":"Създаване на файл на VTSI",
        "STR_VTSI_CREATE_TIP":"Този път няма да се извършва запис на устройството, а само ще бъде създаден файл на VTSI#@Продължаване?",
        "STR_VTSI_CREATE_SUCCESS":"Файлът на VTSI е създаден успешно!#@Може да използвате Rufus(3.15+), за да го запишете на устройството и така да завършите инсталацията на Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Грешка при създаване на файла на VTSI.",
        "STR_MENU_PART_RESIZE":"Неразрушително инсталиране",
        "STR_PART_RESIZE_TIP":"Ventoy ще направи опит за неразрушително инсталиране, ако е възможно.#@Продължаване?",
        "STR_PART_RESIZE_SUCCESS":"Поздравления!#@Неразрушителното инсталиране на Ventoy е успешно.",
        "STR_PART_RESIZE_FAILED":"Неразрушителното инсталиране на Ventoy е неуспешно. Проверете /var/log/ventoy.log за подробности.",
        "STR_PART_RESIZE_UNSUPPORTED":"Неразрушителното инсталиране на Ventoy е спряно, защото някои от условията не могат да бъдат изпълнени. Проверете /var/log/ventoy.log за подробности.",
        "STR_INSTALL_YES_TIP1":"Внимание: Ще има загуба на данни!",
        "STR_INSTALL_YES_TIP2":"Въведете „YES“ в полето отдолу, за да потвърдите, че искате да извършите нова инсталация вместо обновяване.",
        "STR_PART_VENTOY_FS":"Файлова система за дял на Ventoy",
        "STR_PART_FS":"Файлова система",
        "STR_PART_CLUSTER":"Размер на клъстера",
        "STR_PART_CLUSTER_DEFAULT":"По подразбиране от системата",
        "STR_DONATE":"Дарете",
        "STR_4KN_UNSUPPORTED":"За момента Ventoy не поддържа устройства с клъстери, чиито размери са 4КБ.",
        
        "STRXXX":""
    },
    {
        "name":"Armenian (Հայերեն)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Egho",

        "STR_ERROR":"Սխալ",
        "STR_WARNING":"Զգուշացում",
        "STR_INFO":"Տեղեկություն",
        "STR_INCORRECT_DIR":"Խնդրում ենք գործարկել ճիշտ թղթախանակում!",
        "STR_INCORRECT_TREE_DIR":"Չգործարկել այստեղ, խնդրում եմ ներբեռնեք թողարկված տեղադրման փաթեթը և գործարկել այնտեղ:",
        "STR_DEVICE":"Կրիչը",
        "STR_LOCAL_VER":"Ventoy-ը փաթեթում",
        "STR_DISK_VER":"Ventoy-ը կրիչում",
        "STR_STATUS":"Կարգավիճակ - ՊԱՏՐԱՍՏ",
        "STR_INSTALL":"Տեղադրել",
        "STR_UPDATE":"Թարմացնել",
        "STR_UPDATE_TIP":"Թարմացման գործողությունը անվտանգ է, ISO ֆայլերը կմնան անփոփոխ.#@Շարունակել?",
        "STR_INSTALL_TIP":"Սկավառակը կձևաչափվի և բոլոր տվյալները կկորչեն.#@Շարունակել?",
        "STR_INSTALL_TIP2":"Սկավառակը կձևաչափվի և բոլոր տվյալները կկորչեն:#@Դուք համոզված եք, մաքրել կրիչը? (Double Check)",
        "STR_INSTALL_SUCCESS":"Շնորհավորում ենք!#@Ventoy- ը հաջողությամբ տեղադրվել է կրիչում:",
        "STR_INSTALL_FAILED":"Տեղադրման ընթացքում տեղի ունեցավ սխալ: Կարող եք նորից միացնել USB- կրիչը և կրկին փորձել: Մանրամասների համար ստուգել /var/log/ventoy.log-ը ",
        "STR_UPDATE_SUCCESS":"Շնորհավորում ենք!#@Ventoy-ը հաջողությամբ թարմացվեց կրիչում:",
        "STR_UPDATE_FAILED":"Թարմացման ընթացքում տեղի ունեցավ սխալ: Կարող եք նորից միացնել USB- կրիչը և կրկին փորձել: Մանրամասների համար ստուգել /var/log/ventoy.log-ը ",
        "STR_WAIT_PROCESS":"Աշխատանքն ընթանում, խնդրում ենք սպասել...",
        "STR_MENU_OPTION":"Ընտրանքներ",
        "STR_MENU_SECURE_BOOT":"Secure Boot աջակցություն",
        "STR_MENU_PART_CFG":"Partition կազմաձևում",
        "STR_BTN_OK":"Այո",
        "STR_BTN_CANCEL":"Չեղարկել",
        "STR_PRESERVE_SPACE":"Պահպանեք որոշակի տարածք կրիչի վերջում",
        "STR_SPACE_VAL_INVALID":"Պահված տարածքի անվավեր արժեք",
        "STR_MENU_CLEAR":"Մաքրել Ventoy-ը",
        "STR_CLEAR_SUCCESS":"Ventoy-ը հաջողությամբ հեռացված է կրիչից:",
        "STR_CLEAR_FAILED":"Ventoy-ը կրիչից մաքրելիս տեղի ունեցավ սխալ: Կարող եք նորից միացնել USB-կրիչը և կրկին փորձել: Մանրամասների համար ստուգել /var/log/ventoy.log-ը",
        "STR_MENU_PART_STYLE":"Partition-ի տեսակը",
        "STR_DISK_2TB_MBR_ERROR":"Խնդրում ենք ընտրել GPT 2TB-ից մեծ կրիչի դեպքում",
        "STR_SHOW_ALL_DEV":"Ցուցադրել բոլոր կրիչները",
        "STR_PART_ALIGN_4KB":"partition-երը գծանշել 4KB-ով",
        "STR_WEB_COMMUNICATION_ERR":"Համակցման սխալ․",
        "STR_WEB_REMOTE_ABNORMAL":"Համակցման սխալ․ Հեռահար համակցումն չստացվեց",
        "STR_WEB_REQUEST_TIMEOUT":"Համակցման սխալ․: Սպասման ժամանակը սպառվեց",
        "STR_WEB_SERVICE_UNAVAILABLE":"Համակցման սխալ․: Ծառայությունը անհասանելի է",
        "STR_WEB_TOKEN_MISMATCH":"Թարմացվել է Demon-ի կարգավիճակը, խնդրում եմ նորից փորձել ավելի ուշ:",
        "STR_WEB_SERVICE_BUSY":"Ծառայությունը զբաղված է, խնդրում եմ նորից փորձել ավելի ուշ:",
        "STR_MENU_VTSI_CREATE":"Ստեղծել VTSI ֆայլ",
        "STR_VTSI_CREATE_TIP":"Այս անգամ չի գրվի կրիչի վրա, այլ միայն կստեղծվի VTSI ֆայլ#@Շարունակել?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI ֆայլը հաջողությամբ ստեղծվեց!#@Դուք կարող եք օգտագործել Rufus (3.15+) կրիչում ձայնագրելու համար, որպեսզի ավարտեք Ventoy-ի տեղադրումը:",
        "STR_VTSI_CREATE_FAILED":"VTSI ֆայլի ստեղծումը ձախողվեց։",
        "STR_MENU_PART_RESIZE":"Non-destructive Install",
        "STR_PART_RESIZE_TIP":"Ventoy will try non-destructive installation if possible. #@Continue?",
        "STR_PART_RESIZE_SUCCESS":"Congratulations!#@Ventoy non-destructive installation successfully finished.",
        "STR_PART_RESIZE_FAILED":"Non-destructive installation failed, Check /var/log/ventoy.log for details.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy non-destructive installation stopped because some conditions cannot be met. Check /var/log/ventoy.log for details.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"Նվիրաբերել",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Finnish (suomi)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"heidi.wenger",

        "STR_ERROR":"Virhe",
        "STR_WARNING":"Varoitus",
        "STR_INFO":"Tiedot",
        "STR_INCORRECT_DIR":"Ole hyvä ja aja oikean kansion alla!",
        "STR_INCORRECT_TREE_DIR":"Älä aja minua tässä, ole hyvä ja lataa julkaistu asennuspaketti, ja suorita ajo siellä.",
        "STR_DEVICE":"Laite",
        "STR_LOCAL_VER":"Ventoy paketissa",
        "STR_DISK_VER":"Ventoy laitteessa",
        "STR_STATUS":"Tila - VALMIS",
        "STR_INSTALL":"Asenna",
        "STR_UPDATE":"Päivitä",
        "STR_UPDATE_TIP":"Päivytystoiminto on turvallinen, ISO -tiedostot pysyvät muuttumattomana.#@Jatketaanko?",
        "STR_INSTALL_TIP":"Laite tullaan alustamaan ja tämän johdosta kaikki tieto sen sisällä menetetään.#@Jatketaanko?",
        "STR_INSTALL_TIP2":"Laite tullaan alustamaan ja tämän johdosta kaikki tieto sen sisällä menetetään.#@Jatketaanko? (Kaksoistarkistus)",
        "STR_INSTALL_SUCCESS":"Onnittelut!#@Ventoy on asennettu laitteeseen onnistuneesti.",
        "STR_INSTALL_FAILED":"Asennuksen yhteydessä ilmeni virhe. Voit nypätä USB-laitteen irti ja koettaa uudelleen. Katso /var/log/ventoy.log nähdäksesi yksityiskohdat. Mikäli toimi epäonnistuu aina, lue usein kysytyt kysymykset (FAQ) virallisella sivustolla.",
        "STR_UPDATE_SUCCESS":"Onnittelut!#@Ventoy päivitettiin laitteeseen onnistuneesti.",
        "STR_UPDATE_FAILED":"Päivityksen aikana ilmeni virhe. Voit nypätä USB-laitteen irti ja koettaa uudelleen. Katso /var/log/ventoy.log nähdäksesi yksityiskohdat. Mikäli toimi epäonnistuu aina, lue usein kysytyt kysymykset (FAQ) virallisella sivustolla.",
        "STR_WAIT_PROCESS":"Toimi on ajossa juuri nyt, ole hyvä ja odota...",
        "STR_MENU_OPTION":"Vaihtoehto",
        "STR_MENU_SECURE_BOOT":"Secure Boot (turvakäynnistys) -tuki",
        "STR_MENU_PART_CFG":"Osioasetukset",
        "STR_BTN_OK":"SELVÄ",
        "STR_BTN_CANCEL":"Peruuta",
        "STR_PRESERVE_SPACE":"Jätä hiukan vapaata tilaa levyn loppuun",
        "STR_SPACE_VAL_INVALID":"Epäkelpo määrä tilavaraukselle",
        "STR_MENU_CLEAR":"Poista Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy on nyt poistettu laitteesta onnistuneesti.",
        "STR_CLEAR_FAILED":"Ilmeni virhe poistettaessa Ventoyta levyltä. Voit nypätä laitteen irti ja koettaa uudelleen. Katso /var/log/ventoy.log nähdäksesi yksityiskohdat.",
        "STR_MENU_PART_STYLE":"Osiotyyli",
        "STR_DISK_2TB_MBR_ERROR":"Ole hyvä ja valitse GPT laitteille joissa on tilavuutta yli 2 teratavua",
        "STR_SHOW_ALL_DEV":"Näytä kaikki laitteet",
        "STR_PART_ALIGN_4KB":"Järjestä osiot 4Kt mukaisesti",
        "STR_WEB_COMMUNICATION_ERR":"Kommunikaatiovirhe:",
        "STR_WEB_REMOTE_ABNORMAL":"Kommunikaatiovirhe: epätavanomainen etävaste",
        "STR_WEB_REQUEST_TIMEOUT":"Kommunikaatiovirhe: pyynnön aikavaste ylittyi",
        "STR_WEB_SERVICE_UNAVAILABLE":"Kommunikaatiovirhe: palvelu on tavoittamattomissa",
        "STR_WEB_TOKEN_MISMATCH":"Taustaohjelman tila päivittyi, ole hyvä ja yritä myöhemmin uudelleen.",
        "STR_WEB_SERVICE_BUSY":"Palvelu ruuhkautunut, ole hyvä ja yritä myöhemmin uudelleen.",
        "STR_MENU_VTSI_CREATE":"Luo VTSI -tiedosto",
        "STR_VTSI_CREATE_TIP":"Tällä kerralla laitteeseen ei kirjoiteta, vaan luodaan ainoastaan VTSI -tiedosto#@Jatketaanko?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI -tiedosto luotiin onnistuneesti!#@Voit käyttää Rufus-ohjelmaa(3.15+) kirjoittaaksesi sen laitteeseen viimeistelläksesi Ventoy-asennuksen.",
        "STR_VTSI_CREATE_FAILED":"VTSI -tiedoston luominen epäonnistui.",
        "STR_MENU_PART_RESIZE":"Ei-tuhoisa asennus",
        "STR_PART_RESIZE_TIP":"Ventoy kokeilee ei-tuhoisaa asennusta mikäli tämä on mahdollista. #@Jatketaanko?",
        "STR_PART_RESIZE_SUCCESS":"Onnittelut!#@Ventoyn ei-tuhoisa asennus saapui päätökseen onnistuneesti.",
        "STR_PART_RESIZE_FAILED":"Ei-tuhoisa asennusmuoto epäonnistui, tarkista /var/log/ventoy.log nähdäksesi yksityiskohtaiset tiedot.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoyn ei-tuhoisa asennus pysähtyi koska jotkut ehdot eivät täyttyneet. Tarkista /var/log/ventoy.log nähdäksesi yksityiskohdat.",
        "STR_INSTALL_YES_TIP1":"Varoitus: Data menetetään!",
        "STR_INSTALL_YES_TIP2":"Kirjoita YES alla olevaan tekstikenttään vahvistaaksesi että haluat tehdä uuden asennuksen päivityksen sijaan.",
        "STR_PART_VENTOY_FS":"Ventoy -osion tiedostojärjestelmä",
        "STR_PART_FS":"Tiedostojärjestelmä",
        "STR_PART_CLUSTER":"Klusterin koko",
        "STR_PART_CLUSTER_DEFAULT":"Järjestelmän oletusarvo",
        "STR_DONATE":"Lahjoittaa",
        "STR_4KN_UNSUPPORTED":"Tällä hetkellä Ventoy ei tue 4K laitteita.",
        
        "STRXXX":""
    },
    {
        "name":"Galician (Galego)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"gbrea",
        
        "STR_ERROR":"Erro",
        "STR_WARNING":"Atención",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Por favor, execute no directorio correto!",
        "STR_INCORRECT_TREE_DIR":"Non me executes aquí, por favor transfire o pacote de instalación, e execute alí.",
        "STR_DEVICE":"Dispositivo",
        "STR_LOCAL_VER":"Ventoy na aplicación",
        "STR_DISK_VER":"Ventoy no dispositivo",
        "STR_STATUS":"Estado - PRONTO",
        "STR_INSTALL":"Instalar",
        "STR_UPDATE":"Actualizar",
        "STR_UPDATE_TIP":"A operación de atualización é segura, os ficheiros ISO non serán alterados.#@Continuar?",
        "STR_INSTALL_TIP":"O disco será formatado e perderanse todos os datos.#@Continuar?",
        "STR_INSTALL_TIP2":"O disco será formatado e perderanse todos os datos.#@Continuar? (Confirmación)",
        "STR_INSTALL_SUCCESS":"Parabéns!#@Ventoy foi instalado con suceso no dispositivo.",
        "STR_INSTALL_FAILED":"Un erro ocorreu durante a instalación. Podes reconectar o dispositivo USB e tentar novamente. Verifica o ficheiro /var/log/ventoy.log para máis detalles.",
        "STR_UPDATE_SUCCESS":"Parabéns!#@Ventoy foi actualizado con suceso no dispositivo.",
        "STR_UPDATE_FAILED":"Un erro ocorreu durante a actualización. Podes reconectar o dispositivo USB e tentar novamente. Verifica o ficheiro /var/log/ventoy.log para máis detalles.",
        "STR_WAIT_PROCESS":"Un proceso está en execución, por favor espera...",
        "STR_MENU_OPTION":"Opción",
        "STR_MENU_SECURE_BOOT":"Soporte de arranque seguro",
        "STR_MENU_PART_CFG":"Configuración da Partición",
        "STR_BTN_OK":"OK",
        "STR_BTN_CANCEL":"Cancelar",
        "STR_PRESERVE_SPACE":"Preservar algún espazo no final do disco",
        "STR_SPACE_VAL_INVALID":"Valor inválido para o espazo reservado",
        "STR_MENU_CLEAR":"Remover o Ventoy",
        "STR_CLEAR_SUCCESS":"O Ventoy foi removido deste dispositivo con suceso.",
        "STR_CLEAR_FAILED":"Un erro ocorreu ao remover o Ventoy do disco. Podes reconectar o dispositivo USB e tentar novamente. Verifica o ficheiro /var/log/ventoy.log para máis detalles.",
        "STR_MENU_PART_STYLE":"Estilo de Partición",
        "STR_DISK_2TB_MBR_ERROR":"Por favor seleciona GPT para discos maiores que 2TB",
        "STR_SHOW_ALL_DEV":"Mostrar todos os dispositivos",
        "STR_PART_ALIGN_4KB":"Aliñar particións con 4KB",
        "STR_WEB_COMMUNICATION_ERR":"Erro de comunicación:",
        "STR_WEB_REMOTE_ABNORMAL":"Erro de comunicación: remoto anormal",
        "STR_WEB_REQUEST_TIMEOUT":"Erro de comunicación: Tempo de espera esgotado",
        "STR_WEB_SERVICE_UNAVAILABLE":"Erro de comunicación: Servizo non dispoñible",
        "STR_WEB_TOKEN_MISMATCH":"Estado do daemon actualizado, por favor inténtao máis tarde.",
        "STR_WEB_SERVICE_BUSY":"O servizo está ocupado, por favor inténtao máis tarde.",
        "STR_MENU_VTSI_CREATE":"Xerar arquivo VTSI",
        "STR_VTSI_CREATE_TIP":"Non será grabado no dispositivo, desta vez, apenas se xerará un ficheiro VTSI#@Continuar?",
        "STR_VTSI_CREATE_SUCCESS":"Arquivo VTSI creado con suceso!#@Podes usar Rufus(3.15+) para grabalo no dispositivo e completar a instalación do Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Erro na creación do arquivo VTSI.",
        "STR_MENU_PART_RESIZE":"Instalación non destructiva",
        "STR_PART_RESIZE_TIP":"Ventoy tentará unha instalación non destructiva, se é posible.#@Continuar?",
        "STR_PART_RESIZE_SUCCESS":"Parabéns!#@Instalación non destructiva do Ventoy concluida con suceso.",
        "STR_PART_RESIZE_FAILED":"Erro na instalación non destructiva, revisa o arquivo /var/log/ventoy.log para ver os detalles.",
        "STR_PART_RESIZE_UNSUPPORTED":"Instalación non destructiva do Ventoy interrompida porque non se poden cumprir algunhas condicións. Revisa o arquivo /var/log/ventoy.log para mais detalles.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"Doa",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Catalan (Català)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"guillemglez",
        
        "STR_ERROR":"Error",
        "STR_WARNING":"Avís",
        "STR_INFO":"Info",
        "STR_INCORRECT_DIR":"Cal executar-ho en el directori correcte!",
        "STR_INCORRECT_TREE_DIR":"No m'executis aquí, si us plau, baixa el paquet d'instal·lació publicat i executa-ho allí.",
        "STR_DEVICE":"Dispositiu",
        "STR_LOCAL_VER":"Ventoy localment",
        "STR_DISK_VER":"Ventoy al dispositiu",
        "STR_STATUS":"Estat - Llest",
        "STR_INSTALL":"Instal·la",
        "STR_UPDATE":"Actualitza",
        "STR_UPDATE_TIP":"L'actualització és segura i els fitxers ISO no es veuran afectats.#@Continuar?",
        "STR_INSTALL_TIP":"El dispositiu serà formatat i totes les dades es perdran.#@Continuar?",
        "STR_INSTALL_TIP2":"El dispositiu serà formatat i totes les dades es perdran.#@Continuar? (Confirmeu)",
        "STR_INSTALL_SUCCESS":"Enhorabona!#@Ventoy s'ha instal·lat al vostre dispositiu de forma satisfactòria.",
        "STR_INSTALL_FAILED":"Hi ha hagut un error durant el procés d'instal·lació. Podeu desconnectar i tornar-lo a connectar i provar-ho de nou. Comproveu el fitxer /var/log/ventoy.log per a més detalls. Si això passa sovint visiteu el FAQs a la nostra pàgina oficial.",
        "STR_UPDATE_SUCCESS":"Enhorabona!#@Ventoy s'ha actualitzat al vostre dispositiu de forma satisfactòria.",
        "STR_UPDATE_FAILED":"Hi ha hagut un error durant el procés d'actualització. Podeu desconnectar i tornar-lo a connectar i provar-ho de nou. Comproveu el fitxer /var/log/ventoy.log per a més detalls. Si això passa sovint visiteu el FAQs a la nostra pàgina oficial.",
        "STR_WAIT_PROCESS":"Hi ha una operació en curs, espereu si us plau...",
        "STR_MENU_OPTION":"Opcions",
        "STR_MENU_SECURE_BOOT":"Arrencada segura",
        "STR_MENU_PART_CFG":"Ajustaments de partició",
        "STR_BTN_OK":"D'acord",
        "STR_BTN_CANCEL":"Cancel·la",
        "STR_PRESERVE_SPACE":"Preserva espai al final del disc",
        "STR_SPACE_VAL_INVALID":"Valor d'espai reservat invàlid",
        "STR_MENU_CLEAR":"Esborra el Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy s'ha esborrat satisfactòriament del dispositiu.",
        "STR_CLEAR_FAILED":"Hi ha hagut un error mentre s'esborrava Ventoy del dispositiu. Podeu desconnectar i tornar-lo a connectar i provar-ho de nou. Comproveu el fitxer /var/log/ventoy.log per a més detalls.",
        "STR_MENU_PART_STYLE":"Estil de partició",
        "STR_DISK_2TB_MBR_ERROR":"Sel·leccioneu GPT per dispositius de més de 2 TB",
        "STR_SHOW_ALL_DEV":"Mostra tots els dispositius",
        "STR_PART_ALIGN_4KB":"Alinea les particions a 4 KB",
        "STR_WEB_COMMUNICATION_ERR":"Error de comunicació:",
        "STR_WEB_REMOTE_ABNORMAL":"Error de comunicació: anomalia al servidor",
        "STR_WEB_REQUEST_TIMEOUT":"Error de comunicació: s'ha esgotat el temps d'espera",
        "STR_WEB_SERVICE_UNAVAILABLE":"Error de comunicació: servei no disponible",
        "STR_WEB_TOKEN_MISMATCH":"S'ha actualitzat l'estat del dimoni, torneu a provar-ho més tard.",
        "STR_WEB_SERVICE_BUSY":"El servei està sobrecarregat, torneu a provar-ho més tard.",
        "STR_MENU_VTSI_CREATE":"Genera un fitxer VTSI",
        "STR_VTSI_CREATE_TIP":"Aquest cop no s'escriuran els canvis al dispositiu, sinó que es generarà un fitxer VTSI#@Continuar?",
        "STR_VTSI_CREATE_SUCCESS":"El fitxer VTSI s'ha creat satisfactòriament!#@Pots fer servir Rufus(3.15+) per escriure'l al dispositiu i completar així la instal·lació de Ventoy.",
        "STR_VTSI_CREATE_FAILED":"No s'ha pogut crear el fitxer VTSI",
        "STR_MENU_PART_RESIZE":"Instal·lació no destructiva",
        "STR_PART_RESIZE_TIP":"El Ventoy intentarà una instal·lació no destructiva si és possible. #@Continuar?",
        "STR_PART_RESIZE_SUCCESS":"Enhorabona!#@La instal·lació no destructiva s'ha completat de forma satisfactòria.",
        "STR_PART_RESIZE_FAILED":"Hi ha hagut un error durant el procés d'instal·lació no destructiva, comproveu el fitxer /var/log/ventoy.log per a més detalls.",
        "STR_PART_RESIZE_UNSUPPORTED":"La instal·lació no destructiva de Ventoy no ha estat possible perquè els requeriments no es compleixen. Comproveu el fitxer /var/log/ventoy.log per a més detalls.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"Donar",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    
    {
        "name":"Georgian (ქართული)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"Levan Baindurashvili",
        
        "STR_ERROR":"შეცდომა",
        "STR_WARNING":"გაფრთხილება",
        "STR_INFO":"ინფორმაცია",
        "STR_INCORRECT_DIR":"გთხოვთ, გაუშვით სხვა კატალოგში!",
        "STR_INCORRECT_TREE_DIR":"ნუ გამიშვებ აქ, გთხოვთ გადმოწეროთ გამოშვებული ინსტალაციის პაკეტი და გაუშვათ სხვაგან.",
        "STR_DEVICE":"მოწყობილობა",
        "STR_LOCAL_VER":"Ventoy პაკეტში",
        "STR_DISK_VER":"Ventoy მოწყობილობაზე",
        "STR_STATUS":"სტატუსი - მზადაა",
        "STR_INSTALL":"დაყენება",
        "STR_UPDATE":"განახლება",
        "STR_UPDATE_TIP":"განახლება უსაფრთხოა და ISO ფაილები უცვლელი დარჩება.#@გაგრძელება?",
        "STR_INSTALL_TIP":"დისკი დაფორმატდება და ყველა მონაცემი დაიკარგება.#@გსურთ გააგრძელოთ?",
        "STR_INSTALL_TIP2":"დისკი დაფორმატდება და ყველა მონაცემი დაიკარგება.#@ ნამდვილად გსურთ გაგრძელება?",
        "STR_INSTALL_SUCCESS":"გილოცავთ!#@Ventoy წარმატებით დაინსტალირდა თქვენს მოწყობილობაზე.",
        "STR_INSTALL_FAILED":"Ventoy-ის ინსტალაციისას მოხდა შეცდომა. ხელახლა შეაერთეთ თქვენი მოწყობილობა და სცადეთ ხელახლა. შეამოწმეთ /var/log/ventoy.log შეცდომებისთვის.",
        "STR_UPDATE_SUCCESS":"გილოცავთ!#@Ventoy წარმატებით განახლდა თქვენს მოწყობილობაზე.",
        "STR_UPDATE_FAILED":"Ventoy-ის განახლებისას მოხდა შეცდომა. ხელახლა შეაერთეთ თქვენი მოწყობილობა და სცადეთ ხელახლა. შეამოწმეთ /var/log/ventoy.log შეცდომებისთვის.",
        "STR_WAIT_PROCESS":"პროცესი დაიწყო, გთხოვთ დაელოდოთ...",
        "STR_MENU_OPTION":"Პარამეტრები",
        "STR_MENU_SECURE_BOOT":"Secure Boot-ის მხარდაჭერა",
        "STR_MENU_PART_CFG":"დამატებითი განყოფილება",
        "STR_BTN_OK":"ОК",
        "STR_BTN_CANCEL":"უარყოფა",
        "STR_PRESERVE_SPACE":"შექმენით დამატებითი დანაყოფი დისკის ბოლოს",
        "STR_SPACE_VAL_INVALID":"დანაყოფის ზომა არასწორია",
        "STR_MENU_CLEAR":"Ventoy-ს წაშლა",
        "STR_CLEAR_SUCCESS":"Ventoy წარმატებით წაიშალა მოწყობილობიდან.",
        "STR_CLEAR_FAILED":"Ventoy-ის დეინსტალაციისას მოხდა შეცდომა. ხელახლა შეაერთეთ თქვენი მოწყობილობა და სცადეთ ხელახლა. შეამოწმეთ /var/log/ventoy.log შეცდომებზე.",
        "STR_MENU_PART_STYLE":"განყოფილების განლაგების სტილი",
        "STR_DISK_2TB_MBR_ERROR":"გთხოვთ, აირჩიოთ GPT 2 ტბაიტზე მეტი დისკებისთვის",
        "STR_SHOW_ALL_DEV":"ყველა მოწყობილობის ჩვენება",
        "STR_PART_ALIGN_4KB":"ტიხრების გასწორება 4 KB ზომით",
        "STR_WEB_COMMUNICATION_ERR":"დაკავშირების შეცდომა:",
        "STR_WEB_REMOTE_ABNORMAL":"კომუნიკაციის შეცდომა: დისტანციური კავშირი არასწორია",
        "STR_WEB_REQUEST_TIMEOUT":"კომუნიკაციის შეცდომა: მოთხოვნის დრო ამოიწურა",
        "STR_WEB_SERVICE_UNAVAILABLE":"კომუნიკაციის შეცდომა: სერვისი მიუწვდომელია",
        "STR_WEB_TOKEN_MISMATCH":"დემონის სტატუსი განახლებულია. გთხოვთ სცადოთ მოგვიანებით.",
        "STR_WEB_SERVICE_BUSY":"სერვისი დაკავებულია, გთხოვთ, სცადოთ მოგვიანებით.",
        "STR_MENU_VTSI_CREATE":"VTSI ფაილის შექმნა",
        "STR_VTSI_CREATE_TIP":"ამჯერად Ventoy არ დაინსტალირდება მოწყობილობაზე, მაგრამ შეიქმნება მხოლოდ VTSI ფაილი#@გაგრძელება?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI ფაილი წარმატებით შეიქმნა!#@შეგიძლიათ გამოიყენოთ Rufus(3.15+) ჩაწეროთ მოწყობილობაზე Ventoy ინსტალაციის დასასრულებლად.",
        "STR_VTSI_CREATE_FAILED":"VTSI ფაილის შექმნის შეცდომა.",
        "STR_MENU_PART_RESIZE":"Non-destructive Install",
        "STR_PART_RESIZE_TIP":"Ventoy will try non-destructive installation if possible. #@Continue?",
        "STR_PART_RESIZE_SUCCESS":"Congratulations!#@Ventoy non-destructive installation successfully finished.",
        "STR_PART_RESIZE_FAILED":"Non-destructive installation failed, Check /var/log/ventoy.log for details.",
        "STR_PART_RESIZE_UNSUPPORTED":"Ventoy non-destructive installation stopped because some conditions cannot be met. Check /var/log/ventoy.log for details.",
        "STR_INSTALL_YES_TIP1":"Warning: Data will be lost!",
        "STR_INSTALL_YES_TIP2":"Please enter YES in the text box below to confirm that you indeed want to do a fresh install instead of upgrade.",
        "STR_PART_VENTOY_FS":"File System For Ventoy Partition",
        "STR_PART_FS":"File System",
        "STR_PART_CLUSTER":"Cluster Size",
        "STR_PART_CLUSTER_DEFAULT":"System Default Value",
        "STR_DONATE":"შემოწირულობა",
        "STR_4KN_UNSUPPORTED":"Currently Ventoy does not support 4K native devices.",
        
        "STRXXX":""
    },
    {
        "name":"Tamil (தமிழ்)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"K.B.Dharun Krishna",
        
        "STR_ERROR":"பிழை",
        "STR_WARNING":"எச்சரிக்கை",
        "STR_INFO":"தகவல்",
        "STR_INCORRECT_DIR":"சரியான கோப்பகத்தின் கீழ் இயக்கவும்!",
        "STR_INCORRECT_TREE_DIR":"என்னை இங்கே இயக்க வேண்டாம், வெளியிடப்பட்ட நிறுவல் தொகுப்பை பதிவிறக்கம் செய்து, அங்கு இயக்கவும்.",
        "STR_DEVICE":"சாதனம்",
        "STR_LOCAL_VER":"தொகுப்பில் வென்டோய்",
        "STR_DISK_VER":"சாதனத்தில் வென்டோய்",
        "STR_STATUS":"நிலை - தயார்",
        "STR_INSTALL":"நிறுவு",
        "STR_UPDATE":"புதுப்பிக்கவும்",
        "STR_UPDATE_TIP":"மேம்படுத்தல் செயல்பாடு பாதுகாப்பானது, ISO கோப்புகள் மாறாமல் இருக்கும்.#@தொடரவா?",
        "STR_INSTALL_TIP":"சாதனம் வடிவமைக்கப்படும் மற்றும் எல்லா தரவும் இழக்கப்படும்.#@தொடரவா?",
        "STR_INSTALL_TIP2":"சாதனம் வடிவமைக்கப்படும் மற்றும் எல்லா தரவும் இழக்கப்படும்.#@தொடரவா? (இருமுறை சரிபார்க்கவும்)",
        "STR_INSTALL_SUCCESS":"வாழ்த்துகள்!#@வென்டோய் சாதனத்தில் வெற்றிகரமாக நிறுவப்பட்டது.",
        "STR_INSTALL_FAILED":"நிறுவலின் போது பிழை ஏற்பட்டது. USB சாதனத்தை மீண்டும் செருகி மீண்டும் முயற்சிக்கவும். விவரங்களுக்கு /var/log/ventoy.log ஐப் பார்க்கவும். அது எப்போதும் தோல்வியுற்றால், அதிகாரப்பூர்வ இணையதளத்தில் அடிக்கடி கேட்கப்படும் கேள்விகளைப் பார்க்கவும்.",
        "STR_UPDATE_SUCCESS":"வாழ்த்துகள்!#@வென்டோய் சாதனத்தில் வெற்றிகரமாக புதுப்பிக்கப்பட்டது.",
        "STR_UPDATE_FAILED":"புதுப்பித்தலின் போது பிழை ஏற்பட்டது. USB சாதனத்தை மீண்டும் செருகி மீண்டும் முயற்சிக்கவும். விவரங்களுக்கு /var/log/ventoy.log ஐப் பார்க்கவும். அது எப்போதும் தோல்வியுற்றால், அதிகாரப்பூர்வ இணையதளத்தில் அடிக்கடி கேட்கப்படும் கேள்விகளைப் பார்க்கவும்.",
        "STR_WAIT_PROCESS":"ஒரு நூல் இயங்குகிறது, காத்திருக்கவும்...",
        "STR_MENU_OPTION":"விருப்பத்தேர்வுகள்",
        "STR_MENU_SECURE_BOOT":"பாதுகாப்பான பூட் ஆதரவு",
        "STR_MENU_PART_CFG":"பகிர்வு கட்டமைப்பு",
        "STR_BTN_OK":"சரி",
        "STR_BTN_CANCEL":"ரத்துசெய்",
        "STR_PRESERVE_SPACE":"வட்டின் முடிவில் சிறிது இடத்தை சேமிக்கவும்",
        "STR_SPACE_VAL_INVALID":"ஒதுக்கப்பட்ட இடத்திற்கான தவறான மதிப்பு",
        "STR_MENU_CLEAR":"தெளிவான வெண்டோய்",
        "STR_CLEAR_SUCCESS":"சாதனத்திலிருந்து வென்டோய் வெற்றிகரமாக அகற்றப்பட்டது.",
        "STR_CLEAR_FAILED":"வென்டோயை வட்டில் இருந்து அகற்றும்போது பிழை ஏற்பட்டது. USB சாதனத்தை மீண்டும் செருகி மீண்டும் முயற்சிக்கவும். விவரங்களுக்கு /var/log/ventoy.log ஐப் பார்க்கவும்.",
        "STR_MENU_PART_STYLE":"பகிர்வு நடை",
        "STR_DISK_2TB_MBR_ERROR":"2TB க்கும் அதிகமான சாதனங்களுக்கு GPTஐத் தேர்ந்தெடுக்கவும்",
        "STR_SHOW_ALL_DEV":"எல்லா சாதனங்களையும் காட்டு",
        "STR_PART_ALIGN_4KB":"பகிர்வுகளை 4KB உடன் சீரமைக்கவும்",
        "STR_WEB_COMMUNICATION_ERR":"தகவல் பிழை:",
        "STR_WEB_REMOTE_ABNORMAL":"தொடர்பு பிழை: தொலைநிலை அசாதாரணமானது",
        "STR_WEB_REQUEST_TIMEOUT":"தகவல்தொடர்பு பிழை: கோரிக்கை நேரம் முடிந்தது",
        "STR_WEB_SERVICE_UNAVAILABLE":"தொடர்பு பிழை: சேவை கிடைக்கவில்லை",
        "STR_WEB_TOKEN_MISMATCH":"டீமான் நிலை புதுப்பிக்கப்பட்டது, பிறகு முயற்சிக்கவும்.",
        "STR_WEB_SERVICE_BUSY":"சேவை பிஸியாக உள்ளது, பிறகு முயற்சிக்கவும்.",
        "STR_MENU_VTSI_CREATE":"VTSI கோப்பை உருவாக்கவும்",
        "STR_VTSI_CREATE_TIP":"இந்த முறை சாதனத்தில் எழுதப்படாது, ஆனால் VTSI கோப்பை மட்டும் உருவாக்குகிறது#@தொடரவா?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI கோப்பு வெற்றிகரமாக உருவாக்கப்பட்டது!#@ரூஃபஸ்(3.15+) ஐப் பயன்படுத்தி, வென்டோயின் நிறுவலை முடிக்க, சாதனத்தில் எழுதலாம்.",
        "STR_VTSI_CREATE_FAILED":"VTSI கோப்பு உருவாக்கப்படுவதில் தோல்வி.",
        "STR_MENU_PART_RESIZE":"அழிவில்லாத நிறுவல்",
        "STR_PART_RESIZE_TIP":"வென்டோய் முடிந்தால், அழிவில்லாத நிறுவலை முயற்சிக்கும். #@தொடரவா?",
        "STR_PART_RESIZE_SUCCESS":"வாழ்த்துகள்!#@வென்டோய் அழிவில்லாத நிறுவல் வெற்றிகரமாக முடிந்தது.",
        "STR_PART_RESIZE_FAILED":"அழிவில்லாத நிறுவல் தோல்வியடைந்தது, விவரங்களுக்கு /var/log/ventoy.log ஐப் பார்க்கவும்.",
        "STR_PART_RESIZE_UNSUPPORTED":"சில நிபந்தனைகளை பூர்த்தி செய்ய முடியாததால், வென்டோய் அழிவில்லாத நிறுவல் நிறுத்தப்பட்டது. விவரங்களுக்கு /var/log/ventoy.log ஐப் பார்க்கவும்.",
        "STR_INSTALL_YES_TIP1":"எச்சரிக்கை: தரவு தொலைந்து போகும்!",
        "STR_INSTALL_YES_TIP2":"மேம்படுத்துவதற்குப் பதிலாக புதிதாக நிறுவ விரும்புகிறீர்கள் என்பதை உறுதிப்படுத்த, கீழே உள்ள உரைப் பெட்டியில் ஆம் என உள்ளிடவும்.",
        "STR_PART_VENTOY_FS":"வென்டோய் பகிர்வுக்கான கோப்பு முறைமை",
        "STR_PART_FS":"கோப்பு முறைமை",
        "STR_PART_CLUSTER":"கொத்து அளவு",
        "STR_PART_CLUSTER_DEFAULT":"கணினி இயல்புநிலை மதிப்பு",
        "STR_DONATE":"தானம் செய்",
        "STR_4KN_UNSUPPORTED":"தற்போது, வெண்டோய் 4K சொந்த சாதனங்களை ஆதரிக்கவில்லை.",
        
        "STRXXX":""
    },
    {
        "name":"Belarussian (Беларуская)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author": "dmitrygribenchuk",

        "STR_ERROR":"Памылка",
        "STR_WARNING":"Папярэджанне",
        "STR_INFO":"Інфармацыя",
        "STR_INCORRECT_DIR":"Калі ласка, запусціце ў іншым каталогу!",
        "STR_INCORRECT_TREE_DIR":"Не запускайце мяне тут, калі ласка, загрузіце выпушчаны ўсталявальны пакет і запусціце яго ў іншым месцы.",
        "STR_DEVICE":"Прылада",
        "STR_LOCAL_VER":"Ventoy у пакеце",
        "STR_DISK_VER":"Ventoy на прыладзе",
        "STR_STATUS":"Статус - ГАТОВЫ",
        "STR_INSTALL":"Усталяваць",
        "STR_UPDATE":"Абнавіць",
        "STR_UPDATE_TIP":"Абнаўленне бяспечна, ISO-файлы застануцца без змен. #@Працягнуць?",
        "STR_INSTALL_TIP":"Дыск будзе адфарматаваны і ўсе дадзеныя будуць страчаныя. #@Працягнуць?",
        "STR_INSTALL_TIP2":"Дыск будзе адфарматаваны і ўсе дадзеныя будуць страчаныя. #@ Вы сапраўды жадаеце працягнуць?",
        "STR_INSTALL_SUCCESS":"Віншуем! #@ Ventoy быў паспяхова ўстаноўлены на прыладу.",
        "STR_INSTALL_FAILED":"Падчас усталёўкі Ventoy адбылася памылка. Перападлучыце прыладу і паспрабуйце зноў. Праверце файл /var/log/ventoy.log для атрымання падрабязнай інфармацыі.",
        "STR_UPDATE_SUCCESS":"Віншуем! #@ Ventoy быў паспяхова абноўлены на прыладзе.",
        "STR_UPDATE_FAILED":"Падчас абнаўлення Ventoy адбылася памылка. Перападлучыце прыладу і паспрабуйце зноў. Праверце файл /var/log/ventoy.log для атрымання падрабязнай інфармацыі.",
        "STR_WAIT_PROCESS":"Працэс запушчаны, калі ласка пачакайце...",
        "STR_MENU_OPTION":"Параметры",
        "STR_MENU_SECURE_BOOT":"Падтрымка Secure Boot",
        "STR_MENU_PART_CFG":"Дадатковы раздзел",
        "STR_BTN_OK":"ОК",
        "STR_BTN_CANCEL":"Адмена",
        "STR_PRESERVE_SPACE":"Стварыць дадатковы раздзел ў канцы дыска",
        "STR_SPACE_VAL_INVALID":"Няправільнае значэнне памеру раздзела",
        "STR_MENU_CLEAR":"Выдалiць Ventoy",
        "STR_CLEAR_SUCCESS":"Ventoy быў паспяхова выдалены з прылады.",
        "STR_CLEAR_FAILED":"Падчас выдалення Ventoy адбылася памылка. Перападлучыце прыладу і паспрабуйце зноў. Праверце файл /var/log/ventoy.log для атрымання падрабязнай інфармацыі.",
        "STR_MENU_PART_STYLE":"Стыль разметкі раздзелаў",
        "STR_DISK_2TB_MBR_ERROR":"Калі ласка, абярыце GPT для дыскаў больш за 2ТБ",
        "STR_SHOW_ALL_DEV":"Паказаць усе прылады",
        "STR_PART_ALIGN_4KB":"Выраўнаваць раздзелы з памерам 4КБ",
        "STR_WEB_COMMUNICATION_ERR":"Памылка сувязі:",
        "STR_WEB_REMOTE_ABNORMAL":"Памылка сувязі: Выдаленае злучэнне несапраўдна",
        "STR_WEB_REQUEST_TIMEOUT":"Памылка сувязі: Скончыўся час чакання запыту",
        "STR_WEB_SERVICE_UNAVAILABLE":"Памылка сувязі: Служба недаступная",
        "STR_WEB_TOKEN_MISMATCH":"Статус дэмана абноўлены. Паўтарыце спробу пазней.",
        "STR_WEB_SERVICE_BUSY":"Служба занятая, паспрабуйце пазней.",
        "STR_MENU_VTSI_CREATE":"Стварыць файл VTSI",
        "STR_VTSI_CREATE_TIP":"На гэты раз Ventoy не будзе ўсталяваны на прыладу, а будзе толькі створаны файл VTSI#@Працягнуць?",
        "STR_VTSI_CREATE_SUCCESS":"Файл VTSI створаны паспяхова! #@ Вы можаце выкарыстоўваць Rufus (3.15 +), каб запісаць яго на прыладу, для завяршэння ўстаноўкі Ventoy.",
        "STR_VTSI_CREATE_FAILED":"Памылка стварэння файла VTSI.",
        "STR_MENU_PART_RESIZE":"Неразбуральная ўстаноўка",
        "STR_PART_RESIZE_TIP":"Ventoy паспрабуе неразбуральную ўстаноўку, калі гэта магчыма. #@Працягнуць?",
        "STR_PART_RESIZE_SUCCESS":"Віншуем! #@ Неразбуральная ўстаноўка Ventoy паспяхова завершана.",
        "STR_PART_RESIZE_FAILED":"Не атрымалася выканаць неразбуральную ўстаноўку, праверце файл /var/log/ventoy.log для атрымання падрабязнай інфармацыі.",
        "STR_PART_RESIZE_UNSUPPORTED":"Неразбуральная ўстаноўка Ventoy спынена, паколькі некаторыя ўмовы не могуць быць выкананы. Праверце файл /var/log/ventoy.log для атрымання падрабязнай інфармацыі.",
        "STR_INSTALL_YES_TIP1":"Папярэджанне: Дадзеныя будуць страчаныя!",
        "STR_INSTALL_YES_TIP2":"Калі ласка, увядзіце YES у тэкставае поле ніжэй, каб пацвердзіць, што вы сапраўды хочаце выканаць новую ўстаноўку замест абнаўлення.",
        "STR_PART_VENTOY_FS":"Файлавая сістэма для Ventoy раздзела",
        "STR_PART_FS":"Файлавая сістэма",
        "STR_PART_CLUSTER":"Памер кластара",
        "STR_PART_CLUSTER_DEFAULT":"Сістэмнае значэнне па змаўчанні",
        "STR_DONATE":"Ахвяраваць",
        "STR_4KN_UNSUPPORTED":"Зараз Ventoy не падтрымлівае ўласныя прылады 4K.",
        
        "STRXXX":""
    },
    {
        "name":"Malayalam (മലയാളം)",
        "FontFamily":"Courier New",
        "FontSize":16,
        "Author":"SIDDHARTH K P",

        "STR_ERROR":"പിശക്",
        "STR_WARNING":"മുന്നറിയിപ്പ്",
        "STR_INFO":"വിവരം",
        "STR_INCORRECT_DIR":"ദയവായി ശരിയായ ഡയറക്ടറിയിൽ പ്രവർത്തിപ്പിക്കുക!",
        "STR_INCORRECT_TREE_DIR":"എന്നെ ഇവിടെ പ്രവർത്തിപ്പിക്കരുത്, ദയവായി ഇൻസ്റ്റാളേഷൻ പാക്കേജ് ഡൗൺലോഡ് ചെയ്ത് അവിടെ പ്രവർത്തിപ്പിക്കുക.",
        "STR_DEVICE":"ഉപകരണം",
        "STR_LOCAL_VER":"പാക്കേജിലെ വെൻ്റോയ്",
        "STR_DISK_VER":"ഉപകരണത്തിലെ വെൻ്റോയ്",
        "STR_STATUS":"നില - തയ്യാറാണ്",
        "STR_INSTALL":"ഇൻസ്റ്റാൾ ചെയ്യുക",
        "STR_UPDATE":"അപ്ഡേറ്റ് ചെയ്യുക",
        "STR_UPDATE_TIP":"അപ്‌ഡേറ്റ് പ്രവർത്തനം സുരക്ഷിതമാണ്, ISO ഫയലുകൾ മാറ്റമില്ലാതെ തുടരും.#@തുടരട്ടേ?",
        "STR_INSTALL_TIP":"ഡിസ്ക് ഫോർമാറ്റ് ചെയ്യപ്പെടും, എല്ലാ ഡാറ്റയും നഷ്‌ടപ്പെടും.#@തുടരട്ടേ?",
        "STR_INSTALL_TIP2":"ഡിസ്ക് ഫോർമാറ്റ് ചെയ്യപ്പെടും, എല്ലാ ഡാറ്റയും നഷ്‌ടപ്പെടും.#@തുടരട്ടേ? (ഒന്നുകൂടി ഉറപ്പിക്കുക)",
        "STR_INSTALL_SUCCESS":"അഭിനന്ദനങ്ങൾ!#@വെൻ്റോയ് ഉപകരണത്തിൽ വിജയകരമായി ഇൻസ്റ്റാൾ ചെയ്തു.",
        "STR_INSTALL_FAILED":"ഇൻസ്റ്റാളേഷൻ സമയത്ത് ഒരു പിശക് സംഭവിച്ചു. നിങ്ങൾക്ക് യുഎസ്ബി ഉപകരണം വീണ്ടും ഘടിപ്പിച്ച് വീണ്ടും ശ്രമിക്കാവുന്നതാണ്. വിവരങ്ങൾക്കായി /var/log/ventoy.log പരിശോധിക്കുക.",
        "STR_UPDATE_SUCCESS":"അഭിനന്ദനങ്ങൾ!#@വെൻ്റോയ് ഉപകരണത്തിൽ വിജയകരമായി അപ്ഡേറ്റ് ചെയ്തു.",
        "STR_UPDATE_FAILED":"അപ്‌ഡേറ്റ് ചെയ്യുമ്പോൾ ഒരു പിശക് സംഭവിച്ചു. നിങ്ങൾക്ക് യുഎസ്ബി ഉപകരണം വീണ്ടും ഘടിപ്പിച്ച് വീണ്ടും ശ്രമിക്കാവുന്നതാണ്. വിവരങ്ങൾക്കായി /var/log/ventoy.log പരിശോധിക്കുക.",
        "STR_WAIT_PROCESS":"ഒരു പ്രവർത്തനം പുരോഗമിക്കുന്നു, ദയവായി കാത്തിരിക്കുക...",
        "STR_MENU_OPTION":"ഓപ്ഷനുകൾ",
        "STR_MENU_SECURE_BOOT":"സുരക്ഷിത ബൂട്ട് പിന്തുണ",
        "STR_MENU_PART_CFG":"പാർട്ടീഷൻ കോൺഫിഗറേഷൻ",
        "STR_BTN_OK":"ശരി",
        "STR_BTN_CANCEL":"റദ്ദാക്കുക",
        "STR_PRESERVE_SPACE":"ഡിസ്കിൻ്റെ അവസാനം കുറച്ച് സ്ഥലം ഒഴിച്ചിടുക",
        "STR_SPACE_VAL_INVALID":"സംവരണം ചെയ്ത സ്ഥലത്തിന് അസാധുവായ മൂല്യം",
        "STR_MENU_CLEAR":"വെൻ്റോയ് നീക്കംചെയ്യുക",
        "STR_CLEAR_SUCCESS":"ഉപകരണത്തിൽ നിന്ന് വെൻ്റോയ് വിജയകരമായി നീക്കംചെയ്‌തു.",
        "STR_CLEAR_FAILED":"ഡിസ്കിൽ നിന്ന് വെൻ്റോയ് നീക്കം ചെയ്യുമ്പോൾ ഒരു പിശക് സംഭവിച്ചു. നിങ്ങൾക്ക് യുഎസ്ബി ഉപകരണം വീണ്ടും ഘടിപ്പിച്ച് വീണ്ടും ശ്രമിക്കാവുന്നതാണ്. വിവരങ്ങൾക്കായി /var/log/ventoy.log പരിശോധിക്കുക.",
        "STR_MENU_PART_STYLE":"പാർട്ടീഷൻ രീതി",
        "STR_DISK_2TB_MBR_ERROR":"2TB-യിൽ കൂടുതൽ വലുപ്പമുള്ള ഡിസ്കിനായി ദയവായി GPT തിരഞ്ഞെടുക്കുക",
        "STR_SHOW_ALL_DEV":"എല്ലാ ഉപകരണങ്ങളും കാണിക്കുക",
        "STR_PART_ALIGN_4KB":"പാർട്ടീഷനുകൾ 4KB ഉപയോഗിച്ച് വിന്യസിക്കുക",
        "STR_WEB_COMMUNICATION_ERR":"ആശയവിനിമയ പിശക്:",
        "STR_WEB_REMOTE_ABNORMAL":"ആശയവിനിമയ പിശക്: റിമോട്ട് തകരാറ്",
        "STR_WEB_REQUEST_TIMEOUT":"ആശയവിനിമയ പിശക്: അഭ്യർത്ഥനയുടെ സമയം കഴിഞ്ഞു",
        "STR_WEB_SERVICE_UNAVAILABLE":"ആശയവിനിമയ പിശക്: സേവനം ലഭ്യമല്ല",
        "STR_WEB_TOKEN_MISMATCH":"ഡെമൺ നില അപ്‌ഡേറ്റ് ചെയ്‌തു, ദയവായി പിന്നീട് വീണ്ടും ശ്രമിക്കുക.",
        "STR_WEB_SERVICE_BUSY":"സേവനം തിരക്കിലാണ്, ദയവായി പിന്നീട് വീണ്ടും ശ്രമിക്കുക.",
        "STR_MENU_VTSI_CREATE":"VTSI ഫയൽ സൃഷ്‌ടിക്കുക",
        "STR_VTSI_CREATE_TIP":"ഇത് ഇത്തവണ ഉപകരണത്തിലേക്ക് എഴുതുകയില്ല, ഒരു VTSI ഫയൽ മാത്രം സൃഷ്ടിക്കും#@തുടരട്ടേ?",
        "STR_VTSI_CREATE_SUCCESS":"VTSI ഫയൽ വിജയകരമായി സൃഷ്ടിച്ചു!#@വെൻ്റോയ് ഇൻസ്റ്റാളേഷൻ പൂർത്തിയാക്കുന്നതിന് നിങ്ങൾക്ക് Rufus(3.15+) ഉപയോഗിച്ച് ഇത് ഉപകരണത്തിലേക്ക് എഴുതാം.",
        "STR_VTSI_CREATE_FAILED":"VTSI ഫയൽ നിർമ്മാണം പരാജയപ്പെട്ടു.",
        "STR_MENU_PART_RESIZE":"ഡാറ്റാ നഷ്ടപ്പെടാതെയുള്ള ഇൻസ്റ്റാളേഷൻ",
        "STR_PART_RESIZE_TIP":"സാധ്യമെങ്കിൽ, ഡാറ്റാ നഷ്ടപ്പെടാതെയുള്ള ഇൻസ്റ്റാളേഷൻ വെൻ്റോയ് ശ്രമിക്കും. #@തുടരട്ടേ?",
        "STR_PART_RESIZE_SUCCESS":"അഭിനന്ദനങ്ങൾ!#@വെൻ്റോയ് ഡാറ്റാ നഷ്ടപ്പെടാതെയുള്ള ഇൻസ്റ്റാളേഷൻ വിജയകരമായി പൂർത്തിയാക്കി.",
        "STR_PART_RESIZE_FAILED":"ഡാറ്റാ നഷ്ടപ്പെടാതെയുള്ള ഇൻസ്റ്റാളേഷൻ പരാജയപ്പെട്ടു, വിശദാംശങ്ങൾക്കായി /var/log/ventoy.log പരിശോധിക്കുക.",
        "STR_PART_RESIZE_UNSUPPORTED":"ചില വ്യവസ്ഥകൾ പാലിക്കാൻ കഴിയാത്തതിനാൽ വെൻ്റോയ് ഡാറ്റാ നഷ്ടപ്പെടാതെയുള്ള ഇൻസ്റ്റാളേഷൻ നിർത്തി. വിശദാംശങ്ങൾക്കായി /var/log/ventoy.log പരിശോധിക്കുക.",
        "STR_INSTALL_YES_TIP1":"മുന്നറിയിപ്പ്: ഡാറ്റ നഷ്ടപ്പെടും!",
        "STR_INSTALL_YES_TIP2":"അപ്‌ഗ്രേഡിന് പകരം ഒരു പുതിയ ഇൻസ്റ്റാളേഷൻ നടത്താൻ നിങ്ങൾ ആഗ്രഹിക്കുന്നുവെന്ന് സ്ഥിരീകരിക്കുന്നതിന് ദയവായി താഴെയുള്ള ടെക്സ്റ്റ് ബോക്സിൽ YES എന്ന് നൽകുക.",
        "STR_PART_VENTOY_FS":"വെൻ്റോയ് പാർട്ടീഷനായുള്ള ഫയൽ സിസ്റ്റം",
        "STR_PART_FS":"ഫയൽ സിസ്റ്റം",
        "STR_PART_CLUSTER":"ക്ലസ്റ്റർ വലുപ്പം",
        "STR_PART_CLUSTER_DEFAULT":"സിസ്റ്റം ഡിഫോൾട്ട് മൂല്യം",
        "STR_DONATE":"സംഭാവന ചെയ്യുക",
        "STR_4KN_UNSUPPORTED":"നിലവിൽ വെൻ്റോയ് 4K നേറ്റീവ് ഉപകരണങ്ങളെ പിന്തുണയ്ക്കുന്നില്ല.",
    
        "STRXXX":""
    }
]
;
